@@ -145,8 +145,9 @@ Fp_red_FpX(GEN x, GEN p, long v)
   GEN z;
   if (!signe(x)) return pol_0(v);
   z = cgetg(3, t_POL);
-  z[1] = evalsigne(1) | evalvarn(v);
-  gel(z,2) = Fp_red(x,p); return z;
+  gel(z,2) = Fp_red(x,p);
+  z[1] = evalvarn(v);
+  return FpX_renormalize(z, 3);
 }
 
 static GEN
@@ -155,8 +156,9 @@ Fp_neg_FpX(GEN x, GEN p, long v)
   GEN z;
   if (!signe(x)) return pol_0(v);
   z = cgetg(3, t_POL);
-  z[1] = evalsigne(1) | evalvarn(v);
-  gel(z,2) = Fp_neg(x,p); return z;
+  gel(z,2) = Fp_neg(x,p);
+  z[1] = evalvarn(v);
+  return FpX_renormalize(z, 3);
 }
 
 GEN
@@ -1585,6 +1587,7 @@ Fp_FpXQ_log(GEN a, GEN g, GEN o, GEN T, GEN p)
     g = constant_term(g);
   }
   n_q = Fp_log(a,g,op,p);
+  if (lg(n_q)==1) return gerepileuptoleaf(av, n_q);
   if (q) n_q = mulii(q, n_q);
   return gerepileuptoint(av, n_q);
 }
@@ -1622,8 +1625,7 @@ static const struct bb_group FpXQ_star={_FpXQ_mul,_FpXQ_pow,_FpXQ_rand,hash_GEN,
 
 const struct bb_group *get_FpXQ_star(void **E, GEN T, GEN p)
 {
-  GEN z = new_chunk(sizeof(struct _FpXQ));
-  struct _FpXQ *e = (struct _FpXQ *) z;
+  struct _FpXQ *e = (struct _FpXQ *) stack_malloc(sizeof(struct _FpXQ));
   e->T = T; e->p  = p; e->aut =  FpXQ_pow(pol_x(get_FpX_var(T)), p, T, p);
   *E = (void*)e; return &FpXQ_star;
 }
