@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -39,7 +37,7 @@ BuildTree(GEN link, GEN V, GEN W, GEN a, GEN T, GEN p)
   long k = lg(a)-1;
   long i, j, s, minp, mind;
 
-  for (i=1; i<=k; i++) { V[i] = a[i]; link[i] = -i; }
+  for (i=1; i<=k; i++) { gel(V,i) = gel(a,i); link[i] = -i; }
 
   for (j=1; j <= 2*k-5; j+=2,i++)
   {
@@ -67,7 +65,7 @@ BuildTree(GEN link, GEN V, GEN W, GEN a, GEN T, GEN p)
   {
     GEN d, u, v;
     d = FqX_extgcd(gel(V,j), gel(V,j+1), T, p, &u, &v);
-    if (degpol(d) > 0) pari_err(talker, "relatively prime polynomials expected");
+    if (degpol(d) > 0) pari_err_COPRIME("BuildTree", gel(V,j), gel(V,j+1));
     d = gel(d,2);
     if (!gequal1(d))
     {
@@ -140,18 +138,20 @@ static void
 ZpXQ_HenselLift(GEN V, GEN W, long j, GEN f, GEN Td, GEN T1, GEN pd, GEN p0, GEN p1, int noinv)
 {
   pari_sp av = avma;
+  const long n = degpol(T1), vT = varn(T1);
   long space = lg(f) * lgefint(p1) * lg(T1);
   GEN a2, b2, g, z, s, t;
   GEN a = gel(V,j), b = gel(V,j+1);
   GEN u = gel(W,j), v = gel(W,j+1);
 
   (void)new_chunk(space); /* HACK */
-  g = RgX_sub(f, RgX_mul(a,b));
+  g = RgX_sub(f, Kronecker_to_ZXX(ZXX_mul_Kronecker(a,b,n), n, vT));
   g = FpXQX_red(g, T1, p1);
   g = RgX_Rg_divexact(g, p0);
   z = FpXQX_mul(v,g, Td,pd);
   t = FpXQX_divrem(z,a, Td,pd, &s);
-  t = RgX_add(RgX_mul(u,g), RgX_mul(t,b));
+  t = ZX_add(ZXX_mul_Kronecker(u,g,n), ZXX_mul_Kronecker(t,b,n));
+  t = Kronecker_to_ZXX(t, n, vT);
   t = FpXQX_red(t, Td, pd);
   t = RgX_Rg_mul(t,p0);
   s = RgX_Rg_mul(s,p0);
@@ -166,13 +166,15 @@ ZpXQ_HenselLift(GEN V, GEN W, long j, GEN f, GEN Td, GEN T1, GEN pd, GEN p0, GEN
 
   av = avma;
   (void)new_chunk(space); /* HACK */
-  g = RgX_add(RgX_mul(u,a2), RgX_mul(v,b2));
+  g = ZX_add(ZXX_mul_Kronecker(u,a2,n), ZXX_mul_Kronecker(v,b2,n));
+  g = Kronecker_to_ZXX(g, n, vT);
   g = Rg_RgX_sub(gen_1, g);
   g = FpXQX_red(g, T1, p1);
   g = RgX_Rg_divexact(g, p0);
   z = FpXQX_mul(v,g, Td,pd);
   t = FpXQX_divrem(z,a, Td,pd, &s);
-  t = RgX_add(RgX_mul(u,g), RgX_mul(t,b));
+  t = ZX_add(ZXX_mul_Kronecker(u,g,n), ZXX_mul_Kronecker(t,b,n));
+  t = Kronecker_to_ZXX(t, n, vT);
   t = FpXQX_red(t, Td, pd);
   t = RgX_Rg_mul(t,p0);
   s = RgX_Rg_mul(s,p0);
@@ -243,11 +245,12 @@ MultiLift(GEN f, GEN a, GEN T, GEN p, long e0, long flag)
   ulong mask;
   pari_timer Ti;
 
-  if (k < 2 || e0 < 1) pari_err(talker, "MultiLift: bad args");
+  if (k < 2) pari_err_DOMAIN("MultiLift", "#(modular factors)", "<", gen_2,a);
+  if (e0 < 1) pari_err_DOMAIN("MultiLift", "precision", "<", gen_1,stoi(e0));
   if (e0 == 1) return a;
 
   if (DEBUGLEVEL > 3) timer_start(&Ti);
-  if (typ(a[1]) == t_INT)
+  if (typ(gel(a,1)) == t_INT)
   { /* a = TreeLift structure */
     e = itos(gel(a,1));
     link = gel(a,2);
@@ -377,28 +380,26 @@ polhensellift(GEN pol, GEN L, GEN p, long N)
   long i, l, t;
   pari_sp av = avma;
 
-  if (typ(pol) != t_POL) pari_err(talker, "not a polynomial in polhensellift");
-  RgX_check_ZXY(pol, "polhensellift");
-  t = typ(L);
-  if (!is_vec_t(t) || lg(L) < 3)
-    pari_err(talker, "not a factorization in polhensellift");
+  if (typ(pol) != t_POL) pari_err_TYPE("polhensellift",pol);
+  RgX_check_ZXX(pol, "polhensellift");
+  if (!is_vec_t(typ(L)) || lg(L) < 3) pari_err_TYPE("polhensellift",L);
   t = typ(p);
   if (t == t_VEC) /* [p, T] */
   {
     T = gel(p,2);
-    if (typ(T) != t_POL) pari_err(talker, "incorrect T in polhensellift");
+    if (typ(T) != t_POL) pari_err_TYPE("polhensellift",pol);
     RgX_check_ZX(T, "polhensellift");
     p = gel(p,1); t = typ(p);
   }
-  if (t != t_INT) pari_err(talker, "incorrect p in polhensellift");
-  if (N < 1) pari_err(talker, "not a positive exponent in polhensellift");
+  if (t != t_INT) pari_err_TYPE("polhensellift",p);
+  if (N < 1) pari_err_DOMAIN("polhensellift", "precision", "<", gen_1,stoi(N));
 
   l = lg(L); L = leafcopy(L);
   for (i = 1; i < l; i++)
   {
     if (typ(gel(L,i)) != t_POL)
       gel(L,i) = scalar_ZX_shallow(gel(L,i), varn(pol));
-    RgX_check_ZXY(gel(L,i), "polhensellift");
+    RgX_check_ZXX(gel(L,i), "polhensellift");
   }
   return gerepilecopy(av, ZpX_liftfact(pol, L, T, p, N, powiu(p,N)));
 }
@@ -435,29 +436,51 @@ ZpX_liftroot(GEN f, GEN a, GEN p, long e)
     W = Fp_sub(shifti(W,1), Fp_mul(Fp_sqr(W,q), FpX_eval(ZX_deriv(fr),a,q), q), q);
   }
 }
-GEN
-ZpXQX_liftroot(GEN f, GEN a, GEN T, GEN p, long e)
-{
-  pari_sp av = avma;
-  GEN q = p, fr, W;
-  ulong mask;
 
-  a = Fq_red(a, T, q);
-  if (e == 1) return a;
-  mask = quadratic_prec_mask(e);
-  fr = FpXQX_red(f, T, q);
-  W = Fq_inv(FqX_eval(RgX_deriv(fr), a, T, q), T, q); /* 1/f'(a) mod (T,p) */
-  for(;;)
+GEN
+ZpXQX_liftroot_vald(GEN f, GEN a, long v, GEN T, GEN p, long e)
+{
+  pari_sp av = avma, av2, lim;
+  GEN pv = p, q, qv, W, df, Tq, fr, dfr;
+  ulong mask;
+  a = Fq_red(a, T, p);
+  if (e <= v+1) return a;
+  df = RgX_deriv(f);
+  if (v) { pv = powiu(p,v); qv = mulii(pv,p); df = ZXX_Z_divexact(df, pv); }
+  else qv = p;
+  mask = quadratic_prec_mask(e-v);
+  Tq = FpXT_red(T, qv); dfr = FpXQX_red(df, Tq, p);
+  W = Fq_inv(FqX_eval(dfr, a, Tq, p), Tq, p); /* 1/f'(a) mod (T,p) */
+  q = p;
+  av2 = avma; lim = stack_lim(av2, 2);
+  for (;;)
   {
-    q = sqri(q);
-    if (mask & 1) q = diviiexact(q, p);
+    GEN u, fa, qv, q2v, q2, Tq2;
+    q2 = q; q = sqri(q);
+    if (mask & 1) q = diviiexact(q,p);
     mask >>= 1;
-    fr = FpXQX_red(f,T,q);
-    a = Fq_sub(a, Fq_mul(W, FqX_eval(fr, a, T,q), T,q), T,q);
+    if (v) { qv = mulii(q, pv); q2v = mulii(q2, pv); }
+    else { qv = q; q2v = q2; }
+    Tq2 = FpXT_red(T, q2v); Tq = FpXT_red(T, qv);
+    fr = FpXQX_red(f, Tq, qv);
+    fa = FqX_eval(fr, a, Tq, qv);
+    fa = typ(fa)==t_INT? diviiexact(fa,q2v): ZX_Z_divexact(fa, q2v);
+    a = Fq_sub(a, ZX_Z_mul(Fq_mul(W, fa, Tq2, q2v), q2), Tq, qv);
     if (mask == 1) return gerepileupto(av, a);
-    W = Fq_sub(gmul2n(W,1), Fq_mul(Fq_sqr(W,T,q), FqX_eval(RgX_deriv(fr),a,T,q), T, q), T,q);
+    dfr = FpXQX_red(df, Tq, q);
+    u = ZX_Z_divexact(FpX_Fp_sub(Fq_mul(W,FqX_eval(dfr,a,Tq,q),Tq,q),gen_1,q),q2);
+    W = Fq_sub(W,ZX_Z_mul(Fq_mul(u,W,Tq2,q2),q2),Tq,q);
+    if (low_stack(lim, stack_lim(av2,2)))
+    {
+      if(DEBUGMEM>1) pari_warn(warnmem,"ZpXQX_liftroot, e = %ld", e);
+      gerepileall(av2, 3, &a, &W, &q);
+    }
   }
 }
+
+GEN
+ZpXQX_liftroot(GEN f, GEN a, GEN T, GEN p, long e) { return ZpXQX_liftroot_vald(f,a,0,T,p,e); }
+
 /* Apply ZpX_liftroot to all roots in S and trace trick.
  * Elements of S must be distinct simple roots mod p for all p|q. */
 GEN
@@ -565,28 +588,213 @@ Zp_sqrtnlift(GEN b, GEN n, GEN a, GEN p, long e)
   }
   return gerepileuptoint(ltop,a);
 }
-/* Same as ZpXQX_liftroot for the polynomial X^n-b */
-GEN
-ZpXQ_sqrtnlift(GEN b, GEN n, GEN a, GEN T, GEN p, long e)
+
+/* Compute (x-1)/(x+1)/p^k */
+static GEN
+ZpXQ_log_to_ath(GEN x, long k, GEN T, GEN p, long e, GEN pe)
 {
   pari_sp av = avma;
-  GEN q = p, n_1, w;
-  ulong mask;
-
-  a = Fq_red(a, T, q);
-  if (e == 1) return a;
-  n_1 = subis(n,1);
-  mask = quadratic_prec_mask(e);
-  w = Fq_inv(Fq_mul(n, Fq_pow(a, n_1, T,q), T,q), T,q);
-  for(;;)
+  long vT = get_FpX_var(T);
+  GEN bn, bdi;
+  GEN bd = ZX_Z_add(x, gen_1);
+  if (equaliu(p,2)) /*For p=2, we need to simplify by 2*/
   {
-    q = sqri(q);
-    if (mask & 1) q = diviiexact(q, p);
-    mask >>= 1;
-    /* a -= w (a^n - b) */
-    a = Fq_sub(a, Fq_mul(w, Fq_sub(Fq_pow(a, n, T,q), b, T,q), T,q), T,q);
-    if (mask == 1) return gerepileupto(av, a);
-    /* w += w - w^2 n a^(n-1)*/
-    w = Fq_sub(gmul2n(w,1), Fq_mul(Fq_sqr(w,T,q), Fq_mul(n, Fq_pow(a,n_1,T, q), T,q), T,q), T,q);
+    bn = ZX_shifti(x,-(k+1));
+    bdi= ZpXQ_invlift(ZX_shifti(bd ,-1), pol_1(vT), T, p, e);
   }
+  else
+  {
+    bn = ZX_Z_divexact(ZX_Z_sub(x, gen_1),powiu(p,k));
+    bdi= ZpXQ_invlift(bd, scalarpol(Fp_inv(gen_2,p),vT), T, p, e);
+  }
+  return gerepileupto(av, FpXQ_mul(bn, bdi, T, pe));
+}
+
+/* Assume p odd, a = 1 [p], return log(a) */
+GEN
+ZpXQ_log(GEN a, GEN T, GEN p, long N)
+{
+  pari_sp av = avma;
+  pari_timer ti;
+  long is2 = equaliu(p,2);
+  ulong pp = is2 ? 0: itou_or_0(p);
+  double lp = is2 ? 1: pp ? log2(pp): expi(p);
+  long k = maxss(1 , (long) .5+pow((double)(N>>1)/(lp*lp), 1./3));
+  GEN ak, s, b, pol;
+  long e = is2 ? N-1: N;
+  long i, l = (e-2)/(2*(k+is2));
+  GEN pe = powiu(p,e);
+  GEN TNk, pNk = powiu(p,N+k);
+  if( DEBUGLEVEL>=3) timer_start(&ti);
+  TNk = FpX_get_red(get_FpX_mod(T), pNk);
+  ak = FpXQ_pow(a, powiu(p,k), TNk, pNk);
+  if( DEBUGLEVEL>=3) timer_printf(&ti,"FpXQ_pow(%ld)",k);
+  b = ZpXQ_log_to_ath(ak, k, T, p, e, pe);
+  if( DEBUGLEVEL>=3) timer_printf(&ti,"ZpXQ_log_to_ath");
+  pol= cgetg(l+3,t_POL);
+  pol[1] = evalsigne(1)|evalvarn(0);
+  for(i=0; i<=l; i++)
+  {
+    GEN g;
+    ulong z = 2*i+1;
+    if (pp)
+    {
+      long w = u_lvalrem(z, pp, &z);
+      g = powuu(pp,2*i*k-w);
+    }
+    else g = powiu(p,2*i*k);
+    gel(pol,i+2) = Fp_div(g, utoi(z),pe);
+  }
+  if( DEBUGLEVEL>=3) timer_printf(&ti,"pol(%ld)",l);
+  s = FpX_FpXQ_eval(pol, FpXQ_sqr(b, T, pe), T,  pe);
+  if( DEBUGLEVEL>=3) timer_printf(&ti,"FpX_FpXQ_eval");
+  s = ZX_shifti(FpXQ_mul(b, s, T, pe), 1);
+  return gerepileupto(av, is2? s: FpX_red(s, pe));
+}
+
+/***********************************************************************/
+/**                                                                   **/
+/**                 Generic quadratic hensel lift over Zp[X]          **/
+/**                                                                   **/
+/***********************************************************************/
+/* q = p^N */
+
+GEN
+gen_ZpX_Dixon(GEN F, GEN V, GEN q, GEN p, long N, void *E,
+                            GEN lin(void *E, GEN F, GEN d, GEN q),
+                            GEN invl(void *E, GEN d))
+{
+  pari_sp av = avma;
+  long N2, M;
+  GEN VN2, V2, VM, bil;
+  GEN q2, qM;
+  V = FpX_red(V, q);
+  if (N == 1) return invl(E, V);
+  N2 = (N + 1)>>1; M = N - N2;
+  F = FpXT_red(F, q);
+  qM = powiu(p, M);
+  q2 = M == N2? qM: mulii(qM, p);
+  /* q2 = p^N2, qM = p^M, q = q2 * qM */
+  VN2 = gen_ZpX_Dixon(F, V, q2, p, N2, E, lin, invl);
+  bil = lin(E, F, VN2, q);
+  V2 = ZX_Z_divexact(ZX_sub(V, bil), q2);
+  VM = gen_ZpX_Dixon(F, V2, qM, p, M, E, lin, invl);
+  return gerepileupto(av, FpX_red(ZX_add(VN2, ZX_Z_mul(VM, q2)), q));
+}
+
+GEN
+gen_ZpX_Newton(GEN x, GEN p, long n, void *E,
+                      GEN eval(void *E, GEN f, GEN q),
+                      GEN invd(void *E, GEN V, GEN v, GEN q, long M))
+{
+  pari_sp ltop = avma, av, st_lim;
+  long N = 1, N2, M;
+  long mask;
+  GEN q = p;
+  if (n == 1) return gcopy(x);
+  mask = quadratic_prec_mask(n);
+  av = avma; st_lim = stack_lim(av, 1);
+  while (mask > 1)
+  {
+    GEN qM, q2, v, V;
+    N2 = N; N <<= 1;
+    q2 = q;
+    if (mask&1UL) { /* can never happen when q2 = p */
+      N--; M = N2-1;
+      qM = diviiexact(q2,p); /* > 1 */
+      q = mulii(qM,q2);
+    } else {
+      M = N2;
+      qM = q2;
+      q = sqri(q2);
+    }
+    /* q2 = p^N2, qM = p^M, q = p^N = q2 * qM */
+    mask >>= 1;
+    v = eval(E, x, q);
+    V = ZX_Z_divexact(gel(v,1), q2);
+    x = FpX_sub(x, ZX_Z_mul(invd(E, V, v, qM, M), q2), q);
+    if (low_stack(st_lim, stack_lim(av, 1)))
+    {
+      if(DEBUGMEM>1) pari_warn(warnmem,"gen_ZpX_Newton");
+      gerepileall(av, 2, &x, &q);
+    }
+  }
+  return gerepileupto(ltop, x);
+}
+
+struct _ZpXQ_inv
+{
+  GEN T, a, p ,n;
+};
+
+static GEN
+_inv_invd(void *E, GEN V, GEN v, GEN q, long M/*unused*/)
+{
+  struct _ZpXQ_inv *d = (struct _ZpXQ_inv *) E;
+  GEN Tq = FpXT_red(d->T, q);
+  (void)M;
+  return FpXQ_mul(V, gel(v,2), Tq, q);
+}
+
+static GEN
+_inv_eval(void *E, GEN x, GEN q)
+{
+  struct _ZpXQ_inv *d = (struct _ZpXQ_inv *) E;
+  GEN Tq = FpXT_red(d->T, q);
+  GEN f = FpX_Fp_sub(FpXQ_mul(x, FpX_red(d->a, q), Tq, q), gen_1, q);
+  return mkvec2(f, x);
+}
+
+GEN
+ZpXQ_invlift(GEN a, GEN x, GEN T, GEN p, long e)
+{
+  struct _ZpXQ_inv d;
+  d.a = a; d.T = T; d.p = p;
+  return gen_ZpX_Newton(x, p, e, &d, _inv_eval, _inv_invd);
+}
+
+GEN
+ZpXQ_inv(GEN a, GEN T, GEN p, long e)
+{
+  pari_sp av=avma;
+  GEN ai;
+  if (lgefint(p)==3)
+  {
+    ulong pp = p[2];
+    ai = Flx_to_ZX(Flxq_inv(ZX_to_Flx(a,pp), ZXT_to_FlxT(T, pp), pp));
+  } else
+    ai = FpXQ_inv(FpX_red(a,p), FpXT_red(T,p),p);
+  return gerepileupto(av, ZpXQ_invlift(a, ai, T, p, e));
+}
+
+struct _ZpXQ_sqrtn
+{
+  GEN T, a, n, ai;
+};
+
+static GEN
+_sqrtn_invd(void *E, GEN V, GEN v, GEN q, long M)
+{
+  struct _ZpXQ_sqrtn *d = (struct _ZpXQ_sqrtn *) E;
+  GEN Tq = FpX_red(d->T, q), aiq = FpX_red(d->ai, q);
+  (void)M;
+  return FpXQ_mul(FpXQ_mul(V, gel(v,2), Tq, q), aiq, Tq, q);
+}
+
+static GEN
+_sqrtn_eval(void *E, GEN x, GEN q)
+{
+  struct _ZpXQ_sqrtn *d = (struct _ZpXQ_sqrtn *) E;
+  GEN Tq = FpX_red(d->T, q);
+  GEN f = FpX_sub(FpXQ_pow(x, d->n, Tq, q), d->a, q);
+  return mkvec2(f, x);
+}
+
+GEN
+ZpXQ_sqrtnlift(GEN a, GEN n, GEN x, GEN T, GEN p, long e)
+{
+  struct _ZpXQ_sqrtn d;
+  d.a = a; d.T = T; d.n = n;
+  d.ai = ZpXQ_inv(ZX_Z_mul(a, n),T,p,(e+1)>>1);
+  return gen_ZpX_Newton(x, p, e, &d, _sqrtn_eval, _sqrtn_invd);
 }
