@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000-2005  The PARI group.
+/* Copyright (C) 2000-2005  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -23,70 +21,67 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 void
 check_quaddisc(GEN x, long *s, long *r, const char *f)
 {
-  if (typ(x) != t_INT) pari_err(arither1);
+  if (typ(x) != t_INT) pari_err_TYPE(f,x);
   *s = signe(x);
-  if (Z_issquare(x)) pari_err(talker,"square discriminant in %s", f);
+  if (Z_issquare(x)) pari_err_DOMAIN(f,"issquare(disc)","=", gen_1,x);
   *r = mod4(x); if (*s < 0 && *r) *r = 4 - *r;
-  if (*r > 1) pari_err(talker, "discriminant not congruent to 0,1 mod 4 in %s", f);
+  if (*r > 1) pari_err_DOMAIN(f,"disc % 4",">", gen_1,x);
 }
 void
 check_quaddisc_real(GEN x, long *r, const char *f)
 {
   long sx; check_quaddisc(x, &sx, r, f);
-  if (sx < 0) pari_err(talker, "negative discriminant in %s", f);
+  if (sx < 0) pari_err_DOMAIN(f, "disc","<",gen_0,x);
 }
 void
 check_quaddisc_imag(GEN x, long *r, const char *f)
 {
   long sx; check_quaddisc(x, &sx, r, f);
-  if (sx > 0) pari_err(talker, "positive discriminant in %s", f);
+  if (sx > 0) pari_err_DOMAIN(f, "disc",">",gen_0,x);
 }
 
-static GEN
-Zquadpoly(GEN x, long v)
+/* X^2 + b X + c is the canonical quadratic t_POL of discriminant D.
+ * Dodd is non-zero iff D is odd */
+static void
+quadpoly_bc(GEN D, long Dodd, GEN *b, GEN *c)
 {
-  long res, sx;
-  GEN y, p1;
-
-  check_quaddisc(x, &sx, &res, "quadpoly");
-  y = cgetg(5,t_POL);
-  y[1] = evalsigne(1) | evalvarn(v);
-
-  p1 = shifti(x,-2); togglesign(p1);
-  /* p1 = - floor(x/4) [ = -x/4 or (1-x)/4 ] */
-  if (!res) gel(y,3) = gen_0;
+  if (Dodd)
+  {
+    pari_sp av = avma;
+    *b = gen_m1;
+    *c = gerepileuptoint(av, shifti(subui(1,D), -2));
+  }
   else
   {
-    if (sx < 0) p1 = gerepileuptoint((pari_sp)y, addsi(1,p1));
-    gel(y,3) = gen_m1;
+    *b = gen_0;
+    *c = shifti(D,-2); togglesign(*c);
   }
-  gel(y,2) = p1;
+}
+/* X^2 - X - (D-1)/4 or X^2 - D/4 */
+GEN
+quadpoly(GEN D)
+{
+  long Dmod4, s;
+  GEN b, c, y = cgetg(5,t_POL);
+  check_quaddisc(D, &s, &Dmod4, "quadpoly");
+  y[1] = evalsigne(1) | evalvarn(0);
+  quadpoly_bc(D, Dmod4, &b,&c);
+  gel(y,2) = c;
+  gel(y,3) = b;
   gel(y,4) = gen_1; return y;
 }
 
 GEN
 quadpoly0(GEN x, long v)
 {
-  if (is_matvec_t(typ(x)))
-  {
-    long i, l;
-    GEN y = cgetg_copy(x, &l);
-    for (i=1; i<l; i++) gel(y,i) = quadpoly0(gel(x,i),v);
-    return y;
-  }
-  if (v < 0) v = 0;
-  return Zquadpoly(x,v);
+  GEN T = quadpoly(x);
+  if (v > 0) setvarn(T, v);
+  return T;
 }
 
-GEN
-quadpoly(GEN x) { return quadpoly0(x, -1); }
-
 GEN
 quadgen(GEN x)
-{
-  GEN y = cgetg(4,t_QUAD);
-  gel(y,1) = Zquadpoly(x,0); gel(y,2) = gen_0; gel(y,3) = gen_1; return y;
-}
+{ retmkquad(quadpoly(x), gen_0, gen_1); }
 
 /***********************************************************************/
 /**                                                                   **/
@@ -96,21 +91,14 @@ quadgen(GEN x)
 GEN
 qfi(GEN x, GEN y, GEN z)
 {
-  GEN t = cgetg(4,t_QFI);
-  if (signe(x) < 0) pari_err(impl,"negative definite t_QFI");
-  gel(t,1) = icopy(x);
-  gel(t,2) = icopy(y);
-  gel(t,3) = icopy(z); return t;
+  if (signe(x) < 0) pari_err_IMPL("negative definite t_QFI");
+  retmkqfi(icopy(x),icopy(y),icopy(z));
 }
 GEN
 qfr(GEN x, GEN y, GEN z, GEN d)
 {
-  GEN t = cgetg(5,t_QFR);
-  if (typ(d) != t_REAL) pari_err(talker,"Shanks distance must be a t_REAL in qfr");
-  gel(t,1) = icopy(x);
-  gel(t,2) = icopy(y);
-  gel(t,3) = icopy(z);
-  gel(t,4) = rcopy(d); return t;
+  if (typ(d) != t_REAL) pari_err_TYPE("qfr",d);
+  retmkqfr(icopy(x),icopy(y),icopy(z),rcopy(d));
 }
 
 GEN
@@ -118,11 +106,13 @@ Qfb0(GEN x, GEN y, GEN z, GEN d, long prec)
 {
   pari_sp av = avma;
   GEN D;
-  long s;
-  if (typ(x)!=t_INT || typ(y)!=t_INT || typ(z)!=t_INT) pari_err(typeer,"Qfb");
+  long s, r;
+  if (typ(x)!=t_INT) pari_err_TYPE("Qfb",x);
+  if (typ(y)!=t_INT) pari_err_TYPE("Qfb",y);
+  if (typ(z)!=t_INT) pari_err_TYPE("Qfb",z);
   D = qfb_disc3(x,y,z);
-  if (Z_issquare(D)) pari_err(talker,"square discriminant in Qfb");
-  s = signe(D); avma = av;
+  check_quaddisc(D, &s, &r, "Qfb");
+  avma = av;
   if (s < 0) return qfi(x, y, z);
 
   d = d? gtofp(d,prec): real_0(prec);
@@ -133,9 +123,9 @@ Qfb0(GEN x, GEN y, GEN z, GEN d, long prec)
 static void
 qfb_sqr(GEN z, GEN x)
 {
-  GEN c, d1, x2, y2, v1, v2, c3, m, p1, r;
+  GEN c, d1, x2, v1, v2, c3, m, p1, r;
 
-  d1 = bezout(gel(x,2),gel(x,1),&x2,&y2);
+  d1 = bezout(gel(x,2),gel(x,1),&x2, NULL); /* usually 1 */
   c = gel(x,3);
   m = mulii(c,x2);
   if (is_pm1(d1))
@@ -158,11 +148,10 @@ qfb_sqr(GEN z, GEN x)
 static void
 qfb_comp(GEN z, GEN x, GEN y)
 {
-  GEN s, n, c, d, y1, v1, v2, c3, m, p1, r;
+  GEN n, c, d, y1, v1, v2, c3, m, p1, r;
 
   if (x == y) { qfb_sqr(z,x); return; }
-  s = shifti(addii(gel(x,2),gel(y,2)), -1);
-  n = subii(gel(y,2),s);
+  n = shifti(subii(gel(y,2),gel(x,2)), -1);
   v1 = gel(x,1);
   v2 = gel(y,1);
   c  = gel(y,3);
@@ -171,6 +160,7 @@ qfb_comp(GEN z, GEN x, GEN y)
     m = mulii(y1,n);
   else
   {
+    GEN s = subii(gel(y,2), n);
     GEN x2, y2, d1 = bezout(s,d,&x2,&y2); /* x2 s + y2 (x1 v1 + y1 v2) = d1 */
     if (!is_pm1(d1))
     {
@@ -187,19 +177,18 @@ qfb_comp(GEN z, GEN x, GEN y)
   c3 = addii(c, mulii(r,addii(gel(y,2),p1)));
   gel(z,1) = mulii(v1,v2);
   gel(z,2) = addii(gel(y,2), shifti(p1,1));
-  gel(z,3) = dvmdii(c3,v1, &s);
-  if (signe(s)) pari_err(talker,"different discriminants in qfb_comp");
+  gel(z,3) = diviiexact(c3,v1);
 }
 
+static GEN redimag_av(pari_sp av, GEN q);
 static GEN
 qficomp0(GEN x, GEN y, int raw)
 {
   pari_sp av = avma;
   GEN z = cgetg(4,t_QFI);
-  if (absi_cmp(gel(x,1), gel(y,1)) > 0) swap(x, y);
   qfb_comp(z, x,y);
   if (raw) return gerepilecopy(av,z);
-  return gerepileupto(av, redimag(z));
+  return redimag_av(av, z);
 }
 static GEN
 qfrcomp0(GEN x, GEN y, int raw)
@@ -222,12 +211,12 @@ GEN
 qfbcompraw(GEN x, GEN y)
 {
   long tx = typ(x);
-  if (typ(y) != tx) pari_err(typeer,"composition");
+  if (typ(y) != tx) pari_err_TYPE2("*",x,y);
   switch(tx) {
     case t_QFI: return qficompraw(x,y);
     case t_QFR: return qfrcompraw(x,y);
   }
-  pari_err(typeer,"composition");
+  pari_err_TYPE("composition",x);
   return NULL; /* not reached */
 }
 
@@ -237,10 +226,10 @@ qfisqr0(GEN x, long raw)
   pari_sp av = avma;
   GEN z = cgetg(4,t_QFI);
 
-  if (typ(x)!=t_QFI) pari_err(typeer,"composition");
+  if (typ(x)!=t_QFI) pari_err_TYPE("composition",x);
   qfb_sqr(z,x);
   if (raw) return gerepilecopy(av,z);
-  return gerepileupto(av, redimag(z));
+  return redimag_av(av, z);
 }
 static GEN
 qfrsqr0(GEN x, long raw)
@@ -248,7 +237,7 @@ qfrsqr0(GEN x, long raw)
   pari_sp av = avma;
   GEN z = cgetg(5,t_QFR);
 
-  if (typ(x)!=t_QFR) pari_err(typeer,"composition");
+  if (typ(x)!=t_QFR) pari_err_TYPE("composition",x);
   qfb_sqr(z,x); gel(z,4) = shiftr(gel(x,4),1);
   if (raw) return gerepilecopy(av,z);
   return gerepileupto(av, redreal(z));
@@ -280,11 +269,8 @@ qfr_1_by_disc(GEN D, long prec)
 GEN
 qfr_1(GEN x)
 {
-  long prec;
-  if (typ(x) != t_QFR) pari_err(typeer,"qfr_1");
-  prec = precision(gel(x,4));
-  if (!prec) pari_err(talker,"not a t_REAL in 4th component of a t_QFR");
-  return qfr_1_by_disc(qfb_disc(x), prec);
+  if (typ(x) != t_QFR) pari_err_TYPE("qfr_1",x);
+  return qfr_1_by_disc(qfb_disc(x), precision(gel(x,4)));
 }
 
 static void
@@ -309,32 +295,23 @@ static GEN
 qfr3_1(struct qfr_data *S)
 {
   GEN y = cgetg(4, t_VEC);
-  qfr_1_fill(y, S);
-  return y;
+  qfr_1_fill(y, S); return y;
 }
 
+/* Assume D < 0 is the discriminant of a t_QFI */
 static GEN
 qfi_1_by_disc(GEN D)
 {
-  GEN y = cgetg(4,t_QFI);
-  long r;
-
-  check_quaddisc_imag(D, &r, "qfi_1_by_disc");
+  GEN b,c, y = cgetg(4,t_QFI);
+  quadpoly_bc(D, mod2(D), &b,&c);
   gel(y,1) = gen_1;
-  gel(y,2) = r? gen_1: gen_0;
-  /* upon return, y[3] = (1-D) / 4 or -D / 4, whichever is an integer */
-  gel(y,3) = shifti(D,-2);
-  if (r) {
-    pari_sp av = avma;
-    gel(y,3) = gerepileuptoint(av, addis(gel(y,3),-1));
-  }
-  /* at this point y[3] < 0 */
-  setsigne(y[3], 1); return y;
+  gel(y,2) = b;
+  gel(y,3) = c; return y;
 }
 GEN
 qfi_1(GEN x)
 {
-  if (typ(x) != t_QFI) pari_err(typeer,"qfi_1");
+  if (typ(x) != t_QFI) pari_err_TYPE("qfi_1",x);
   return qfi_1_by_disc(qfb_disc(x));
 }
 
@@ -352,7 +329,7 @@ qfrpowraw(GEN x, long n)
   long m;
   GEN y;
 
-  if (typ(x) != t_QFR) pari_err(talker,"not a t_QFR in qfrpowraw");
+  if (typ(x) != t_QFR) pari_err_TYPE("qfrpowraw",x);
   if (!n) return qfr_1(x);
   if (n== 1) return gcopy(x);
   if (n==-1) return invraw(x);
@@ -374,7 +351,7 @@ qfipowraw(GEN x, long n)
   long m;
   GEN y;
 
-  if (typ(x) != t_QFI) pari_err(talker,"not a t_QFI in qfipow");
+  if (typ(x) != t_QFI) pari_err_TYPE("qfipow",x);
   if (!n) return qfi_1(x);
   if (n== 1) return gcopy(x);
   if (n==-1) return invraw(x);
@@ -414,10 +391,11 @@ nucomp(GEN x, GEN y, GEN L)
 {
   pari_sp av = avma;
   long z;
-  GEN a, a1, a2, b2, b, d, d1, g, n, p1, q1, q2, s, u, u1, v, v1, v2, v3, Q;
+  GEN a, a1, a2, b2, b, d, d1, g, n, p1, q1, q2, s, u, u1, v, v2, v3, Q;
 
   if (x==y) return nudupl(x,L);
-  if (typ(x) != t_QFI || typ(y) != t_QFI) pari_err(talker,"not a t_QFI in nucomp");
+  if (typ(x) != t_QFI) pari_err_TYPE("nucomp",x);
+  if (typ(y) != t_QFI) pari_err_TYPE("nucomp",y);
 
   if (absi_cmp(gel(x,1),gel(y,1)) < 0) swap(x, y);
   s = shifti(addii(gel(x,2),gel(y,2)), -1);
@@ -436,7 +414,7 @@ nucomp(GEN x, GEN y, GEN L)
     else
     {
       GEN p2, l;
-      d1 = bezout(s,d,&u1,&v1);
+      d1 = bezout(s,d,&u1,NULL);
       if (!is_pm1(d1))
       {
         a1 = diviiexact(a1,d1);
@@ -474,7 +452,7 @@ nucomp(GEN x, GEN y, GEN L)
   q2 = addii(q1,n);
   gel(Q,2) = addii(b2, z? addii(q1,q2): shifti(q1, 1));
   gel(Q,3) = addii(mulii(v3,diviiexact(q2,d)), mulii(g,v2));
-  return gerepileupto(av, redimag(Q));
+  return redimag_av(av, Q);
 }
 
 GEN
@@ -484,10 +462,10 @@ nudupl(GEN x, GEN L)
   long z;
   GEN u, v, d, d1, p1, a, b, c, a2, b2, c2, Q, v2, v3, g;
 
-  if (typ(x) != t_QFI) pari_err(talker,"not a t_QFI in nudupl");
+  if (typ(x) != t_QFI) pari_err_TYPE("nudupl",x);
   a = gel(x,1);
   b = gel(x,2);
-  d1 = bezout(b,a, &u,&v);
+  d1 = bezout(b,a, &u,NULL);
   if (!is_pm1(d1))
   {
     a = diviiexact(a, d1);
@@ -515,7 +493,7 @@ nudupl(GEN x, GEN L)
   }
   gel(Q,2) = addii(b2, subii(sqri(addii(d,v3)), addii(a2,c2)));
   gel(Q,3) = addii(c2, mulii(g,v2));
-  return gerepileupto(av, redimag(Q));
+  return redimag_av(av, Q);
 }
 
 static GEN
@@ -526,15 +504,16 @@ GEN
 nupow(GEN x, GEN n)
 {
   pari_sp av;
-  GEN y, l;
+  GEN y, D;
 
-  if (typ(n) != t_INT) pari_err(talker,"not an integer exponent in nupow");
+  if (typ(n) != t_INT) pari_err_TYPE("nupow",n);
+  if (typ(x) != t_QFI) pari_err_TYPE("nupow",x);
   if (gequal1(n)) return gcopy(x);
-  av = avma; y = qfi_1(x);
+  av = avma;
+  D = qfb_disc(x);
+  y = qfi_1_by_disc(D);
   if (!signe(n)) return y;
-
-  l = sqrti(shifti(sqrti(gel(y,3)),1));
-  y = gen_pow(x, n, (void*)l, &mul_nudupl, &mul_nucomp);
+  y = gen_pow(x, n, (void*)sqrtnint(absi(D), 4), &mul_nudupl, &mul_nucomp);
   if (signe(n) < 0
   && !absi_equal(gel(y,1),gel(y,2))
   && !absi_equal(gel(y,1),gel(y,3))) togglesign(gel(y,2));
@@ -543,6 +522,7 @@ nupow(GEN x, GEN n)
 
 /* Reduction */
 
+/* assume a > 0. Write b = q*2a + r, with -a < r <= a */
 static GEN
 dvmdii_round(GEN b, GEN a, GEN *r)
 {
@@ -554,14 +534,71 @@ dvmdii_round(GEN b, GEN a, GEN *r)
   }
   return q;
 }
+/* Assume 0 < a <= LONG_MAX. Ensure no overflow */
+static long
+dvmdsu_round(long b, ulong a, long *r)
+{
+  ulong a2 = a << 1, q, ub, ur;
+  if (b >= 0) {
+    ub = b;
+    q = ub / a2;
+    ur = ub % a2;
+    if (ur > a) { ur -= a; q++; *r = (long)ur; *r -= (long)a; }
+    else *r = (long)ur;
+    return (long)q;
+  } else { /* r <= 0 */
+    ub = (ulong)-b; /* |b| */
+    q = ub / a2;
+    ur = ub % a2;
+    if (ur >= a) { ur -= a; q++; *r = (long)ur; *r = (long)a - *r; }
+    else *r = -(long)ur;
+    return -(long)q;
+  }
+}
 /* reduce b mod 2*a. Update b,c */
 static void
 REDB(GEN a, GEN *b, GEN *c)
 {
   GEN r, q = dvmdii_round(*b, a, &r);
+  if (!signe(q)) return;
   *c = subii(*c, mulii(q, shifti(addii(*b, r),-1)));
   *b = r;
 }
+/* Assume a > 0. Reduce b mod 2*a. Update b,c */
+static void
+sREDB(ulong a, long *b, ulong *c)
+{
+  long r, q;
+  ulong uz;
+  if (a > LONG_MAX) return; /* b already reduced */
+  q = dvmdsu_round(*b, a, &r);
+  if (q == 0) return;
+  /* Final (a,r,c2) satisfies |r| <= |b| hence c2 <= c, c2 = c - q*z,
+   * where z = (b+r) / 2, representable as long, has the same sign as q. */
+  if (*b < 0)
+  { /* uz = -z >= 0, q < 0 */
+    if (r >= 0) /* different signs=>no overflow, exact division */
+      uz = (ulong)-((*b + r)>>1);
+    else
+    {
+      ulong ub = (ulong)-*b, ur = (ulong)-r;
+      uz = (ub + ur) >> 1;
+    }
+    *c -= (-q) * uz; /* c -= qz */
+  }
+  else
+  { /* uz = z >= 0, q > 0 */
+    if (r <= 0)
+      uz = (*b + r)>>1;
+    else
+    {
+      ulong ub = (ulong)*b, ur = (ulong)r;
+      uz = ((ub + ur) >> 1);
+    }
+    *c -= q * uz; /* c -= qz */
+  }
+  *b = r;
+}
 static void
 REDBU(GEN a, GEN *b, GEN *c, GEN u1, GEN *u2)
 { /* REDB(a,b,c) */
@@ -626,16 +663,73 @@ redimagsl2(GEN q, GEN *U)
   *U = mkmat2(mkcol2(u1,v1), mkcol2(u2,v2)); return Q;
 }
 
-GEN
-redimag(GEN q)
+static GEN
+setq_b0(ulong a, ulong c)
+{ retmkqfi( utoipos(a), gen_0, utoipos(c) ); }
+/* assume |sb| = 1 */
+static GEN
+setq(ulong a, ulong b, ulong c, long sb)
+{ retmkqfi( utoipos(a), sb == 1? utoipos(b): utoineg(b), utoipos(c) ); }
+/* 0 < a, c < 2^BIL, b = 0 */
+static GEN
+redimag_1_b0(ulong a, ulong c)
+{ return (a <= c)? setq_b0(a, c): setq_b0(c, a); }
+
+/* 0 < a, c < 2^BIL: single word affair */
+static GEN
+redimag_1(pari_sp av, GEN a, GEN b, GEN c)
+{
+  ulong ua, ub, uc;
+  long sb;
+  for(;;)
+  { /* at most twice */
+    long lb = lgefint(b); /* <= 3 after first loop */
+    if (lb == 2) return redimag_1_b0(a[2],c[2]);
+    if (lb == 3 && (ulong)b[2] <= (ulong)LONG_MAX) break;
+    REDB(a,&b,&c);
+    if ((ulong)a[2] <= (ulong)c[2])
+    { /* lg(b) <= 3 but may be too large for itos */
+      long s = signe(b);
+      avma = av;
+      if (!s) return redimag_1_b0(a[2], c[2]);
+      if (a[2] == c[2]) s = 1;
+      return setq(a[2], b[2], c[2], s);
+    }
+    swap(a,c); b = negi(b);
+  }
+  /* b != 0 */
+  avma = av;
+  ua = a[2];
+  ub = sb = b[2]; if (signe(b) < 0) sb = -sb;
+  uc = c[2];
+  if (ua < ub)
+    sREDB(ua, &sb, &uc);
+  else if (ua == ub && sb < 0) sb = (long)ub;
+  while(ua > uc)
+  {
+    lswap(ua,uc); sb = -sb;
+    sREDB(ua, &sb, &uc);
+  }
+  if (!sb) return setq_b0(ua, uc);
+  else
+  {
+    long s = 1;
+    if (sb < 0)
+    {
+      sb = -sb;
+      if (ua != uc) s = -1;
+    }
+    return setq(ua, sb, uc, s);
+  }
+}
+
+static GEN
+redimag_av(pari_sp av, GEN q)
 {
-  GEN Q = cgetg(4, t_QFI);
-  pari_sp av = avma, av2, lim = stack_lim(av, 1);
   GEN a = gel(q,1), b = gel(q,2), c = gel(q,3);
-  long cmp;
-  /* upper bound for size of final (a,b,c) */
-  (void)new_chunk(lgefint(a) + lgefint(b) + lgefint(c) + 3);
-  av2 = avma;
+  long cmp, lc = lgefint(c);
+
+  if (lgefint(a) == 3 && lc == 3) return redimag_1(av, a, b, c);
   cmp = absi_cmp(a, b);
   if (cmp < 0)
     REDB(a,&b,&c);
@@ -644,20 +738,20 @@ redimag(GEN q)
   for(;;)
   {
     cmp = absi_cmp(a, c); if (cmp <= 0) break;
+    lc = lgefint(a); /* lg(future c): we swap a & c next */
+    if (lc == 3) return redimag_1(av, a, b, c);
     swap(a,c); b = negi(b); /* apply rho */
     REDB(a,&b,&c);
-    if (low_stack(lim, stack_lim(av, 1))) {
-      if (DEBUGMEM>1) pari_warn(warnmem, "redimag");
-      gerepileall(av2, 3, &a,&b,&c);
-    }
   }
   if (cmp == 0 && signe(b) < 0) b = negi(b);
-
+  /* size of reduced Qfb(a,b,c) <= 3 lg(c) + 4 <= 4 lg(c) */
+  (void)new_chunk(lc<<2);
+  a = icopy(a); b = icopy(b); c = icopy(c);
   avma = av;
-  gel(Q,1) = icopy(a);
-  gel(Q,2) = icopy(b);
-  gel(Q,3) = icopy(c); return Q;
+  retmkqfi(icopy(a), icopy(b), icopy(c));
 }
+GEN
+redimag(GEN q) { return redimag_av(avma, q); }
 
 static GEN
 rhoimag(GEN x)
@@ -668,7 +762,7 @@ rhoimag(GEN x)
     int fg = absi_cmp(a, b);
     if (fg >= 0) {
       x = qfi(a,b,c);
-      if ((!fl || !fg) && signe(x[2]) < 0) setsigne(x[2], 1);
+      if ((!fl || !fg) && signe(gel(x,2)) < 0) setsigne(gel(x,2), 1);
       return x;
     }
   }
@@ -700,10 +794,9 @@ rhoimag(GEN x)
 static void
 fix_expo(GEN x)
 {
-  long d = expo(x[5]) - (1L << EMAX);
-  if (d >= 0) {
+  if (expo(gel(x,5)) >= (1L << EMAX)) {
     gel(x,4) = addsi(1, gel(x,4));
-    setexpo(x[5], d);
+    shiftr_inplace(gel(x, 5), - (1L << EMAX));
   }
 }
 
@@ -714,16 +807,16 @@ qfr5_dist(GEN e, GEN d, long prec)
   GEN t = logr_abs(d);
   if (signe(e)) {
     GEN u = mulir(e, mplog2(prec));
-    setexpo(u, expo(u+EMAX)); t = addrr(t, u);
+    shiftr_inplace(u, EMAX); t = addrr(t, u);
   }
-  setexpo(t, expo(t)-1); return t;
+  shiftr_inplace(t, -1); return t;
 }
 
 static void
 rho_get_BC(GEN *B, GEN *C, GEN b, GEN c, struct qfr_data *S)
 {
   GEN t, u;
-  u = shifti(c,1); if (u == gen_0) pari_err(talker, "reducible form in qfr_rho");
+  u = shifti(c,1);
   t = (absi_cmp(S->isqrtD,c) >= 0)? S->isqrtD: c;
   u = remii(addii_sign(t,1, b,signe(b)), u);
   *B = addii_sign(t, 1, u, -signe(u)); /* |t| - (|t|+b) % |2c| */
@@ -737,13 +830,9 @@ rho_get_BC(GEN *B, GEN *C, GEN b, GEN c, struct qfr_data *S)
 GEN
 qfr3_rho(GEN x, struct qfr_data *S)
 {
-  GEN B, C, y, b = gel(x,2), c = gel(x,3);
-
+  GEN B, C, b = gel(x,2), c = gel(x,3);
   rho_get_BC(&B, &C, b, c, S);
-  y = cgetg(4, t_VEC);
-  gel(y,1) = c;
-  gel(y,2) = B;
-  gel(y,3) = C; return y;
+  return mkvec3(c,B,C);
 }
 /* Not stack-clean */
 GEN
@@ -784,13 +873,13 @@ qfr5_to_qfr(GEN x, GEN d0)
       n = addis(shifti(n, EMAX), expo(d));
       setexpo(d, 0); d = logr_abs(d);
       if (signe(n)) d = addrr(d, mulir(n, mplog2(lg(d0))));
-      setexpo(d, expo(d)-1);
+      shiftr_inplace(d, -1);
       d0 = addrr(d0, d);
     }
     else if (!gequal1(d)) /* avoid loss of precision */
     {
       d = logr_abs(d);
-      setexpo(d, expo(d)-1);
+      shiftr_inplace(d, -1);
       d0 = addrr(d0, d);
     }
   }
@@ -847,8 +936,8 @@ static void
 get_disc(GEN x, struct qfr_data *S)
 {
   if (!S->D) S->D = qfb_disc(x);
-  else if (typ(S->D) != t_INT) pari_err(arither1);
-  if (!signe(S->D)) pari_err(talker,"reducible form in qfr_init");
+  else if (typ(S->D) != t_INT) pari_err_TYPE("qfr_init",S->D);
+  if (!signe(S->D)) pari_err_DOMAIN("qfr_init", "disc", "=", gen_0,x);
 }
 
 void
@@ -863,14 +952,14 @@ static GEN
 qfr5_init(GEN x, struct qfr_data *S)
 {
   GEN d = gel(x,4);
-  long prec = lg(d), l = nbits2prec(-expo(d));
+  long prec = realprec(d), l = nbits2prec(-expo(d));
   if (l > prec) prec = l;
-  if (prec < 3) prec = 3;
+  if (prec < LOWDEFAULTPREC) prec = LOWDEFAULTPREC;
   x = qfr_to_qfr5(x,prec);
 
   get_disc(x, S);
   if (!S->sqrtD) S->sqrtD = sqrtr(itor(S->D,prec));
-  else if (typ(S->sqrtD) != t_REAL) pari_err(arither1);
+  else if (typ(S->sqrtD) != t_REAL) pari_err_TYPE("qfr_init",S->sqrtD);
 
   if (!S->isqrtD)
   {
@@ -879,7 +968,7 @@ qfr5_init(GEN x, struct qfr_data *S)
     S->isqrtD = gcvtoi(S->sqrtD,&e);
     if (e>-2) { avma = av; S->isqrtD = sqrti(S->D); }
   }
-  else if (typ(S->isqrtD) != t_INT) pari_err(arither1);
+  else if (typ(S->isqrtD) != t_INT) pari_err_TYPE("qfr_init",S->isqrtD);
   return x;
 }
 static GEN
@@ -887,7 +976,7 @@ qfr3_init(GEN x, struct qfr_data *S)
 {
   get_disc(x, S);
   if (!S->isqrtD) S->isqrtD = sqrti(S->D);
-  else if (typ(S->isqrtD) != t_INT) pari_err(arither1);
+  else if (typ(S->isqrtD) != t_INT) pari_err_TYPE("qfr_init",S->isqrtD);
   return x;
 }
 
@@ -900,7 +989,7 @@ redreal0(GEN x, long flag, GEN D, GEN isqrtD, GEN sqrtD)
   pari_sp av = avma;
   struct qfr_data S;
   GEN d;
-  if (typ(x) != t_QFR) pari_err(talker,"not a real quadratic form in redreal");
+  if (typ(x) != t_QFR) pari_err_TYPE("redreal",x);
   d = gel(x,4);
   S.D = D;
   S.sqrtD = sqrtD;
@@ -911,7 +1000,7 @@ redreal0(GEN x, long flag, GEN D, GEN isqrtD, GEN sqrtD)
     case qf_NOD:         x = qfr3_red(x,&S); break;
     case qf_STEP:        x = qfr5_rho(x,&S); break;
     case qf_STEP|qf_NOD: x = qfr3_rho(x,&S); break;
-    default: pari_err(flagerr,"qfbred");
+    default: pari_err_FLAG("qfbred");
   }
   return gerepilecopy(av, qfr5_to_qfr(x,d));
 }
@@ -1047,17 +1136,19 @@ primeform_u(GEN x, ulong p)
 
   s = mod8(x); if (signe(x) < 0 && s) s = 8-s;
   /* 2 or 3 mod 4 */
-  if (s & 2) pari_err(talker,"discriminant not congruent to 0,1 mod 4 in primeform");
+  if (s & 2) pari_err_DOMAIN("primeform", "disc % 4", ">",gen_1, x);
   if (p == 2) {
     switch(s) {
       case 0: b = 0; break;
       case 1: b = 1; break;
       case 4: b = 2; break;
-      default: pari_err(sqrter5); b = 0; /* -Wall */
+      default: pari_err_SQRTN("primeform", mkintmod(x,utoi(p)) );
+               b = 0; /* -Wall */
     }
     c = shifti(subsi(s,x), -3);
   } else {
-    b = Fl_sqrt(umodiu(x,p), p); if (b == ~0UL) pari_err(sqrter5);
+    b = Fl_sqrt(umodiu(x,p), p);
+    if (b == ~0UL) pari_err_SQRTN("primeform", mkintmod(x,utoi(p)) );
     /* mod(b) != mod2(x) ? */
     if ((b ^ s) & 1) b = p - b;
     c = diviuexact(shifti(subii(sqru(b), x), -2), p);
@@ -1071,26 +1162,33 @@ primeform_u(GEN x, ulong p)
 GEN
 primeform(GEN x, GEN p, long prec)
 {
+  const char *f = "primeform";
   pari_sp av;
   long s, sx = signe(x), sp = signe(p);
   GEN y, b, absp;
 
-  if (typ(x) != t_INT || !sx) pari_err(arither1);
-  if (typ(p) != t_INT || !sp) pari_err(arither1);
+  if (typ(x) != t_INT) pari_err_TYPE(f,x);
+  if (typ(p) != t_INT) pari_err_TYPE(f,p);
+  if (!sp) pari_err_DOMAIN(f,"p","=",gen_0,p);
+  if (!sx) pari_err_DOMAIN(f,"D","=",gen_0,x);
   if (lgefint(p) == 3)
   {
-    if (p[2] == 1) {
+    ulong pp = p[2];
+    if (pp == 1) {
       if (sx < 0) {
-        if (sp < 0) pari_err(impl,"negative definite t_QFI");
+        long r;
+        if (sp < 0) pari_err_IMPL("negative definite t_QFI");
+        r = mod4(x);
+        if (r && r != 3) pari_err_DOMAIN(f,"disc % 4",">", gen_1,x);
         return qfi_1_by_disc(x);
       }
       y = qfr_1_by_disc(x,prec);
       if (sp < 0) { gel(y,1) = gen_m1; togglesign(gel(y,3)); }
       return y;
     }
-    y = primeform_u(x, p[2]);
+    y = primeform_u(x, pp);
     if (sx < 0) {
-      if (sp < 0) pari_err(impl,"negative definite t_QFI");
+      if (sp < 0) pari_err_IMPL("negative definite t_QFI");
       return y;
     }
     if (sp < 0) { togglesign(gel(y,1)); togglesign(gel(y,3)); }
@@ -1099,7 +1197,7 @@ primeform(GEN x, GEN p, long prec)
   s = mod8(x);
   if (sx < 0)
   {
-    if (sp < 0) pari_err(impl,"negative definite t_QFI");
+    if (sp < 0) pari_err_IMPL("negative definite t_QFI");
     if (s) s = 8-s;
     y = cgetg(4, t_QFI);
   }
@@ -1109,9 +1207,9 @@ primeform(GEN x, GEN p, long prec)
     gel(y,4) = real_0(prec);
   }
   /* 2 or 3 mod 4 */
-  if (s & 2) pari_err(talker,"discriminant not congruent to 0,1 mod 4 in primeform");
+  if (s & 2) pari_err_DOMAIN(f, "disc % 4", ">",gen_1, x);
   absp = absi(p); av = avma;
-  b = Fp_sqrt(x, absp); if (!b) pari_err(sqrter5);
+  b = Fp_sqrt(x, absp); if (!b) pari_err_SQRTN(f, mkintmod(x,absp));
   s &= 1; /* s = x mod 2 */
   /* mod(b) != mod2(x) ? [Warning: we may have b == 0] */
   if ((!signe(b) && s) || mod2(b) != s) b = gerepileuptoint(av, subii(absp,b));
@@ -1123,38 +1221,26 @@ primeform(GEN x, GEN p, long prec)
   return y;
 }
 
-/* Let M and N in SL_2(Z), return (N*M^-1)[,1]
- *
-*/
+/* Let M and N in SL2(Z), return (N*M^-1)[,1] */
 static GEN
 SL2_div_mul_e1(GEN N, GEN M)
 {
-  GEN b = gcoeff(M, 2, 1);
-  GEN d = gcoeff(M, 2, 2);
-  GEN p2 = cgetg(3, t_VEC);
-  gel(p2,1) = subii(mulii(gcoeff(N, 1, 1), d),
-                    mulii(gcoeff(N, 1, 2), b));
-  gel(p2,2) = subii(mulii(gcoeff(N, 2, 1), d),
-                    mulii(gcoeff(N, 2, 2), b));
-  return p2;
+  GEN b = gcoeff(M,2,1), d = gcoeff(M,2,2);
+  GEN p = subii(mulii(gcoeff(N,1,1), d), mulii(gcoeff(N,1,2), b));
+  GEN q = subii(mulii(gcoeff(N,2,1), d), mulii(gcoeff(N,2,2), b));
+  return mkvec2(p,q);
 }
-/* Let M and N in SL_2(Z), return (N*[1,0;0,-1]*M^-1)[,1]
- *
-*/
+/* Let M and N in SL2(Z), return (N*[1,0;0,-1]*M^-1)[,1] */
 static GEN
 SL2_swap_div_mul_e1(GEN N, GEN M)
 {
-  GEN b = gcoeff(M, 2, 1);
-  GEN d = gcoeff(M, 2, 2);
-  GEN p2 = cgetg(3, t_VEC);
-  gel(p2,1) = addii(mulii(gcoeff(N, 1, 1), d),
-                    mulii(gcoeff(N, 1, 2), b));
-  gel(p2,2) = addii(mulii(gcoeff(N, 2, 1), d),
-                    mulii(gcoeff(N, 2, 2), b));
-  return p2;
+  GEN b = gcoeff(M,2,1), d = gcoeff(M,2,2);
+  GEN p = addii(mulii(gcoeff(N,1,1), d), mulii(gcoeff(N,1,2), b));
+  GEN q = addii(mulii(gcoeff(N,2,1), d), mulii(gcoeff(N,2,2), b));
+  return mkvec2(p,q);
 }
 
-/*Test equality modulo GL2*/
+/* Test equality modulo GL2 of two reduced forms */
 static int
 GL2_qfb_equal(GEN a, GEN b)
 {
@@ -1183,8 +1269,8 @@ qfisolvep(GEN Q, GEN p)
   {
     a = gel(Q,1);
     c = gel(Q,3); /* if principal form, use faster cornacchia */
-    if (gequal1(a)) return qfbsolve_cornacchia(c, p, 0);
-    if (gequal1(c)) return qfbsolve_cornacchia(a, p, 1);
+    if (equali1(a)) return qfbsolve_cornacchia(c, p, 0);
+    if (equali1(c)) return qfbsolve_cornacchia(a, p, 1);
   }
   d = qfb_disc(Q); if (kronecker(d,p) < 0) return gen_0;
   a = redimagsl2(Q, &N);
@@ -1216,24 +1302,18 @@ qfisolvep(GEN Q, GEN p)
 }
 
 GEN
-redrealsl2step(GEN A)
+redrealsl2step(GEN A, GEN d, GEN rd)
 {
   pari_sp ltop = avma;
-  GEN N;
-  GEN V = gel(A,1);
-  GEN M = gel(A,2);
-  GEN a = gel(V,1);
-  GEN b = gel(V,2);
-  GEN c = gel(V,3);
-  GEN d = qfb_disc3(a,b,c);
-  GEN rd = sqrti(d);
+  GEN N, V = gel(A,1), M = gel(A,2);
+  GEN a = gel(V,1), b = gel(V,2), c = gel(V,3);
   GEN ac = mpabs(c);
   GEN r = addii(b, gmax(rd, ac));
   GEN q = truedvmdii(r, shifti(ac, 1), NULL);
   r = subii(mulii(shifti(q, 1), ac), b);
   a = c; b = r;
   c = truedvmdii(subii(sqri(r), d), shifti(c,2), NULL);
-  if (signe(a) < 0) q = negi(q);
+  if (signe(a) < 0) togglesign(q);
   N = mkmat2(gel(M,2),
              mkcol2(subii(mulii(q, gcoeff(M, 1, 2)), gcoeff(M, 1, 1)),
                     subii(mulii(q, gcoeff(M, 2, 2)), gcoeff(M, 2, 1))));
@@ -1241,17 +1321,11 @@ redrealsl2step(GEN A)
 }
 
 GEN
-redrealsl2(GEN V)
+redrealsl2(GEN V, GEN d, GEN rd)
 {
-  pari_sp ltop = avma, btop, st_lim;
-  GEN u1, u2, v1, v2;
-  GEN M;
-  GEN a = gel(V,1);
-  GEN b = gel(V,2);
-  GEN c = gel(V,3);
-  GEN d = qfb_disc3(a,b,c);
-  GEN rd = sqrti(d);
-  btop = avma; st_lim = stack_lim(btop, 1);
+  pari_sp ltop = avma, st_lim = stack_lim(ltop, 1);
+  GEN M, u1, u2, v1, v2;
+  GEN a = gel(V,1), b = gel(V,2), c = gel(V,3);
   u1 = v2 = gen_1; v1 = u2 = gen_0;
   while (!ab_isreduced(a,b,rd))
   {
@@ -1264,10 +1338,10 @@ redrealsl2(GEN V)
     if (signe(a) < 0) togglesign(q);
     r = u1; u1 = v1; v1 = subii(mulii(q, v1), r);
     r = u2; u2 = v2; v2 = subii(mulii(q, v2), r);
-    if (low_stack(st_lim, stack_lim(btop, 1)))
+    if (low_stack(st_lim, stack_lim(ltop, 1)))
     {
       if (DEBUGMEM>1) pari_warn(warnmem,"redrealsl2");
-      gerepileall(btop, 7, &a,&b,&c,&u1,&u2,&v1,&v2);
+      gerepileall(ltop, 7, &a,&b,&c,&u1,&u2,&v1,&v2);
     }
   }
   M = mkmat2(mkcol2(u1,u2), mkcol2(v1,v2));
@@ -1278,34 +1352,36 @@ GEN
 qfrsolvep(GEN Q, GEN p)
 {
   pari_sp ltop = avma, btop, st_lim;
-  GEN N, P, P1, P2, M, d = qfb_disc(Q);
+  GEN N, P, P1, P2, M, rd, d = qfb_disc(Q);
   if (kronecker(d, p) < 0) { avma = ltop; return gen_0; }
-  M = N = redrealsl2(Q);
+  rd = sqrti(d);
+  M = N = redrealsl2(Q, d,rd);
   P = primeform(d, p, DEFAULTPREC);
-  P1 = redrealsl2(P);
+  P1 = redrealsl2(P, d,rd);
   togglesign( gel(P,2) );
-  P2 = redrealsl2(P);
+  P2 = redrealsl2(P, d,rd);
   btop = avma; st_lim = stack_lim(btop, 1);
-  while (!ZV_equal(gel(M,1), gel(P1,1)) && !ZV_equal(gel(M,1), gel(P2,1)))
+  for(;;)
   {
-    M = redrealsl2step(M);
+    if (ZV_equal(gel(M,1), gel(P1,1))) { N = gel(P1,2); break; }
+    if (ZV_equal(gel(M,1), gel(P2,1))) { N = gel(P2,2); break; }
+    M = redrealsl2step(M, d,rd);
     if (ZV_equal(gel(M,1), gel(N,1))) { avma = ltop; return gen_0; }
     if (low_stack(st_lim, stack_lim(btop, 1))) M = gerepileupto(btop, M);
   }
-  N = ZV_equal(gel(M,1),gel(P1,1))? gel(P1,2): gel(P2,2);
   return gerepilecopy(ltop, SL2_div_mul_e1(gel(M,2),N));
 }
 
 GEN
 qfbsolve(GEN Q,GEN n)
 {
-  if (typ(n)!=t_INT) pari_err(typeer,"qfbsolve");
+  if (typ(n)!=t_INT) pari_err_TYPE("qfbsolve",n);
   switch(typ(Q))
   {
   case t_QFI: return qfisolvep(Q,n);
   case t_QFR: return qfrsolvep(Q,n);
   default:
-    pari_err(typeer,"qfbsolve");
+    pari_err_TYPE("qfbsolve",Q);
     return NULL; /* NOT REACHED */
   }
 }
@@ -1317,8 +1393,9 @@ cornacchia(GEN d, GEN p, GEN *px, GEN *py)
   pari_sp av = avma, av2, lim;
   GEN a, b, c, L, r;
 
-  if (typ(d) != t_INT || typ(p) != t_INT) pari_err(typeer, "cornacchia");
-  if (signe(d) <= 0) pari_err(talker, "d must be positive");
+  if (typ(d) != t_INT) pari_err_TYPE("cornacchia", d);
+  if (typ(p) != t_INT) pari_err_TYPE("cornacchia", p);
+  if (signe(d) <= 0) pari_err_DOMAIN("cornacchia", "d","<=",gen_0,d);
   *px = *py = gen_0;
   b = subii(p, d);
   if (signe(b) < 0) return 0;
@@ -1351,11 +1428,12 @@ cornacchia2(GEN d, GEN p, GEN *px, GEN *py)
   GEN a, b, c, L, r, px4;
   long k;
 
-  if (typ(d) != t_INT || typ(p) != t_INT) pari_err(typeer, "cornacchia");
-  if (signe(d) <= 0) pari_err(talker, "d must be positive");
+  if (typ(d) != t_INT) pari_err_TYPE("cornacchia2", d);
+  if (typ(p) != t_INT) pari_err_TYPE("cornacchia2", p);
+  if (signe(d) <= 0) pari_err_DOMAIN("cornacchia2", "d","<=",gen_0,d);
   *px = *py = gen_0;
   k = mod4(d);
-  if (k == 1 || k == 2) pari_err(talker,"d must be 0 or 3 mod 4");
+  if (k == 1 || k == 2) pari_err_DOMAIN("cornacchia2","-d mod 4", ">",gen_1,d);
   px4 = shifti(p,2);
   if (absi_cmp(px4, d) < 0) { avma = av; return 0; }
   if (equaliu(p, 2))
