@@ -596,7 +596,7 @@ nfpow_u(GEN nf, GEN z, ulong n)
   nf = checknf(nf); T = nf_get_pol(nf); N = degpol(T);
   if (!n) return scalarcol_shallow(gen_1,N);
   x = nf_to_scalar_or_basis(nf, z);
-  if (typ(x) != t_COL) { GEN y = zerocol(N); gel(y,1) = powiu(x,n); return y; }
+  if (typ(x) != t_COL) { GEN y = zerocol(N); gel(y,1) = gpowgs(x,n); return y; }
   x = primitive_part(x, &cx);
   x = gen_powu(x, n, (void*)nf, _sqr, _mul);
   if (cx) x = RgC_Rg_mul(x, powgi(cx, utoipos(n)));
@@ -713,6 +713,55 @@ nfval(GEN nf, GEN x, GEN pr)
   avma = av; return w;
 }
 
+/* want to write p^v = uniformizer^(e*v) * z^v, z coprime to pr */
+/* z := tau^e / p^(e-1), algebraic integer coprime to pr; return z^v */
+static GEN
+powp(GEN nf, GEN pr, long v)
+{
+  GEN b, z;
+  long e;
+  if (!v) return gen_1;
+  b = pr_get_tau(pr);
+  if (typ(b) == t_INT) return gen_1;
+  e = pr_get_e(pr);
+  z = gel(b,1);
+  if (e != 1) z = gdiv(nfpow_u(nf, z, e), powiu(pr_get_p(pr),e-1));
+  return nfpow_u(nf, z, v);
+}
+long
+nfvalrem(GEN nf, GEN x, GEN pr, GEN *py)
+{
+  pari_sp av = avma;
+  long w, e;
+  GEN cx, p, t;
+
+  if (!py) return nfval(nf,x,pr);
+  if (gequal0(x)) { *py = gcopy(x); return LONG_MAX; }
+  nf = checknf(nf);
+  checkprid(pr);
+  p = pr_get_p(pr);
+  e = pr_get_e(pr);
+  x = nf_to_scalar_or_basis(nf, x);
+  if (typ(x) != t_COL) {
+    w = Q_pvalrem(x,p, py);
+    if (!w) { *py = gerepilecopy(av, x); return 0; }
+    *py = gerepileupto(av, gmul(powp(nf, pr, w), *py));
+    return e*w;
+  }
+  x = Q_primitive_part(x, &cx);
+  w = ZC_nfvalrem(nf,x,pr, py);
+  if (cx)
+  {
+    long v = Q_pvalrem(cx,p, &t);
+    *py = nfmul(nf, *py, gmul(powp(nf,pr,v), t));
+    *py = gerepileupto(av, *py);
+    w += e*v;
+  }
+  else
+    *py = gerepilecopy(av, *py);
+  return w;
+}
+
 GEN
 coltoalg(GEN nf, GEN x)
 {
