@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -11,7 +9,7 @@ ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 
 /*******************************************************************/
 /*                                                                 */
@@ -22,92 +20,170 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include "pari.h"
 #include "paripriv.h"
 
-static GEN
-_checkrnfeq(GEN x)
-{
-  if (typ(x) == t_VEC)
-    switch(lg(x))
-    {
-      case 13: /* checkrnf(x); */ return gel(x,11);
-      case  4: return x;
-    }
-  return NULL;
-}
-
-GEN
-checkrnfeq(GEN x)
-{
-  x = _checkrnfeq(x);
-  if (!x) pari_err(talker,"please apply rnfequation(,,1)");
-  return x;
-}
-
+/* must return a t_POL */
 GEN
 eltreltoabs(GEN rnfeq, GEN x)
 {
-  long i, k, va;
+  long i, k, v;
   pari_sp av = avma;
-  GEN polabs, teta, alpha, s;
+  GEN T, pol, teta, a, s;
 
-  rnfeq = checkrnfeq(rnfeq);
-  polabs= gel(rnfeq,1);
-  alpha = lift_intern(gel(rnfeq,2));
-  k     = itos(gel(rnfeq,3));
+  pol = gel(rnfeq,1);
+  a = gel(rnfeq,2);
+  k = itos(gel(rnfeq,3));
+  T = gel(rnfeq,4);
 
-  va = varn(polabs);
-  if (varncmp(gvar(x), va) > 0) x = scalarpol(x,va);
-  /* Mod(X - k alpha, polabs(X)), alpha root of the polynomial defining base */
-  teta = gadd(pol_x(va), gmulsg(-k,alpha));
+  v = varn(pol);
+  if (varncmp(gvar(x), v) > 0) x = scalarpol(x,v);
+  x = RgX_nffix("eltreltoabs", T, x, 1);
+  /* Mod(X - k a, pol(X)), a root of the polynomial defining base */
+  teta = gadd(pol_x(v), gmulsg(-k,a));
   s = gen_0;
   for (i=lg(x)-1; i>1; i--)
   {
     GEN c = gel(x,i);
-    long tc = typ(c);
-    switch(tc)
-    {
-      case t_POLMOD: c = gel(c,2);
-        if (typ(c) != t_POL) break;
-        c = RgX_RgXQ_eval(c, alpha, polabs); break;
-      case t_POL:
-        c = RgX_RgXQ_eval(c, alpha, polabs); break;
-      default:
-        if (!is_const_t(tc)) pari_err(talker, "incorrect data in eltreltoabs");
-    }
-    s = RgX_rem(gadd(c, gmul(teta,s)), polabs);
+    if (typ(c) == t_POL) c = RgX_RgXQ_eval(c, a, pol);
+    s = RgX_rem(gadd(c, gmul(teta,s)), pol);
   }
   return gerepileupto(av, s);
 }
+GEN
+rnfeltreltoabs(GEN rnf,GEN x)
+{
+  const char *f = "rnfeltreltoabs";
+  GEN pol;
+  checkrnf(rnf);
+  pol = rnf_get_polabs(rnf);
+  switch(typ(x))
+  {
+    case t_INT: return icopy(x);
+    case t_FRAC: return gcopy(x);
+    case t_POLMOD:
+      if (RgX_equal_var(gel(x,1), pol))
+      { /* already in 'abs' form, unless possibly if nf = Q */
+        if (rnf_get_nfdegree(rnf) == 1)
+        {
+          GEN y = gel(x,2);
+          pari_sp av = avma;
+          y = simplify_shallow(liftpol_shallow(y));
+          return gerepilecopy(av, mkpolmod(y, pol));
+        }
+        return gcopy(x);
+      }
+      x = polmod_nffix(f,rnf,x,0);
+      if (typ(x) == t_POLMOD) return rnfeltup(rnf,x);
+      retmkpolmod(eltreltoabs(rnf_get_map(rnf), x), RgX_copy(pol));
+    case t_POL:
+      if (varn(x) == rnf_get_nfvarn(rnf)) return rnfeltup(rnf,x);
+      retmkpolmod(eltreltoabs(rnf_get_map(rnf), x), RgX_copy(pol));
+  }
+  pari_err_TYPE(f,x); return NULL;
+}
 
-/* x a t_VEC of rnf elements in 'alg' form */
+GEN
+eltabstorel_lift(GEN rnfeq, GEN P)
+{
+  GEN k, T = gel(rnfeq,4), relpol = gel(rnfeq,5);
+  if (is_scalar_t(typ(P))) return P;
+  k = gel(rnfeq,3);
+  P = lift_intern(P);
+  if (signe(k)) P = RgXQX_translate(P, deg1pol_shallow(k, gen_0, varn(T)), T);
+  P = RgXQX_rem(P, relpol, T);
+  return QXQX_to_mod_shallow(P, T);
+}
+/* rnfeq = [pol,a,k,T,relpol], P a t_POL or scalar
+ * Return Mod(P(x + k Mod(y, T(y))), pol(x)) */
+GEN
+eltabstorel(GEN rnfeq, GEN P)
+{
+  GEN T = gel(rnfeq,4), relpol = gel(rnfeq,5);
+  return mkpolmod(eltabstorel_lift(rnfeq,P), QXQX_to_mod_shallow(relpol,T));
+}
+GEN
+rnfeltabstorel(GEN rnf,GEN x)
+{
+  const char *f = "rnfeltabstorel";
+  pari_sp av = avma;
+  GEN pol, T, P;
+  checkrnf(rnf);
+  T = rnf_get_nfpol(rnf);
+  P = rnf_get_pol(rnf);
+  switch(typ(x))
+  {
+    case t_INT: return icopy(x);
+    case t_FRAC: return gcopy(x);
+    case t_POLMOD:
+      if (RgX_equal_var(P, gel(x,1)))
+      {
+        x = polmod_nffix(f, rnf, x, 0);
+        return gerepilecopy(av, mkpolmod(x,P));
+      }
+      if (RgX_equal_var(T, gel(x,1))) { x = Rg_nffix(f, T, x, 0); goto END; }
+      pol = rnf_get_polabs(rnf);
+      if (!RgX_equal_var(pol, gel(x,1))) pari_err_MODULUS(f, gel(x,1),pol);
+      x = gel(x,2);
+      switch(typ(x))
+      {
+        case t_INT: return icopy(x);
+        case t_FRAC: return gcopy(x);
+        case t_POL: break;
+        default: pari_err_TYPE(f, x);
+      }
+      break;
+    case t_POL:
+      pol = rnf_get_polabs(rnf);
+      break;
+    default:
+      pari_err_TYPE(f,x);
+      return NULL;
+  }
+  if (!RgX_is_QX(x)) pari_err_TYPE(f,x);
+  if (varn(x) != varn(pol))
+  {
+    if (varn(x) == varn(T)) { x = Rg_nffix(f,T,x,0); goto END; }
+    pari_err_VAR(f, x,pol);
+  }
+  switch(lg(x))
+  {
+    case 2: avma = av; return gen_0;
+    case 3: return gerepilecopy(av, gel(x,2));
+  }
+END:
+  return gerepilecopy(av, eltabstorel(rnf_get_map(rnf), x));
+}
+
+
+/* x a t_VEC of rnf elements in 'alg' form (t_POL). Assume maximal rank or 0 */
 static GEN
 modulereltoabs(GEN rnf, GEN x)
 {
-  GEN W = gel(x,1), I = gel(x,2), nf = gel(rnf,10), rnfeq = gel(rnf,11);
-  GEN M, basnf, cobasnf, T = nf_get_pol(nf), polabs = gel(rnfeq,1);
-  long i, j, k, n = lg(W)-1, m = degpol(T);
+  GEN W=gel(x,1), I=gel(x,2), rnfeq = rnf_get_map(rnf), polabs = gel(rnfeq,1);
+  long i, j, k, m, N = lg(W)-1;
+  GEN zknf, czknf, M;
 
-  M = cgetg(n*m+1, t_VEC);
-  basnf = lift_intern( gsubst(nf_get_zk(nf), varn(T), gel(rnfeq,2)) );
-  basnf = Q_primitive_part(basnf, &cobasnf); /* remove denom. --> faster */
-  for (k=i=1; i<=n; i++)
+  if (!N) return cgetg(1, t_VEC);
+  rnf_get_nfzk(rnf, &zknf,&czknf);
+  m = rnf_get_nfdegree(rnf);
+  M = cgetg(N*m+1, t_VEC);
+  for (k=i=1; i<=N; i++)
   {
     GEN c0, cid, w = gel(W,i), id = gel(I,i);
 
     if (lg(id) == 1) continue; /* must be a t_MAT */
     id = Q_primitive_part(id, &cid);
     w = Q_primitive_part(eltreltoabs(rnfeq,w), &c0);
-    c0 = mul_content(c0, mul_content(cid,cobasnf));
+    c0 = mul_content(c0, mul_content(cid,czknf));
     if (typ(id) == t_INT)
       for (j=1; j<=m; j++)
       {
-        GEN z = RgX_rem(gmul(w, gel(basnf,j)), polabs);
+        GEN z = RgX_rem(gmul(w, gel(zknf,j)), polabs);
         if (c0) z = RgX_Rg_mul(z, c0);
         gel(M,k++) = z;
       }
     else
       for (j=1; j<=m; j++)
       {
-        GEN c, z = Q_primitive_part(gmul(basnf,gel(id,j)), &c);
+        GEN c, z = Q_primitive_part(RgV_RgC_mul(zknf,gel(id,j)), &c);
         z = RgX_rem(gmul(w, z), polabs);
         c = mul_content(c, c0); if (c) z = RgX_Rg_mul(z, c);
         gel(M,k++) = z;
@@ -116,34 +192,42 @@ modulereltoabs(GEN rnf, GEN x)
   setlg(M, k); return M;
 }
 
+/* Z-basis for absolute maximal order, as a t_MAT */
+GEN
+rnf_basM(GEN rnf)
+{
+  GEN M, d, pol = rnf_get_polabs(rnf);
+  long n = degpol(pol);
+  /* t_VEC of t_POL */
+  M = Q_remove_denom(modulereltoabs(rnf, rnf_get_zk(rnf)), &d);
+  if (d)
+  {
+    M = ZM_hnfmodall(RgXV_to_RgM(M,n), d, hnf_MODID|hnf_CENTER);
+    M = RgM_Rg_div(M, d);
+  }
+  else
+    M = matid(n);
+  return M;
+}
+
+/* only fill in nf[1,3,4,7,8,9] */
 static GEN
 makenfabs(GEN rnf)
 {
-  GEN M, d, rnfeq, pol, nf, NF = zerovec(9);
-  long n;
-
-  rnfeq = gel(rnf,11); pol = gel(rnfeq,1);
-  nf = gel(rnf,10);
-
-  M = modulereltoabs(rnf, gel(rnf,7));
-  n = degpol(pol);
-  M = RgXV_to_RgM(Q_remove_denom(M, &d), n);
-  if (d) M = RgM_Rg_div(ZM_hnfmodall(M, d, hnf_MODID|hnf_CENTER), d);
-  else   M = matid(n);
-
+  GEN nf = rnf_get_nf(rnf), pol = rnf_get_polabs(rnf), NF = zerovec(9);
+  GEN M = rnf_basM(rnf);
   gel(NF,1) = pol;
   gel(NF,3) = mulii(powiu(nf_get_disc(nf), rnf_get_degree(rnf)),
-                    idealnorm(nf, gel(rnf,3)));
+                    idealnorm(nf, rnf_get_disc(rnf)));
   nf_set_multable(NF, M, NULL);
-  /* possibly wrong, but correct prime divisors [for idealprimedec] */
-  gel(NF,4) = Q_denom(gel(NF,7));
+  gel(NF,4) = get_nfindex(nf_get_zk(NF));
   return NF;
 }
 
 static GEN
 makenorms(GEN rnf)
 {
-  GEN f = gel(rnf,4);
+  GEN f = rnf_get_index(rnf);
   return typ(f) == t_INT? gen_1: RgM_det_triangular(f);
 }
 
@@ -151,27 +235,36 @@ makenorms(GEN rnf)
 #define NORMS 2
 GEN
 check_and_build_nfabs(GEN rnf) {
-  return check_and_build_obj(rnf, NFABS, &makenfabs);
+  return obj_checkbuild(rnf, NFABS, &makenfabs);
 }
 GEN
 check_and_build_norms(GEN rnf) {
-  return check_and_build_obj(rnf, NORMS, &makenorms);
+  return obj_checkbuild(rnf, NORMS, &makenorms);
+}
+
+void
+nf_nfzk(GEN nf, GEN rnfeq, GEN *zknf, GEN *czknf)
+{
+  GEN pol = gel(rnfeq,1), a = gel(rnfeq,2);
+  GEN zk = QXV_QXQ_eval(nf_get_zk(nf), a, pol);
+  *zknf = Q_primitive_part(zk, czknf);
+  if (!*czknf) *czknf = gen_1;
 }
 
 GEN
-rnfinit(GEN nf, GEN pol)
+rnfinit(GEN nf, GEN polrel)
 {
   pari_sp av = avma;
-  GEN rnf, bas, D,d,f, B;
-
+  GEN rnf, bas, D,d,f, B, rnfeq, basnf,cobasnf;
   nf = checknf(nf);
-  bas = rnfallbase(nf,&pol, &D,&d, &f);
+  bas = rnfallbase(nf,&polrel, &D,&d, &f);
   B = matbasistoalg(nf,gel(bas,1));
-  gel(bas,1) = lift_if_rational( RgM_to_RgXV(B,varn(pol)) );
-
+  gel(bas,1) = lift_if_rational( RgM_to_RgXV(B,varn(polrel)) );
+  rnfeq = nf_rnfeq(nf,polrel);
+  nf_nfzk(nf, rnfeq, &basnf, &cobasnf);
   rnf = cgetg(13, t_VEC);
-  gel(rnf,1) = pol;
-  gel(rnf,2) = cgetg(1, t_VEC); /* dummy */
+  gel(rnf,1) = polrel;
+  gel(rnf,2) = mkvec2(basnf, cobasnf);
   gel(rnf,3) = mkvec2(D, d);
   gel(rnf,4) = f;
   gel(rnf,5) = cgetg(1, t_VEC); /* dummy */
@@ -179,171 +272,132 @@ rnfinit(GEN nf, GEN pol)
   gel(rnf,7) = bas;
   gel(rnf,8) = lift_if_rational( RgM_inv(B) );
   gel(rnf,9) = cgetg(1,t_VEC); /* dummy */
-  gel(rnf,10) = nf;
-  gel(rnf,11) = rnfequation2(nf,pol);
-  gel(rnf,12) = gen_0;
+  gel(rnf,10)= nf;
+  gel(rnf,11)= rnfeq;
+  gel(rnf,12)= zerovec(2);
   return gerepilecopy(av, rnf);
 }
 
 GEN
-rnfelementreltoabs(GEN rnf,GEN x)
-{
-  long i, lx;
-  GEN z;
-
-  switch(typ(x))
-  {
-    case t_VEC: case t_COL: case t_MAT:
-      z = cgetg_copy(x, &lx);
-      for (i=1; i<lx; i++) gel(z,i) = rnfelementreltoabs(rnf, gel(x,i));
-      return z;
-
-    case t_POLMOD: x = gel(x,2); /* fall through */
-    case t_POL: return eltreltoabs(rnf, x);
-    default: return gcopy(x);
-  }
-}
-
-/* Let T,pol t_POL. T(y) defines base field, pol(x) defines rnf over T
- * Return Mod(x + k y, pol) */
-GEN
-get_theta_abstorel(GEN T, GEN pol, GEN k)
-{
-  GEN u, ky = signe(k)? deg1pol_shallow(k, gen_0, varn(T)): gen_0;
-  u = deg1pol_shallow(mkpolmod(gen_1, T), mkpolmod(ky, T), varn(pol));
-  if (degpol(pol) == 1) u = RgX_rem(u,pol);
-  return mkpolmod(u, pol);
-}
-GEN
-eltabstorel(GEN x, GEN T, GEN pol, GEN k)
-{
-  return poleval(x, get_theta_abstorel(T,pol,k));
-}
-
-GEN
-rnfelementabstorel(GEN rnf,GEN x)
+rnfeltup(GEN rnf, GEN x)
 {
   pari_sp av = avma;
-  long i, lx;
-  GEN z;
+  GEN zknf, czknf;
+  checkrnf(rnf);
+  if (typ(x) == t_POLMOD && RgX_equal_var(gel(x,1), rnf_get_polabs(rnf)))
+    return gcopy(x);
+  rnf_get_nfzk(rnf, &zknf, &czknf);
+  x = nfeltup(rnf_get_nf(rnf), x, zknf, czknf);
+  if (typ(x) == t_POL) x = mkpolmod(x, rnf_get_polabs(rnf));
+  return gerepilecopy(av, x);
+}
+
+GEN
+nfeltup(GEN nf, GEN x, GEN zknf, GEN czknf)
+{
+  GEN c;
+  x = nf_to_scalar_or_basis(nf, x);
+  if (typ(x) != t_COL) return x;
+  x = Q_primitive_part(x, &c);
+  if (!RgV_is_ZV(x)) pari_err_TYPE("rnfeltup", x);
+  c = mul_content(c, czknf);
+  x = RgV_RgC_mul(zknf, x); if (c) x = RgX_Rg_mul(x, c);
+  return x;
+}
+
+static void
+fail(const char *f, GEN x)
+{ pari_err_DOMAIN(f,"element","not in", strtoGENstr("the base field"),x); }
+GEN
+rnfeltdown(GEN rnf,GEN x)
+{
+  const char *f = "rnfeltdown";
+  pari_sp av = avma;
+  GEN z, T;
+  long v;
 
   checkrnf(rnf);
+  T = rnf_get_nfpol(rnf);
+  v = varn(T);
   switch(typ(x))
-  {
-    case t_VEC: case t_COL: case t_MAT:
-      z = cgetg_copy(x, &lx);
-      for (i=1; i<lx; i++) gel(z,i) = rnfelementabstorel(rnf,gel(x,i));
-      return z;
-
+  { /* directly belonging to base field ? */
+    case t_INT: return icopy(x);
+    case t_FRAC:return gcopy(x);
     case t_POLMOD:
-      x = gel(x,2);
-      if (typ(x) != t_POL) return gcopy(x);
-      /* fall through */
+      if (RgX_equal_var(gel(x,1), rnf_get_polabs(rnf))) break;
+      x = polmod_nffix(f,rnf,x,0);
+      /* x was defined mod the relative polynomial & non constant => fail */
+      if (typ(x) == t_POL) fail(f,x);
+      return gerepilecopy(av, x);
+
     case t_POL:
-    {
-      GEN k, T, pol, rnfeq = gel(rnf,11), nf = gel(rnf,10);
-      k = gel(rnfeq,3);
-      T = nf_get_pol(nf);
-      pol = gel(rnf,1);
-      return gerepileupto(av, eltabstorel(x, T, pol, k));
-    }
-
-    default: return gcopy(x);
+      if (varn(x) != v) break;
+      x = Rg_nffix(f,T,x,0);
+      return gerepilecopy(av, x);
   }
-}
-
-/* x t_POLMOD or t_POL or vector of such objects */
-GEN
-rnfelementup(GEN rnf,GEN x)
-{
-  long i, lx;
-  GEN z;
-
-  checkrnf(rnf);
-  switch(typ(x))
+  /* x defined mod the absolute equation */
+  z = rnfeltabstorel(rnf,x);
+  switch(typ(z))
   {
-    case t_VEC: case t_COL: case t_MAT:
-      z = cgetg_copy(x, &lx);
-      for (i=1; i<lx; i++) gel(z,i) = rnfelementup(rnf,gel(x,i));
-      return z;
-
-    case t_POLMOD: x = gel(x,2); /* fall through */
-    case t_POL:
-      return poleval(x, gmael(rnf,11,2));
-
-    default: return gcopy(x);
+    case t_INT:
+    case t_FRAC: return z;
   }
-}
-
-/* x t_POLMOD or t_POL or vector of such objects */
-GEN
-rnfelementdown(GEN rnf,GEN x)
-{
-  pari_sp av;
-  long i, lx;
-  GEN z;
-
-  checkrnf(rnf);
-  switch(typ(x))
+  /* typ(z) = t_POLMOD, varn of both components is rnf_get_varn(rnf) */
+  z = gel(z,2);
+  if (typ(z) == t_POL)
   {
-    case t_VEC: case t_COL: case t_MAT:
-      z = cgetg_copy(x, &lx);
-      for (i=1; i<lx; i++) gel(z,i) = rnfelementdown(rnf,gel(x,i));
-      return z;
-
-    case t_POLMOD: x = gel(x,2); /* fall through */
-    case t_POL:
-      if (gequal0(x)) return gen_0;
-      av = avma; z = rnfelementabstorel(rnf,x);
-      if (typ(z)==t_POLMOD && varn(z[1])==varn(rnf[1])) z = gel(z,2);
-      if (varncmp(gvar(z), varn(rnf[1])) <= 0)
-      {
-        lx = lg(z);
-        if (lx == 2) { avma = av; return gen_0; }
-        if (lx > 3)
-          pari_err(talker,"element is not in the base field in rnfelementdown");
-        z = gel(z,2);
-      }
-      return gerepilecopy(av, z);
-
-    default: return gcopy(x);
+    if (lg(z) != 3) fail(f,x);
+    z = gel(z,2);
   }
+  return gerepilecopy(av, z);
 }
 
-/* x est exprime sur la base relative */
+/* vector of rnf elt -> matrix of nf elts */
 static GEN
-rnfprincipaltohermite(GEN rnf,GEN x)
+rnfV_to_nfM(GEN rnf, GEN x)
+{
+  long i, l = lg(x);
+  GEN y = cgetg(l, t_MAT);
+  for (i = 1; i < l; i++) gel(y,i) = rnfalgtobasis(rnf,gel(x,i));
+  return y;
+}
+
+static GEN
+rnfprincipaltohnf(GEN rnf,GEN x)
 {
   pari_sp av = avma;
-  GEN bas = gel(rnf,7), nf = gel(rnf,10);
-
+  GEN bas = rnf_get_zk(rnf), nf = rnf_get_nf(rnf);
   x = rnfbasistoalg(rnf,x);
-  x = rnfalgtobasis(rnf, gmul(x, gmodulo(gel(bas,1), gel(rnf,1))));
-  settyp(x, t_MAT);
-  return gerepileupto(av, nfhnf(nf, mkvec2(x, gel(bas,2))));
+  x = gmul(x, gmodulo(gel(bas,1), rnf_get_pol(rnf)));
+  return gerepileupto(av, nfhnf(nf, mkvec2(rnfV_to_nfM(rnf,x), gel(bas,2))));
 }
 
+/* pseudo-basis for the 0 ideal */
+static GEN
+rnfideal0() { retmkvec2(cgetg(1,t_MAT),cgetg(1,t_VEC)); }
+
 GEN
-rnfidealhermite(GEN rnf, GEN x)
+rnfidealhnf(GEN rnf, GEN x)
 {
   GEN z, nf, bas;
 
-  checkrnf(rnf); nf = gel(rnf,10);
+  checkrnf(rnf); nf = rnf_get_nf(rnf);
   switch(typ(x))
   {
     case t_INT: case t_FRAC:
-      bas = gel(rnf,7); z = cgetg(3,t_VEC);
+      if (isintzero(x)) return rnfideal0();
+      bas = rnf_get_zk(rnf); z = cgetg(3,t_VEC);
       gel(z,1) = matid(rnf_get_degree(rnf));
       gel(z,2) = gmul(x, gel(bas,2)); return z;
 
     case t_VEC:
-      if (lg(x) == 3 && typ(x[1]) == t_MAT) return nfhnf(nf, x);
+      if (lg(x) == 3 && typ(gel(x,1)) == t_MAT) return nfhnf(nf, x);
       return rnfidealabstorel(rnf, x);
 
     case t_POLMOD: case t_POL: case t_COL:
-      return rnfprincipaltohermite(rnf,x);
+      return rnfprincipaltohnf(rnf,x);
   }
-  pari_err(typeer,"rnfidealhermite");
+  pari_err_TYPE("rnfidealhnf",x);
   return NULL; /* not reached */
 }
 
@@ -373,24 +427,19 @@ GEN
 rnfidealnormrel(GEN rnf, GEN id)
 {
   pari_sp av = avma;
-  GEN z, nf;
-  checkrnf(rnf); nf = gel(rnf,10);
-  if (rnf_get_degree(rnf) == 1) return matid(nf_get_degree(nf));
-
-  z = prodid(nf, gel(rnfidealhermite(rnf,id),2));
-  return gerepileupto(av, idealmul(nf,z, gel(rnf,4)));
+  GEN nf, z = gel(rnfidealhnf(rnf,id), 2);
+  if (lg(z) == 1) return cgetg(1, t_MAT);
+  nf = rnf_get_nf(rnf); z = prodid(nf, z);
+  return gerepileupto(av, idealmul(nf,z, rnf_get_index(rnf)));
 }
 
 GEN
 rnfidealnormabs(GEN rnf, GEN id)
 {
   pari_sp av = avma;
-  GEN z, nf;
-
-  checkrnf(rnf);
-  if (rnf_get_degree(rnf) == 1) return gen_1;
-  nf = gel(rnf,10);
-  z = prodidnorm(nf, gel(rnfidealhermite(rnf,id),2));
+  GEN nf, z = gel(rnfidealhnf(rnf,id), 2);
+  if (lg(z) == 1) return gen_0;
+  nf = rnf_get_nf(rnf); z = prodidnorm(nf, z);
   return gerepileupto(av, gmul(z, check_and_build_norms(rnf)));
 }
 
@@ -401,7 +450,7 @@ rnfidealreltoabs(GEN rnf,GEN x)
   long i, l;
   GEN w;
 
-  x = rnfidealhermite(rnf,x);
+  x = rnfidealhnf(rnf,x);
   w = gel(x,1); l = lg(w); settyp(w, t_VEC);
   for (i=1; i<l; i++) gel(w,i) = lift_intern( rnfbasistoalg(rnf, gel(w,i)) );
   return gerepilecopy(av, modulereltoabs(rnf, x));
@@ -410,31 +459,38 @@ rnfidealreltoabs(GEN rnf,GEN x)
 GEN
 rnfidealabstorel(GEN rnf, GEN x)
 {
-  long N, m, j;
+  long N, j;
   pari_sp av = avma;
-  GEN nf, A, I, z, invbas;
+  GEN A, I, invbas;
 
-  checkrnf(rnf); nf = gel(rnf,10); invbas = gel(rnf,8);
-  m = nf_get_degree(nf);
-  N = m * rnf_get_degree(rnf);
-  if (lg(x)-1 != N) pari_err(typeer, "rnfidealabstorel");
-  if (typ(x) != t_VEC) pari_err(typeer,"rnfidealabstorel");
+  checkrnf(rnf);
+  invbas = rnf_get_invzk(rnf);
+  if (typ(x) != t_VEC) pari_err_TYPE("rnfidealabstorel",x);
+  N = lg(x)-1;
+  if (N != rnf_get_absdegree(rnf))
+  {
+    if (!N) return rnfideal0();
+    pari_err_DIM("rnfidealabstorel");
+  }
   A = cgetg(N+1,t_MAT);
-  I = cgetg(N+1,t_VEC); z = mkvec2(A,I);
+  I = cgetg(N+1,t_VEC);
   for (j=1; j<=N; j++)
   {
-    GEN t = lift_intern( rnfelementabstorel(rnf, gel(x,j)) );
+    GEN t = lift_intern( rnfeltabstorel(rnf, gel(x,j)) );
     gel(A,j) = mulmat_pol(invbas, t);
     gel(I,j) = gen_1;
   }
-  return gerepileupto(av, nfhnf(nf,z));
+  return gerepileupto(av, nfhnf(rnf_get_nf(rnf), mkvec2(A,I)));
 }
 
 GEN
 rnfidealdown(GEN rnf,GEN x)
 {
-  pari_sp av = avma; x = rnfidealhermite(rnf,x);
-  return gerepilecopy(av, gmael(x,2,1));
+  pari_sp av = avma;
+  GEN I;
+  x = rnfidealhnf(rnf,x); I = gel(x,2);
+  if (lg(I) == 1) { avma = av; return cgetg(1,t_MAT); }
+  return gerepilecopy(av, gel(I,1));
 }
 
 /* lift ideal x to the relative extension, returns a Z-basis */
@@ -445,9 +501,9 @@ rnfidealup(GEN rnf,GEN x)
   long i, n;
   GEN nf, bas, bas2, I;
 
-  checkrnf(rnf); nf = gel(rnf,10);
+  checkrnf(rnf); nf = rnf_get_nf(rnf);
   n = rnf_get_degree(rnf);
-  bas = gel(rnf,7); bas2 = gel(bas,2);
+  bas = rnf_get_zk(rnf); bas2 = gel(bas,2);
 
   (void)idealtyp(&x, &I); /* I is junk */
   I = cgetg(n+1,t_VEC);
@@ -455,35 +511,40 @@ rnfidealup(GEN rnf,GEN x)
   return gerepilecopy(av, modulereltoabs(rnf, mkvec2(gel(bas,1), I)));
 }
 
-/* x a relative HNF ---> vector of 2 generators (relative polymods) */
+/* x a relative HNF => vector of 2 generators (relative polmods) */
 GEN
 rnfidealtwoelement(GEN rnf, GEN x)
 {
   pari_sp av = avma;
-  GEN y, z, NF;
+  GEN y, cy, z, NF;
 
-  checkrnf(rnf);
-  NF = check_and_build_nfabs(rnf);
   y = rnfidealreltoabs(rnf,x);
+  NF = check_and_build_nfabs(rnf);
   y = matalgtobasis(NF, y); settyp(y, t_MAT);
-  y = idealtwoelt(NF, ZM_hnf(y));
-  z = rnfelementabstorel(rnf, gmul(gel(NF,7), gel(y,2)));
+  y = Q_primitive_part(y, &cy);
+  y = ZM_hnf(y);
+  if (lg(y) == 1) { avma = av; return mkvec2(gen_0, gen_0); }
+  y = idealtwoelt(NF, y);
+  if (cy) y = RgV_Rg_mul(y, cy);
+  z = rnfeltabstorel(rnf, coltoliftalg(NF, gel(y,2)));
   return gerepilecopy(av, mkvec2(gel(y,1), z));
 }
 
 GEN
-rnfidealmul(GEN rnf,GEN x,GEN y) /* x et y sous HNF relative uniquement */
+rnfidealmul(GEN rnf,GEN x,GEN y)
 {
   pari_sp av = avma;
-  GEN z, nf, x1, x2, p1, p2;
+  GEN nf, z, x1, x2, p1, p2, bas;
 
-  z = rnfidealtwoelement(rnf,y);
-  nf = gel(rnf,10);
-  x = rnfidealhermite(rnf,x);
-  x1 = gmodulo(gmul(gmael(rnf,7,1), matbasistoalg(nf,gel(x,1))),gel(rnf,1));
+  y = rnfidealtwoelement(rnf,y);
+  if (isintzero(gel(y,1))) { avma = av; return rnfideal0(); }
+  nf = rnf_get_nf(rnf);
+  bas = rnf_get_zk(rnf);
+  x = rnfidealhnf(rnf,x);
+  x1 = gmodulo(gmul(gel(bas,1), matbasistoalg(nf,gel(x,1))), rnf_get_pol(rnf));
   x2 = gel(x,2);
-  p1 = gmul(gel(z,1), gel(x,1));
-  p2 = rnfalgtobasis(rnf, gmul(gel(z,2), x1)); settyp(p2, t_MAT);
+  p1 = gmul(gel(y,1), gel(x,1));
+  p2 = rnfV_to_nfM(rnf, gmul(gel(y,2), x1));
   z = mkvec2(shallowconcat(p1, p2), shallowconcat(x2, x2));
   return gerepileupto(av, nfhnf(nf,z));
 }
@@ -508,16 +569,15 @@ rnfequationall(GEN A, GEN B, long *pk, GEN *pLPRS)
 
   A = get_nfpol(A, &nf); lA = lg(A);
   if (!nf) {
-    if (lA<=3) pari_err(constpoler,"rnfequation");
+    if (lA<=3) pari_err_CONSTPOL("rnfequation");
     RgX_check_ZX(A,"rnfequation");
   }
-  B = rnf_fix_pol(A,B,1); lB = lg(B);
-  if (lB<=3) pari_err(constpoler,"rnfequation");
+  B = RgX_nffix("rnfequation", A,B,1); lB = lg(B);
+  if (lB<=3) pari_err_CONSTPOL("rnfequation");
   B = Q_primpart(B);
-  RgX_check_ZXY(B,"rnfequation");
 
   if (!nfissquarefree(A,B))
-    pari_err(talker,"inseparable relative equation in rnfequation");
+    pari_err_DOMAIN("rnfequation","issquarefree(B)","=",gen_0,B);
 
   *pk = 0; C = ZX_ZXY_resultant_all(A, B, pk, pLPRS);
   if (gsigne(leading_term(C)) < 0) C = RgX_neg(C);
@@ -544,6 +604,25 @@ GEN
 rnfequation(GEN nf, GEN pol) { return rnfequation0(nf,pol,0); }
 GEN
 rnfequation2(GEN nf, GEN pol) { return rnfequation0(nf,pol,1); }
+GEN
+nf_rnfeq(GEN nf, GEN relpol)
+{
+  GEN pol, a, k, junk, eq;
+  relpol = liftpol_shallow(relpol);
+  eq = rnfequation2(nf, relpol);
+  pol = gel(eq,1);
+  a = gel(eq,2); if (typ(a) == t_POLMOD) a = gel(a,2);
+  k = gel(eq,3);
+  return mkvec5(pol,a,k,get_nfpol(nf, &junk),relpol);
+}
+/* only allow abstorel */
+GEN
+nf_rnfeqsimple(GEN nf, GEN relpol)
+{
+  long sa;
+  GEN junk, pol = rnfequationall(nf, relpol, &sa, NULL);
+  return mkvec5(pol,gen_0/*dummy*/,stoi(sa),get_nfpol(nf, &junk),relpol);
+}
 
 static GEN
 nftau(long r1, GEN x)
@@ -569,7 +648,7 @@ nftocomplex(GEN nf, GEN x)
 {
   GEN M = nf_get_M(nf);
   x = nf_to_scalar_or_basis(nf,x);
-  if (typ(x) != t_COL) return const_col(lg(M[1])-1, x);
+  if (typ(x) != t_COL) return const_col(nbrows(M), x);
   return RgM_RgC_mul(M, x);
 }
 /* assume x a square t_MAT, return a t_VEC of embeddings of its columns */
@@ -591,7 +670,7 @@ mattocomplex(GEN nf, GEN x)
 static GEN
 nf_all_roots(GEN nf, GEN x, long prec)
 {
-  long i, j, l = lg(x), ru = lg(nf[6]);
+  long i, j, l = lg(x), ru = lg(nf_get_roots(nf));
   GEN y = cgetg(l, t_POL), v, z;
 
   x = RgX_to_nfX(nf, x);
@@ -637,7 +716,7 @@ findmin(GEN nf, GEN x, GEN muf)
     {
       x = ZM_lll(x, 0.75, LLL_INPLACE);
       m = lllfp(RgM_mul(G,x), 0.75, 0);
-      if (typ(m) != t_MAT) pari_err(precer,"rnflllgram");
+      if (typ(m) != t_MAT) pari_err_PREC("rnflllgram");
     }
     x = ZM_mul(x, m);
     y = RgM_mul(M, x);
@@ -778,7 +857,7 @@ rnflllgram(GEN nf, GEN pol, GEN order,long prec)
   M = gel(order,1);
   I = gel(order,2); lx = lg(I);
   if (lx < 3) return gcopy(order);
-  if (lx-1 != degpol(pol)) pari_err(consister,"rnflllgram");
+  if (lx-1 != degpol(pol)) pari_err_DIM("rnflllgram");
   I = leafcopy(I);
   H = NULL;
   MPOL = matbasistoalg(nf, M);
@@ -786,7 +865,7 @@ rnflllgram(GEN nf, GEN pol, GEN order,long prec)
 PRECNF:
   if (count == MAX_COUNT)
   {
-    prec = (prec<<1)-2; count = 0;
+    prec = precdbl(prec); count = 0;
     if (DEBUGLEVEL) pari_warn(warnprec,"rnflllgram",prec);
     nf = nfnewprec_shallow(nf,prec);
   }
@@ -811,7 +890,7 @@ PRECPB:
   }
   if (DEBUGLEVEL) err_printf("k = ");
   gel(B,1) = real_i(rnfscal(mth,gel(MC,1),gel(MC,1)));
-  MCS[1] = MC[1];
+  gel(MCS,1) = gel(MC,1);
   kmax = 1; k = 2;
   do
   {
@@ -819,7 +898,7 @@ PRECPB:
     if (DEBUGLEVEL) err_printf("%ld ",k);
     if (k > kmax)
     { /* Incremental Gram-Schmidt */
-      kmax = k; MCS[k] = MC[k];
+      kmax = k; gel(MCS,k) = gel(MC,k);
       for (j=1; j<k; j++)
       {
         gcoeff(mu,k,j) = vecdiv(rnfscal(mth,gel(MCS,j),gel(MC,k)),
@@ -863,7 +942,7 @@ rnfpolred(GEN nf, GEN pol, long prec)
   long i, j, n, v = varn(pol);
   GEN id, w, I, O, bnf, nfpol;
 
-  if (typ(pol)!=t_POL) pari_err(typeer,"rnfpolred");
+  if (typ(pol)!=t_POL) pari_err_TYPE("rnfpolred",pol);
   bnf = nf; nf = checknf(bnf);
   bnf = (nf == bnf)? NULL: checkbnf(bnf);
   if (degpol(pol) <= 1) { w = cgetg(2, t_VEC); gel(w,1) = pol_x(v); return w; }
@@ -911,106 +990,3 @@ rnfpolred(GEN nf, GEN pol, long prec)
   }
   return gerepilecopy(av,w);
 }
-
-/* Let K = Q[X]/T = nf. Given a relative polynomial pol in K[X], L = K[X]/(pol),
- * compute a pseudo-basis for Z_L, then an absolute basis */
-static GEN
-makebasis(GEN nf, GEN pol, GEN rnfeq)
-{
-  GEN T = nf_get_pol(nf), TAB = gel(nf,9), W, I, polabs, a, B, ZK, p1, den, A;
-  pari_sp av = avma;
-  long i, j, k, N = degpol(pol), n = degpol(T), nN = n*N;
-
-  polabs= gel(rnfeq,1); /* in Z[X], L = Q[X] / polabs, and pol | polabs */
-  a     = gel(rnfeq,2); a = lift_intern(a); /* root of T in Q[X]/polabs */
-  p1 = rnfpseudobasis(nf,pol);
-  W = gel(p1,1);
-  I = gel(p1,2);
-  if (DEBUGLEVEL>1) err_printf("relative basis computed\n");
-
-  A = QXQ_powers(a, n-1, polabs);
-  /* ZK = integer basis of K, as elements of L */
-  ZK = RgV_RgM_mul(A, RgXV_to_RgM(nf_get_zk(nf),n));
-
-  W = RgV_RgM_mul(pol_x_powers(N, varn(pol)), W); /* vector of nfX */
-  B = cgetg(nN+1, t_MAT);
-  for(i=k=1; i<=N; i++)
-  {
-    GEN w = gel(W,i), id = gel(I,i);
-    if (typ(id) == t_MAT)
-    {
-      w = typ(w) == t_COL? tablemulvec(TAB, w, id): RgM_Rg_mul(id,w);
-      for(j=1; j<=n; j++)
-      {
-        p1 = grem(RgV_dotproduct(ZK, gel(w,j)), polabs);
-        gel(B,k++) = RgX_to_RgV(p1, nN);
-      }
-    }
-    else
-    { /* scalar */
-      if (typ(id) != t_INT || !is_pm1(id)) w = gmul(w, id);
-      for(j=1; j<=n; j++)
-      {
-        p1 = grem(gmul(gel(ZK,j), w), polabs);
-        gel(B,k++) = RgX_to_RgV(p1, nN);
-      }
-    }
-  }
-  B = Q_remove_denom(B, &den);
-  if (den) { B = ZM_hnfmodid(B, den); B = RgM_Rg_div(B, den); }
-  else B = matid(nN);
-  return gerepilecopy(av, mkvec2(polabs, B));
-}
-
-/* relative polredabs. Returns relative polynomial by default (flag = 0)
- * flag & nf_ORIG: + element (base change)
- * flag & nf_ADDZK: + integer basis
- * flag & nf_ABSOLUTE: absolute polynomial */
-GEN
-rnfpolredabs(GEN nf, GEN relpol, long flag)
-{
-  pari_timer ti;
-  GEN red, bas, elt, pol, T, a;
-  long fl = (flag & nf_ADDZK)? nf_ADDZK: nf_RAW;
-  pari_sp av = avma;
-
-  if (typ(relpol)!=t_POL) pari_err(typeer,"rnfpolredabs");
-  nf = checknf(nf);
-  if (DEBUGLEVEL>1) timer_start(&ti);
-  T = nf_get_pol(nf);
-  relpol = rnf_fix_pol(T, relpol, 0);
-  if ((flag & nf_ADDZK) && !(flag & nf_ABSOLUTE))
-    pari_err(impl,"this combination of flags in rnfpolredabs");
-  if (flag & nf_PARTIALFACT)
-  {
-    long sa;
-    fl |= nf_PARTIALFACT;
-    bas = rnfequationall(nf, relpol, &sa, NULL);
-    a = stoi(sa);
-  }
-  else
-  {
-    GEN eq = rnfequation2(nf,relpol), rel;
-    a = gel(eq,3);
-    /* relpol( X + Mod(-a y, T(y)) )*/
-    rel = RgXQX_translate(relpol, deg1pol_shallow(negi(a),gen_0,varn(T)), T);
-    bas = makebasis(nf, rel, eq);
-    if (DEBUGLEVEL>1)
-    {
-      timer_printf(&ti, "absolute basis");
-      err_printf("original absolute generator: %Ps\n", gel(eq,1));
-    }
-  }
-  red = polredabs0(bas, fl);
-  pol = gel(red,1);
-  if (DEBUGLEVEL>1) err_printf("reduced absolute generator: %Ps\n",pol);
-  if (flag & nf_ABSOLUTE)
-    return gerepilecopy(av, (flag & nf_ADDZK)? red: pol);
-
-  elt = RgXQX_translate(gel(red,2), deg1pol_shallow(a,gen_0,varn(T)), T);
-  elt = rnf_fix_pol(T, elt, 0);
-  pol = RgXQ_charpoly(elt, relpol, varn(relpol));
-  pol = lift_if_rational(pol);
-  if (flag & nf_ORIG) pol = mkvec2(pol, mkpolmod(RgXQ_reverse(elt,relpol),pol));
-  return gerepilecopy(av, pol);
-}
