@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -64,20 +62,14 @@ listconcat(GEN A, GEN B)
 static GEN
 strconcat(GEN x, GEN y)
 {
-  int flx = 0, fly = 0;
   size_t l, lx;
-  char *sx,*sy,*str;
-
-  if (typ(x)==t_STR) sx = GSTR(x); else { flx=1; sx = GENtostr(x); }
-  if (typ(y)==t_STR) sy = GSTR(y); else { fly=1; sy = GENtostr(y); }
+  char *sx = GENtostr_unquoted(x);
+  char *sy = GENtostr_unquoted(y), *str;
   lx = strlen(sx);
   l = nchar2nlong(lx + strlen(sy) + 1);
   x = cgetg(l + 1, t_STR); str = GSTR(x);
   strcpy(str,   sx);
-  strcpy(str+lx,sy);
-  if (flx) pari_free(sx);
-  if (fly) pari_free(sy);
-  return x;
+  strcpy(str+lx,sy); return x;
 }
 
 /* concat A and B vertically. Internal */
@@ -90,9 +82,9 @@ vconcat(GEN A, GEN B)
   if (!A) return B;
   if (!B) return A;
   la = lg(A); if (la==1) return A;
-  T = typ(A[1]); /* t_COL or t_VECSMALL */
-  ha = lg(A[1]); M = cgetg(la,t_MAT);
-  hb = lg(B[1]); hc = ha+hb-1;
+  T = typ(gel(A,1)); /* t_COL or t_VECSMALL */
+  ha = lgcols(A); M = cgetg(la,t_MAT);
+  hb = lgcols(B); hc = ha+hb-1;
   for (j=1; j<la; j++)
   {
     c = cgetg(hc, T); gel(M, j) = c;
@@ -105,11 +97,7 @@ vconcat(GEN A, GEN B)
 }
 
 static void
-err_cat(GEN x, GEN y)
-{
-  pari_err(talker,"impossible concatenation: %s %Ps . %s %Ps",
-      type_name(typ(x)), matsize(x), type_name(typ(y)), matsize(y));
-}
+err_cat(GEN x, GEN y) { pari_err_OP("concatenation",x,y); }
 
 GEN
 shallowconcat(GEN x, GEN y)
@@ -136,7 +124,7 @@ shallowconcat(GEN x, GEN y)
   if (tx == ty)
   {
     if (tx == t_MAT)
-    { if (lg(x[1]) != lg(y[1])) err_cat(x,y); }
+    { if (lgcols(x) != lgcols(y)) err_cat(x,y); }
     else
       if (!is_matvec_t(tx) && tx != t_VECSMALL) return mkvec2(x, y);
     z=cgetg(lx+ly-1,tx);
@@ -152,7 +140,7 @@ shallowconcat(GEN x, GEN y)
     if (ty != t_MAT) p1 = x;
     else
     {
-      if (lg(y[1])!=2) err_cat(x,y);
+      if (lgcols(y)!=2) err_cat(x,y);
       p1 = mkcol(x);
     }
     for (i=2; i<=ly; i++) z[i] = y[i-1];
@@ -164,7 +152,7 @@ shallowconcat(GEN x, GEN y)
     if (tx != t_MAT) p1 = y;
     else
     {
-      if (lg(x[1])!=2) err_cat(x,y);
+      if (lgcols(x)!=2) err_cat(x,y);
       p1 = mkcol(y);
     }
     for (i=1; i<lx; i++) z[i]=x[i];
@@ -195,9 +183,9 @@ shallowconcat(GEN x, GEN y)
           if (ly>=3) break;
           return (ly==1)? x: shallowconcat(x, gel(y,1));
         case t_MAT:
-          if (lx != lg(y[1])) break;
-          z=cgetg(ly+1,t_MAT); gel(z,1) = x;
-          for (i=2; i<=ly; i++) z[i]=y[i-1];
+          if (lx != lgcols(y)) break;
+          z=cgetg(ly+1,t_MAT);  gel(z,1) = x;
+          for (i=2; i<=ly; i++) gel(z,i) = gel(y,i-1);
           return z;
       }
       break;
@@ -210,7 +198,7 @@ shallowconcat(GEN x, GEN y)
           for (i=1; i<lx; i++) gel(z,i) = shallowconcat(gel(x,i), gel(y,i));
           return z;
         case t_COL:
-          if (ly != lg(x[1])) break;
+          if (ly != lgcols(x)) break;
           z=cgetg(lx+1,t_MAT); gel(z,lx) = y;
           for (i=1; i<lx; i++) z[i]=x[i];
           return z;
@@ -232,42 +220,39 @@ catmanyMAT(GEN y1, GEN y2)
     GEN c = gel(y,0);
     long nc = lg(c)-1;
     if (nc == 0) continue;
-    if (h != lg(c[1]))
+    if (h != lgcols(c))
     {
       if (h) err_cat(gel(y2,0), c);
-      h = lg(c[1]);
+      h = lgcols(c);
     }
     L += nc;
     z = new_chunk(nc) - 1;
-    for (i=1; i<=nc; i++) z[i] = c[i];
+    for (i=1; i<=nc; i++) gel(z,i) = gel(c,i);
   }
   z = new_chunk(1);
   *z = evaltyp(t_MAT) | evallg(L);
   return z;
 }
-/* see catmany() */
 static GEN
 catmanySTR(GEN y1, GEN y2)
 {
-  long i, L = 1;
+  long L = 1; /* final \0 */
   GEN z, y;
-  char *s, *S = (char*)avma;
-  pari_sp av = avma;
-  (void)new_chunk(1); *--S = 0;
-  for (y = y2; y >= y1; y--)
+  char *s;
+  for (y = y1; y <= y2; y++)
+  {
+    char *c = GSTR( gel(y,0) );
+    L += strlen(c);
+  }
+  z = cgetg(nchar2nlong(L)+1, t_STR);
+  s = GSTR(z);
+  for (y = y1; y <= y2; y++)
   {
     char *c = GSTR( gel(y,0) );
     long nc = strlen(c);
-    if (nc == 0) continue;
-    L += nc; c += nc;
-    (void)new_chunk(nchar2nlong(nc));
-    for (i=1; i<=nc; i++) *--S = *--c;
+    if (nc) { (void)strncpy(s, c, nc); s += nc; }
   }
-  avma = av;
-  z = cgetg(nchar2nlong(L) + 1, t_STR);
-  s = GSTR(z);
-  if (S != s) { for (i = 0; i <= L; i++) *s++ = *S++; }
-  return z;
+  *s = 0; return z;
 }
 
 /* all entries in y have the same type t = t_VEC, COL, MAT or VECSMALL
@@ -288,7 +273,7 @@ catmany(GEN y1, GEN y2, long t)
     if (nc == 0) continue;
     L += nc;
     z = new_chunk(nc) - 1;
-    for (i=1; i<=nc; i++) z[i] = c[i];
+    for (i=1; i<=nc; i++) gel(z,i) = gel(c,i);
   }
   z = new_chunk(1);
   *z = evaltyp(t) | evallg(L);
@@ -299,20 +284,24 @@ GEN
 shallowconcat1(GEN x)
 {
   pari_sp av = avma, lim = stack_lim(av, 3);
-  long tx = typ(x), lx, t, i;
+  long lx, t, i;
   GEN z;
-
-  if      (tx == t_VEC) lx = lg(x);
-  else if (tx == t_LIST)
-  { x = list_data(x); lx = x ? lg(x): 1; }
-  else
-  { pari_err(typeer,"concat"); return NULL; /* not reached */ }
-  if (lx==1) pari_err(talker,"trying to concat elements of an empty vector");
+  switch(typ(x))
+  {
+    case t_VEC:
+      lx = lg(x);
+      if (lx==1) pari_err_DOMAIN("concat","vector","=",x,x);
+      break;
+    case t_LIST:
+      if (!list_data(x)) pari_err_DOMAIN("concat","vector","=",x,x);
+      x = list_data(x); lx = lg(x);
+      break;
+    default:
+      pari_err_TYPE("concat",x);
+      return NULL; /* not reached */
+  }
   if (lx==2) return gel(x,1);
-
-  z = gel(x,1);
-  t = typ(z);
-  i = 2;
+  z = gel(x,1); t = typ(z); i = 2;
   if (is_matvec_t(t) || t == t_VECSMALL || t == t_STR)
   { /* detect a "homogeneous" object: catmany is faster */
     for (; i<lx; i++)
@@ -337,6 +326,132 @@ concat1(GEN x)
   return gerepilecopy(av, shallowconcat1(x));
 }
 
+/* fill M[xoff+i, yoff+j] with the contents of c ( c * Id_n if scalar ) */
+static void
+matfill(GEN M, GEN c, long xoff, long yoff, long n)
+{
+  long i, j, h, l;
+  l = lg(c); if (l == 1) return;
+  switch(typ(c))
+  {
+    case t_VEC:
+      for (i = 1; i < l; i++)
+        gcoeff(M,xoff+1,yoff+i) = gel(c,i);
+      break;
+    case t_COL:
+      for (i = 1; i < l; i++)
+        gcoeff(M,xoff+i,yoff+1) = gel(c,i);
+      break;
+    case t_MAT:
+      h = lgcols(c);
+      for (j = 1; j < l; j++)
+        for (i = 1; i < h; i++) gcoeff(M,xoff+i,yoff+j) = gcoeff(c,i,j);
+      break;
+    default:
+      for (i = 1; i <= n; i++)
+        gcoeff(M, xoff+i, yoff+i) = c;
+      break;
+  }
+}
+
+static GEN
+_matsize(GEN x)
+{
+  long t = typ(x), L = lg(x) - 1;
+  switch(t)
+  { /* matsize */
+    case t_VEC: return mkvecsmall2(1, L);
+    case t_COL: return mkvecsmall2(L, 1);
+    case t_MAT: return mkvecsmall2(L? nbrows(x): 0, L);
+    default:
+      if (is_noncalc_t(t)) pari_err_TYPE("_matsize", x);
+      return mkvecsmall2(1, 1);
+  }
+}
+
+GEN
+shallowmatconcat(GEN v)
+{
+  long i, j, h, l = lg(v), L = 0, H = 0;
+  GEN M, maxh, maxl;
+  if (l == 1) return cgetg(1,t_MAT);
+  switch(typ(v))
+  {
+    case t_VEC:
+      for (i = 1; i < l; i++)
+      {
+        GEN c = gel(v,i);
+        GEN s = _matsize(c);
+        H = maxss(H, s[1]);
+        L += s[2];
+      }
+      M = zeromatcopy(H, L);
+      L = 0;
+      for (i = 1; i < l; i++)
+      {
+        GEN c = gel(v,i);
+        GEN s = _matsize(c);
+        matfill(M, c, 0, L, 1);
+        L += s[2];
+      }
+      return M;
+
+    case t_COL:
+      for (i = 1; i < l; i++)
+      {
+        GEN c = gel(v,i);
+        GEN s = _matsize(c);
+        H += s[1];
+        L = maxss(L, s[2]);
+      }
+      M = zeromatcopy(H, L);
+      H = 0;
+      for (i = 1; i < l; i++)
+      {
+        GEN c = gel(v,i);
+        GEN s = _matsize(c);
+        matfill(M, c, H, 0, 1);
+        H += s[1];
+      }
+      return M;
+    case t_MAT:
+      h = lgcols(v);
+      maxh = zero_zv(h-1);
+      maxl = zero_zv(l-1);
+      for (j = 1; j < l; j++)
+        for (i = 1; i < h; i++)
+        {
+          GEN c = gcoeff(v,i,j);
+          GEN s = _matsize(c);
+          if (s[1] > maxh[i]) maxh[i] = s[1];
+          if (s[2] > maxl[j]) maxl[j] = s[2];
+        }
+      for (i = 1, H = 0; i < h; i++) H += maxh[i];
+      for (j = 1, L = 0; j < l; j++) L += maxl[j];
+      M = zeromatcopy(H, L);
+      for (j = 1, L = 0; j < l; j++)
+      {
+        for (i = 1, H = 0; i < h; i++)
+        {
+          GEN c = gcoeff(v,i,j);
+          matfill(M, c, H, L, minss(maxh[i], maxl[j]));
+          H += maxh[i];
+        }
+        L += maxl[j];
+      }
+      return M;
+    default:
+      pari_err_TYPE("shallowmatconcat", v);
+      return NULL;
+  }
+}
+GEN
+matconcat(GEN v)
+{
+  pari_sp av = avma;
+  return gerepilecopy(av, shallowmatconcat(v));
+}
+
 GEN
 concat(GEN x, GEN y)
 {
@@ -346,7 +461,11 @@ concat(GEN x, GEN y)
   if (!y) return concat1(x);
   tx = typ(x);
   ty = typ(y);
-  if (tx==t_STR  || ty==t_STR)  return strconcat(x,y);
+  if (tx==t_STR  || ty==t_STR)
+  {
+    pari_sp av = avma;
+    return gerepileuptoleaf(av, strconcat(x,y));
+  }
   if (tx==t_LIST || ty==t_LIST) return listconcat(x,y);
   lx=lg(x); ly=lg(y);
 
@@ -365,7 +484,7 @@ concat(GEN x, GEN y)
 
   if (tx == ty)
   {
-    if (tx == t_MAT && lg(x[1]) != lg(y[1])) err_cat(x,y);
+    if (tx == t_MAT && lgcols(x) != lgcols(y)) err_cat(x,y);
     if (!is_matvec_t(tx))
     {
       if (tx != t_VECSMALL) return mkvec2copy(x, y);
@@ -387,7 +506,7 @@ concat(GEN x, GEN y)
     if (ty != t_MAT) p1 = gcopy(x);
     else
     {
-      if (lg(y[1])!=2) err_cat(x,y);
+      if (lgcols(y)!=2) err_cat(x,y);
       p1 = mkcolcopy(x);
     }
     for (i=2; i<=ly; i++) gel(z,i) = gcopy(gel(y,i-1));
@@ -399,7 +518,7 @@ concat(GEN x, GEN y)
     if (tx != t_MAT) p1 = gcopy(y);
     else
     {
-      if (lg(x[1])!=2) err_cat(x,y);
+      if (lgcols(x)!=2) err_cat(x,y);
       p1 = mkcolcopy(y);
     }
     for (i=1; i<lx; i++) gel(z,i) = gcopy(gel(x,i));
@@ -430,7 +549,7 @@ concat(GEN x, GEN y)
           if (ly>=3) break;
           return (ly==1)? gcopy(x): concat(x,gel(y,1));
         case t_MAT:
-          if (lx != lg(y[1])) break;
+          if (lx != lgcols(y)) break;
           z=cgetg(ly+1,t_MAT); gel(z,1) = gcopy(x);
           for (i=2; i<=ly; i++) gel(z,i) = gcopy(gel(y,i-1));
           return z;
@@ -445,7 +564,7 @@ concat(GEN x, GEN y)
           for (i=1; i<lx; i++) gel(z,i) = concat(gel(x,i),gel(y,i));
           return z;
         case t_COL:
-          if (ly != lg(x[1])) break;
+          if (ly != lgcols(x)) break;
           z=cgetg(lx+1,t_MAT); gel(z,lx) = gcopy(y);
           for (i=1; i<lx; i++) gel(z,i) = gcopy(gel(x,i));
           return z;
