@@ -342,7 +342,8 @@ initgaloisborne(GEN T, GEN dn, long prec, GEN *ptL, GEN *ptprep, GEN *ptdis)
   if (!dn)
   {
     GEN dis, res = divide_conquer_prod(gabs(prep,prec), mpmul);
-    dis = ZX_disc_all(T, expi(ceil_safe(res)));
+    /*Add +2 to cater for accuracy error in res */
+    dis = ZX_disc_all(T, 2+expi(ceil_safe(res)));
     den = indexpartial(T,dis);
     if (ptdis) *ptdis = dis;
   }
