@@ -1183,8 +1183,16 @@ primes_interval(GEN a, GEN b)
   forprime_t S;
   long i, n;
   GEN y, d, p;
-  if (typ(a) != t_INT) a = gceil(a);
-  if (typ(b) != t_INT) b = gfloor(b);
+  if (typ(a) != t_INT)
+  {
+    a = gceil(a);
+    if (typ(a) != t_INT) pari_err_TYPE("primes_interval",a);
+  }
+  if (typ(b) != t_INT)
+  {
+    b = gfloor(b);
+    if (typ(b) != t_INT) pari_err_TYPE("primes_interval",b);
+  }
   if (signe(a) < 0) a = gen_2;
   d = subii(b, a);
   if (signe(d) < 0 || signe(b) <= 0) { avma = av; return cgetg(1, t_VEC); }
