@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -23,6 +21,7 @@ BEGINEXTERN
 void aide(const char *s, long flag);
 void echo_and_log(const char *prompt, const char *s);
 int  get_line_from_readline(const char *prompt, const char *prompt_cont, filtre_t *F);
+void readline_prompt_color(char *s, int c);
 void gp_output(GEN z, gp_data *G);
 void init_readline(void);
 void texmacs_completion(const char *s, long pos);
@@ -44,11 +43,17 @@ extern const char *keyword_list[];
 
 /* gp specific routines */
 void alarm0(long s);
+void dbg_down(long k);
+GEN  dbg_err(void);
+void dbg_up(long k);
 GEN  extern0(const char *cmd);
 GEN  externstr(const char *cmd);
+GEN  gp_alarm(long s, GEN code);
 void gp_quit(long exitcode);
 GEN  input0(void);
+void pari_breakpoint(void);
 GEN  read0(const char *s);
+GEN  readstr(const char *s);
 void system0(const char *cmd);
 int  whatnow(PariOUT *out, const char *s, int silent);
 GEN sd_breakloop(const char *v, long flag);
@@ -58,6 +63,7 @@ GEN sd_graphcolors(const char *v, long flag);
 GEN sd_help(const char *v, long flag);
 GEN sd_histfile(const char *v, long flag);
 GEN sd_lines(const char *v, long flag);
+GEN sd_linewrap(const char *v, long flag);
 GEN sd_prompt(const char *v, long flag);
 GEN sd_prompt_cont(const char *v, long flag);
 GEN sd_psfile(const char *v, long flag);
