@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -49,14 +47,18 @@ struct data_x
   GC gc;
 };
 
+/* after fork(), we don't want the child to recover but to exit */
+static void
+exiterr(const char *str)
+{
+  term_color(c_ERR);
+  err_printf("\n  *** X fatal error: %s\n",str);
+  term_color(c_NONE); exit(1);
+}
+
 static void SetForeground(void *data, long col)
 {
   struct data_x *dx = (struct data_x *) data;
-  if (col >= dx->numcolors)
-  {
-    pari_warn(warner,"non-existent color: %ld", col);
-    col = dx->numcolors-1;
-  }
   XSetForeground(dx->display,dx->gc, PARI_Colors[col].pixel);
 }
 
@@ -112,39 +114,6 @@ static void DrawString(void *data, long x, long y, char *text, long numtext)
   XDrawString(dx->display,dx->win,dx->gc, x,y, text, numtext);
 }
 
-static void
-PARI_ColorSetUp(Display *display, GEN colors)
-{
-  static int init_done = 0;
-  long i, n = lg(colors)-1;
-
-  if (init_done) return;
-  init_done=1;
-
-  PARI_Colormap = DefaultColormap(display, 0);
-  PARI_Colors = (XColor *) pari_malloc((n+1) * sizeof(XColor));
-  for (i=0; i<n; i++)
-  {
-    int r, g, b;
-    color_to_rgb(gel(pari_colormap,i+1), &r, &g, &b);
-    PARI_Colors[i].red   = r*65535/255;
-    PARI_Colors[i].green = g*65535/255;
-    PARI_Colors[i].blue  = b*65535/255;
-    PARI_Colors[i].flags = DoRed | DoGreen | DoBlue;
-    if (!XAllocColor(display, PARI_Colormap, &PARI_Colors[i]))
-      pari_err(talker, "cannot allocate color %ld", i);
-  }
-}
-
-/* after fork(), we don't want the child to recover but to exit */
-static void
-exiterr(const char *str)
-{
-  term_color(c_ERR);
-  err_printf("\n  *** X fatal error: %s\n",str);
-  term_color(c_NONE); exit(1);
-}
-
 #define MAX_BUF 256
 
 static int
@@ -161,6 +130,30 @@ IOerror(Display *d) {
   exiterr(buf); return 0;
 }
 
+static void
+PARI_ColorSetUp(Display *display, GEN colors)
+{
+  static int init_done = 0;
+  long i, n = lg(colors)-1;
+
+  if (init_done) return;
+  init_done=1;
+
+  PARI_Colormap = DefaultColormap(display, 0);
+  PARI_Colors = (XColor *) pari_malloc((n+1) * sizeof(XColor));
+  for (i=0; i<n; i++)
+  {
+    int r, g, b;
+    color_to_rgb(gel(colors,i+1), &r, &g, &b);
+    PARI_Colors[i].red   = r*65535/255;
+    PARI_Colors[i].green = g*65535/255;
+    PARI_Colors[i].blue  = b*65535/255;
+    PARI_Colors[i].flags = DoRed | DoGreen | DoBlue;
+    if (!XAllocColor(display, PARI_Colormap, &PARI_Colors[i]))
+      exiterr("cannot allocate color");
+  }
+}
+
 void
 rectdraw0(long *w, long *x, long *y, long lw)
 {
@@ -180,8 +173,8 @@ rectdraw0(long *w, long *x, long *y, long lw)
 
   if (pari_daemon()) return;  /* parent process returns */
 
+  PARI_get_plot();
   pari_close();
-  PARI_get_plot(1);
 
   display = XOpenDisplay(NULL);
   font_info = XLoadQueryFont(display, "9x15");
@@ -235,6 +228,7 @@ rectdraw0(long *w, long *x, long *y, long lw)
   plotX.ml = &DrawLines;
   plotX.st = &DrawString;
   plotX.pl = &pari_plot;
+  plotX.data = (void*)&dx;
 
   for(;;)
   {
@@ -279,23 +273,19 @@ EXIT:
         ys = ((double)height)/pari_plot.height;
       }
       case Expose:
-        gen_rectdraw0(&plotX, (void *)&dx, w, x, y,lw,xs,ys);
+        gen_rectdraw0(&plotX, w, x, y,lw,xs,ys);
     }
   }
 }
 
 void
-PARI_get_plot(long fatal)
+PARI_get_plot()
 {
   Display *display;
   int screen;
 
   if (pari_plot.init) return;
-  if (!(display = XOpenDisplay(NULL)))
-  {
-    if (fatal) exiterr("no X server");
-    pari_err(talker, "no X server");
-  }
+  if (!(display = XOpenDisplay(NULL))) pari_err(e_MISC, "no X server");
   screen = DefaultScreen(display);
   pari_plot.width  = DisplayWidth(display, screen) - 40;
   pari_plot.height = DisplayHeight(display, screen) - 60;
@@ -306,4 +296,3 @@ PARI_get_plot(long fatal)
   pari_plot.init = 1;
   XCloseDisplay(display);
 }
-
