@@ -94,7 +94,7 @@ typedef long *GEN;
 
 #define evaltyp(x)    (((ulong)(x)) << TYPSHIFT)
 #define evalvarn(x)   (((ulong)(x)) << VARNSHIFT)
-#define evalsigne(x)  ((ulong)(((long)(x)) << SIGNSHIFT))
+#define evalsigne(x)  (((ulong)(x)) << SIGNSHIFT)
 #define _evalexpo(x)  (HIGHEXPOBIT + (x))
 #define _evalvalp(x)  (HIGHVALPBIT + (x))
 #define _evalprecp(x) (((long)(x)) << PRECPSHIFT)
