@@ -1041,7 +1041,8 @@ red_montgomery(GEN T, GEN N, ulong inv)
 
 /* EXACT INTEGER DIVISION */
 
-#if 1 /* use undocumented GMP interface */
+#if 1 && !defined(_WIN64) /* use undocumented GMP interface */
+                          /* mpz_divexact_ui is not LLP64 friendly */
 static void
 GEN2mpz(mpz_t X, GEN x)
 {
