@@ -1,7 +1,5 @@
 #line 2 "../src/kernel/none/add.c"
-/* $Id$
-
-Copyright (C) 2002-2003  The PARI group.
+/* Copyright (C) 2002-2003  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -55,6 +53,34 @@ addsi_sign(long x, GEN y, long sy)
   z = subiuspec(y+2,x, ly-2);
   setsigne(z,sy); return z;
 }
+GEN
+addui_sign(ulong x, GEN y, long sy)
+{
+  long ly;
+  GEN z;
+
+  if (!x) return icopy_sign(y, sy);
+  if (!sy) return utoipos(x);
+  if (sy == 1) return adduispec(x,y+2, lgefint(y)-2);
+  ly=lgefint(y);
+  if (ly==3)
+  {
+    const ulong t = y[2];
+    if (x == t) return gen_0;
+    z=cgeti(3);
+    if (x < t) {
+      z[1] = evalsigne(-1) | evallgefint(3);
+      z[2] = t - x;
+    }
+    else {
+      z[1] = evalsigne(1) | evallgefint(3);
+      z[2] = x - t;
+    }
+    return z;
+  }
+  z = subiuspec(y+2,x, ly-2);
+  setsigne(z,-1); return z;
+}
 
 /* return gen_0 when the sign is 0 */
 GEN
@@ -65,20 +91,21 @@ addii_sign(GEN x, long sx, GEN y, long sy)
 
   if (!sx) return sy? icopy_sign(y, sy): gen_0;
   if (!sy) return icopy_sign(x, sx);
-  lx=lgefint(x); ly=lgefint(y);
-
+  lx = lgefint(x);
+  ly = lgefint(y);
   if (sx==sy)
     z = addiispec(x+2,y+2,lx-2,ly-2);
   else
   { /* sx != sy */
-    long i = lx - ly;
-    if (i==0) /* lx == ly */
-    {
-      i = absi_cmp_lg(x,y,lx);
-      if (!i) return gen_0;
+    long i = cmpiispec(x+2,y+2,lx-2,ly-2);
+    if (!i) return gen_0;
+    /* we must ensure |x| > |y| for subiispec */
+    if (i < 0) {
+      sx = sy;
+      z = subiispec(y+2,x+2,ly-2,lx-2);
     }
-    if (i<0) { sx=sy; swapspec(x,y, lx,ly); } /* ensure |x| >= |y| */
-    z = subiispec(x+2,y+2,lx-2,ly-2);
+    else
+      z = subiispec(x+2,y+2,lx-2,ly-2);
   }
   setsigne(z,sx); return z;
 }
@@ -107,7 +134,7 @@ addir_sign(GEN x, long sx, GEN y, long sy)
     l = ly - divsBIL(e);
     if (l < 3) return rcopy_sign(y, sy);
   }
-  else l = ly + nbits2nlong(-e);
+  else l = ly + nbits2extraprec(-e);
   z = (GEN)avma;
   y = addrr_sign(itor(x,l), sx, y, sy);
   ly = lg(y); while (ly--) *--z = y[ly];
@@ -136,7 +163,7 @@ addsr_sign(long x, GEN y, long sy)
     l = ly - divsBIL(e);
     if (l < 3) return rcopy_sign(y, sy);
   }
-  else l = ly + nbits2nlong(-e);
+  else l = ly + nbits2extraprec(-e);
   z = (GEN)avma;
   y = addrr_sign(stor(x,l), sx, y, sy);
   ly = lg(y); while (ly--) *--z = y[ly];
@@ -271,7 +298,7 @@ addrr_sign(GEN x, long sx, GEN y, long sy)
     if (overflow)
     {
       z[1] = 1; /* stops since z[1] != 0 */
-      for (;;) { z[i] = y[i]+1; if (z[i--]) break; }
+      for (;;) { z[i] = (ulong) y[i]+1; if (z[i--]) break; }
       if (i <= 0)
       {
         shift_right(z,z, 2,lz, 1,1);
@@ -299,7 +326,7 @@ addrr_sign(GEN x, long sx, GEN y, long sy)
     else        z[i] = subll(y[i], x[j--]);
     for (i--; j>=2; i--) z[i] = subllx(y[i], x[j--]);
     if (overflow) /* stops since y[1] != 0 */
-      for (;;) { z[i] = y[i]-1; if (y[i--]) break; }
+      for (;;) { z[i] = (ulong) y[i]-1; if (y[i--]) break; }
     for (; i>=2; i--) z[i] = y[i];
     sx = sy;
   }
@@ -334,7 +361,7 @@ addrr_sign(GEN x, long sx, GEN y, long sy)
       if ((last<<j) & HIGHBIT)
       { /* round up */
         i = lz-1;
-        while (++z[i] == 0 && i > 1) i--;
+        while (++((ulong*)z)[i] == 0 && i > 1) i--;
         if (i == 1) { ez++; z[2] = (long)HIGHBIT; }
       }
     }
