@@ -1,7 +1,5 @@
 #line 2 "../src/kernel/none/level1.h"
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -21,21 +19,28 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 INLINE long
 evallg(long x)
 {
-  if (x & ~LGBITS) pari_err(overflower,"lg()");
+  if (x & ~LGBITS) pari_err_OVERFLOW("lg()");
   return _evallg(x);
 }
 INLINE long
 evalvalp(long x)
 {
   long v = _evalvalp(x);
-  if (v & ~VALPBITS) pari_err(overflower,"valp()");
+  if (v & ~VALPBITS) pari_err_OVERFLOW("valp()");
   return v;
 }
 INLINE long
 evalexpo(long x)
 {
   long v = _evalexpo(x);
-  if (v & ~EXPOBITS) pari_err(overflower,"expo()");
+  if (v & ~EXPOBITS) pari_err_OVERFLOW("expo()");
+  return v;
+}
+INLINE long
+evalprecp(long x)
+{
+  long v = _evalprecp(x);
+  if (x & ~((1UL<<(BITS_IN_LONG-VALPnumBITS))-1)) pari_err_OVERFLOW("precp()");
   return v;
 }
 
@@ -83,7 +88,7 @@ INLINE GEN
 new_chunk(size_t x) /* x is a number of longs */
 {
   GEN z = ((GEN) avma) - x;
-  if (x > (avma-bot) / sizeof(long)) pari_err(errpile);
+  if (x > (avma-bot) / sizeof(long)) pari_err(e_STACK);
   CHECK_CTRLC
   avma = (pari_sp)z;
 
@@ -100,6 +105,20 @@ new_chunk(size_t x) /* x is a number of longs */
   return z;
 }
 
+INLINE char *
+stack_malloc(size_t N)
+{
+  long n = nchar2nlong(N);
+  return (char*)new_chunk(n);
+}
+
+INLINE char *
+stack_calloc(size_t N)
+{
+  char *p = stack_malloc(N);
+  memset(p, 0, N); return p;
+}
+
 /* cgetg(lg(x), typ(x)), set *lx. Implicit unsetisclone() */
 INLINE GEN
 cgetg_copy(GEN x, long *plx) {
@@ -108,6 +127,13 @@ cgetg_copy(GEN x, long *plx) {
   y[0] = x[0] & (TYPBITS|LGBITS); return y;
 }
 INLINE GEN
+cgetg_block(long x, long y)
+{
+  GEN z = newblock((size_t)x);
+  z[0] = evaltyp(y) | evallg(x);
+  return z;
+}
+INLINE GEN
 cgetg(long x, long y)
 {
   GEN z = new_chunk((size_t)x);
@@ -136,6 +162,13 @@ cgetineg(long x)
   return z;
 }
 INLINE GEN
+cgetr_block(long x)
+{
+  GEN z = newblock((size_t)x);
+  z[0] = evaltyp(t_REAL) | evallg(x);
+  return z;
+}
+INLINE GEN
 cgetr(long x)
 {
   GEN z = new_chunk((size_t)x);
@@ -152,17 +185,28 @@ cgetr(long x)
 INLINE GEN
 leafcopy(GEN x)
 {
-  long lx;
-  GEN y = cgetg_copy(x, &lx);
+  register long lx = lg(x);
+  GEN y = new_chunk(lx); /* can't use cgetg_copy, in case x,y overlap */
   while (--lx > 0) y[lx] = x[lx];
-  return y;
+  y[0] = x[0] & (TYPBITS|LGBITS); return y;
 }
 INLINE GEN
 icopy(GEN x)
 {
-  register long lx = lgefint(x);
-  GEN y = cgeti(lx);
-  while (--lx > 0) y[lx] = x[lx];
+  long i = lgefint(x), lx = i;
+  GEN y = new_chunk(lx); /* can't use cgeti, in case x,y overlap */
+  while (--i > 0) y[i] = x[i];
+  y[0] = evaltyp(t_INT) | evallg(lx);
+  return y;
+}
+INLINE GEN
+icopyspec(GEN x, long nx)
+{
+  long i = nx+2, lx = i;
+  GEN y = new_chunk(lx); /* can't use cgeti, in case x,y overlap */
+  x -= 2; while (--i >= 2) y[i] = x[i];
+  y[1] = evalsigne(1) | evallgefint(lx);
+  y[0] = evaltyp(t_INT) | evallg(lx);
   return y;
 }
 INLINE GEN rcopy(GEN x) { return leafcopy(x); }
@@ -170,7 +214,10 @@ INLINE GEN mpcopy(GEN x) { return leafcopy(x); }
 
 INLINE GEN
 mpabs(GEN x) { GEN y = leafcopy(x); setabssign(y); return y; }
+INLINE GEN
+mpabs_shallow(GEN x) { return signe(x) < 0? mpabs(x): x; }
 INLINE GEN absi(GEN x) { return mpabs(x); }
+INLINE GEN absi_shallow(GEN x) { return signe(x) < 0? negi(x): x; }
 INLINE GEN absr(GEN x) { return mpabs(x); }
 
 INLINE GEN
@@ -258,7 +305,7 @@ itos(GEN x)
   if (!s) return 0;
   u = x[2];
   if (lgefint(x) > 3 || u < 0)
-    pari_err(overflower,"t_INT-->long assignment");
+    pari_err_OVERFLOW("t_INT-->long assignment");
   return (s>0) ? u : -u;
 }
 /* as itos, but return 0 if too large. Cf is_bigint */
@@ -275,7 +322,7 @@ itou(GEN x)
     case 2: return 0;
     case 3: return x[2];
     default:
-      pari_err(overflower,"t_INT-->ulong assignment");
+      pari_err_OVERFLOW("t_INT-->ulong assignment");
       return 0; /* not reached */
   }
 }
@@ -290,7 +337,7 @@ itou_or_0(GEN x) {
 INLINE GEN
 real_0_bit(long bitprec) { GEN x=cgetr(2); x[1]=evalexpo(bitprec); return x; }
 INLINE GEN
-real_0(long prec) { return real_0_bit(-bit_accuracy(prec)); }
+real_0(long prec) { return real_0_bit(-prec2nbits(prec)); }
 INLINE GEN
 real_1(long prec) {
   GEN x = cgetr(prec);
@@ -312,6 +359,8 @@ real_m1(long prec) {
 INLINE GEN
 real2n(long n, long prec) { GEN z = real_1(prec); setexpo(z, n); return z; }
 INLINE GEN
+real_m2n(long n, long prec) { GEN z = real_m1(prec); setexpo(z, n); return z; }
+INLINE GEN
 stor(long s, long prec) { GEN z = cgetr(prec); affsr(s,z); return z; }
 INLINE GEN
 utor(ulong s, long prec){ GEN z = cgetr(prec); affur(s,z); return z; }
@@ -320,6 +369,12 @@ itor(GEN x, long prec) { GEN z = cgetr(prec); affir(x,z); return z; }
 INLINE GEN
 rtor(GEN x, long prec) { GEN z = cgetr(prec); affrr(x,z); return z; }
 
+INLINE ulong int_bit(GEN x, long n)
+{
+  long r, q = dvmdsBIL(n, &r);
+  return q < lgefint(x)-2?((ulong)*int_W(x,q) >> r) & 1UL:0;
+}
+
 /*******************************************************************/
 /*                                                                 */
 /*                           COMPARISON                            */
@@ -333,7 +388,7 @@ cmpir(GEN x, GEN y)
 
   if (!signe(x)) return -signe(y);
   if (!signe(y)) return  signe(x);
-  av=avma; z = itor(x, lg(y)); avma=av;
+  av=avma; z = itor(x, realprec(y)); avma=av;
   return cmprr(z,y); /* cmprr does no memory adjustment */
 }
 INLINE int
@@ -345,7 +400,7 @@ cmpsr(long x, GEN y)
   GEN z;
 
   if (!x) return -signe(y);
-  av=avma; z = stor(x, 3); avma=av;
+  av=avma; z = stor(x, LOWDEFAULTPREC); avma=av;
   return cmprr(z,y);
 }
 INLINE int
@@ -500,7 +555,11 @@ subri(GEN x, GEN y) { return addir_sign(y, -signe(y), x, signe(x)); }
 INLINE GEN
 addsi(long x, GEN y) { return addsi_sign(x, y, signe(y)); }
 INLINE GEN
+addui(ulong x, GEN y) { return addui_sign(x, y, signe(y)); }
+INLINE GEN
 subsi(long x, GEN y) { return addsi_sign(x, y, -signe(y)); }
+INLINE GEN
+subui(ulong x, GEN y) { return addui_sign(x, y, -signe(y)); }
 
 /*******************************************************************/
 /*                                                                 */
@@ -547,7 +606,7 @@ sdivss_rem(long x, long y, long *r)
 {
   long q;
   LOCAL_HIREMAINDER;
-  if (!y) pari_err(gdiver);
+  if (!y) pari_err_INV("sdivss_rem",gen_0);
   hiremainder = 0; q = divll((ulong)labs(x),(ulong)labs(y));
   if (x < 0) { hiremainder = -((long)hiremainder); q = -q; }
   if (y < 0) q = -q;
@@ -555,6 +614,12 @@ sdivss_rem(long x, long y, long *r)
 }
 INLINE GEN
 divss_rem(long x, long y, long *r) { return stoi(sdivss_rem(x,y,r)); }
+INLINE ulong
+udivuu_rem(ulong x, ulong y, ulong *r)
+{
+  if (!y) pari_err_INV("udivuu_rem",gen_0);
+  *r = x % y; return x / y;
+}
 
 INLINE ulong
 udivui_rem(ulong x, GEN y, ulong *r)
@@ -562,20 +627,46 @@ udivui_rem(ulong x, GEN y, ulong *r)
   long q, s = signe(y);
   LOCAL_HIREMAINDER;
 
-  if (!s) pari_err(gdiver);
+  if (!s) pari_err_INV("udivui_rem",gen_0);
   if (!x || lgefint(y)>3) { *r = x; return 0; }
   hiremainder=0; q = (long)divll(x, (ulong)y[2]);
   if (s < 0) q = -q;
   *r = hiremainder; return q;
 }
 
+/* assume d != 0 and |n| / d can be represented as an ulong.
+ * Return |n|/d, set *r = |n| % d */
+INLINE ulong
+udiviu_rem(GEN n, ulong d, ulong *r)
+{
+  switch(lgefint(n))
+  {
+    case 2: *r = 0; return 0;
+    case 3:
+    {
+      ulong nn = n[2];
+      *r = nn % d; return nn / d;
+    }
+    default: /* 4 */
+    {
+      ulong n1, n0, q;
+      LOCAL_HIREMAINDER;
+      n0 = *int_W(n,0);
+      n1 = *int_W(n,1);
+      hiremainder = n1;
+      q = divll(n0, d);
+      *r = hiremainder; return q;
+    }
+  }
+}
+
 INLINE long
 sdivsi_rem(long x, GEN y, long *r)
 {
   long q, s = signe(y);
   LOCAL_HIREMAINDER;
 
-  if (!s) pari_err(gdiver);
+  if (!s) pari_err_INV("sdivsi_rem",gen_0);
   if (!x || lgefint(y)>3 || ((long)y[2]) < 0) { *r = x; return 0; }
   hiremainder=0; q = (long)divll(labs(x), (ulong)y[2]);
   if (x < 0) { hiremainder = -((long)hiremainder); q = -q; }
@@ -590,7 +681,7 @@ sdivsi(long x, GEN y)
 {
   long q, s = signe(y);
 
-  if (!s) pari_err(gdiver);
+  if (!s) pari_err_INV("sdivsi",gen_0);
   if (!x || lgefint(y)>3 || ((long)y[2]) < 0) return 0;
   q = labs(x) / y[2];
   if (x < 0) q = -q;
@@ -643,7 +734,7 @@ modsi(long x, GEN y) {
 INLINE ulong
 umodui(ulong x, GEN y)
 {
-  if (!signe(y)) pari_err(gdiver);
+  if (!signe(y)) pari_err_INV("umodui",gen_0);
   if (!x || lgefint(y) > 3) return x;
   return x % (ulong)y[2];
 }
@@ -688,7 +779,7 @@ INLINE void
 rdiviiz(GEN x, GEN y, GEN z)
 {
   pari_sp av = avma;
-  long prec = lg(z);
+  long prec = realprec(z);
   affir(x, z);
   if (!is_bigint(y)) {
     affrr(divrs(z, y[2]), z);
@@ -704,8 +795,8 @@ rdivii(GEN x, GEN y, long prec)
   GEN z = cgetr(prec);
   pari_sp av = avma;
   affir(x, z);
-  if (!is_bigint(y)) {
-    affrr(divrs(z, y[2]), z);
+  if (lg(y) == 3) {
+    affrr(divru(z, y[2]), z);
     if (signe(y) < 0) togglesign(z);
   }
   else
@@ -826,6 +917,20 @@ mpdiv(GEN x, GEN y)
 /*                                                                 */
 /*******************************************************************/
 INLINE ulong
+Fl_double(ulong a, ulong p)
+{
+  ulong res = a << 1;
+  return (res >= p || res < a) ? res - p : res;
+}
+INLINE ulong
+Fl_triple(ulong a, ulong p)
+{
+  ulong res = a << 1;
+  if (res >= p || res < a) res -= p;
+  res += a;
+  return (res >= p || res < a)? res - p: res;
+}
+INLINE ulong
 Fl_add(ulong a, ulong b, ulong p)
 {
   ulong res = a + b;
@@ -848,22 +953,20 @@ Fl_center(ulong u, ulong p, ulong ps2) { return (long) (u > ps2)? u - p: u; }
 INLINE ulong
 Fl_mul(ulong a, ulong b, ulong p)
 {
+  register ulong x;
   LOCAL_HIREMAINDER;
-  {
-    register ulong x = mulll(a,b);
-    (void)divll(x,p);
-  }
-  return hiremainder;
+  x = mulll(a,b);
+  if (!hiremainder) return x % p;
+  (void)divll(x,p); return hiremainder;
 }
 INLINE ulong
 Fl_sqr(ulong a, ulong p)
 {
+  register ulong x;
   LOCAL_HIREMAINDER;
-  {
-    register ulong x = mulll(a,a);
-    (void)divll(x,p);
-  }
-  return hiremainder;
+  x = mulll(a,a);
+  if (!hiremainder) return x % p;
+  (void)divll(x,p); return hiremainder;
 }
 INLINE ulong
 Fl_div(ulong a, ulong b, ulong p) { return Fl_mul(a, Fl_inv(b, p), p); }
@@ -878,8 +981,12 @@ addri(GEN x, GEN y) { return addir(y,x); }
 INLINE GEN
 addis(GEN x, long s) { return addsi(s,x); }
 INLINE GEN
+addiu(GEN x, ulong s) { return addui(s,x); }
+INLINE GEN
 addrs(GEN x, long s) { return addsr(s,x); }
 
+INLINE GEN
+subiu(GEN x, long y) { GEN z = subui(y, x); togglesign(z); return z; }
 INLINE GEN
 subis(GEN x, long y) { return addsi(-y,x); }
 INLINE GEN
@@ -931,7 +1038,7 @@ shiftr(GEN x, long n)
   const long e = evalexpo(expo(x)+n);
   const GEN y = rcopy(x);
 
-  if (e & ~EXPOBITS) pari_err(overflower,"expo()");
+  if (e & ~EXPOBITS) pari_err_OVERFLOW("expo()");
   y[1] = (y[1]&~EXPOBITS) | e; return y;
 }
 INLINE GEN
@@ -977,6 +1084,12 @@ INLINE ulong
 shiftlr(ulong x, ulong y)
 { hiremainder = x<<(BITS_IN_LONG-y); return (x>>y); }
 
+INLINE void
+shiftr_inplace(GEN z, long d)
+{
+  setexpo(z, expo(z)+d);
+}
+
 /*******************************************************************/
 /*                                                                 */
 /*                           ASSIGNMENT                            */
@@ -986,7 +1099,7 @@ INLINE void
 affii(GEN x, GEN y)
 {
   long lx = lgefint(x);
-  if (lg(y)<lx) pari_err(overflower,"t_INT-->t_INT assignment");
+  if (lg(y)<lx) pari_err_OVERFLOW("t_INT-->t_INT assignment");
   while (--lx) y[lx] = x[lx];
 }
 INLINE void
@@ -1013,7 +1126,7 @@ affsr(long x, GEN y)
 
   if (!x)
   {
-    y[1] = evalexpo(-bit_accuracy(ly));
+    y[1] = evalexpo(-prec2nbits(ly));
     return;
   }
   if (x < 0) {
@@ -1035,7 +1148,7 @@ affur(ulong x, GEN y)
 
   if (!x)
   {
-    y[1] = evalexpo(-bit_accuracy(ly));
+    y[1] = evalexpo(-prec2nbits(ly));
     return;
   }
   sh = bfffo(x);
