@@ -230,7 +230,7 @@ mulrrz_3end(GEN z, long sz, long ez, ulong garde)
   else
   {
     z[2] = (z[2]<<1) | (garde>>(BITS_IN_LONG-1));
-    if (garde & (HIGHBIT-1))
+    if (garde & (1UL<<(BITS_IN_LONG-2)))
     {
       z[2]++; /* round properly, z2+1 can overflow */
       if (!z[2]) { z[2] = (long)HIGHBIT; ez++; }
