@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -11,7 +9,7 @@ ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 
 /* This file is common to SuperSparc and MicroSparc */
 /*
