@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -24,14 +22,8 @@ BEGINEXTERN
 { GEN z; GEN __E = code; \
   push_lex(gen_0, __E); z = call; pop_lex(1); return z; }
 #define EXPR_ARG __E, &gp_eval
-
-/* to manipulate 'blocs' */
-#define BL_HEAD 4
-#define bl_base(x) (void*)((x) - BL_HEAD)
-#define bl_refc(x) (((GEN)x)[-4])
-#define bl_next(x) (((GEN*)x)[-3])
-#define bl_prev(x) (((GEN*)x)[-2])
-#define bl_num(x)  (((GEN)x)[-1])
+#define EXPR_ARGUPTO __E, &gp_evalupto
+#define EXPR_ARGBOOL __E, &gp_evalbool
 
 /* functions */
 void   changevalue(entree *ep, GEN val);
@@ -41,12 +33,6 @@ void   pari_fill_hashtable(entree **table, entree *ep);
 void compile_err(const char *msg, const char *str);
 void compile_varerr(const char *str);
 
-extern void *foreignHandler;
-extern GEN  (*foreignExprHandler)(char*);
-extern char foreignExprSwitch;
-extern entree * (*foreignAutoload)(const char*, long len);
-extern void (*foreignFuncFree)(entree *);
-
 #ifdef STACK_CHECK
 extern THREAD void *PARI_stack_limit;
 #endif
@@ -58,10 +44,7 @@ struct node_loc
   const char *start,*end;
 };
 
-union token_value {
-  int val;
-  GEN gen;
-};
+union token_value { long val; };
 
 int pari_lex(union token_value *yylval, struct node_loc *yylloc, char **lex);
 int pari_parse(char **lex);
@@ -74,7 +57,7 @@ void pari_init_parser(void);
 void pari_init_compiler(void);
 void pari_init_evaluator(void);
 void optimizenode(long n);
-void push_frame(GEN C, long lpc);
+void push_frame(GEN C, long lpc, long flag);
 const char * closure_func_err(void);
 GEN  gp_closure(long n);
 long eval_mnemonic(GEN str, const char *tmplate);
