@@ -629,7 +629,7 @@ entree oldfonctions[]={
 {"rootsold",0,(void*)suppressed,11,"G","rootsold(x)=THIS FUNCTION HAS BEEN SUPPRESSED."},
 {"round",1,(void*)ground,2,"G","round(x)=take the nearest integer to all the coefficients of x."},
 {"rounderror",10,(void*)rounderror,2,"lG","rounderror(x)=maximum error found in rounding x."},
-{"series",14,(void*)gtoser,2,"Gn","series(x,v)=convert x (usually a vector) into a power series with variable v, starting with the constant coefficient."},
+{"series",14,(void*)gtoser,2,"GnDP","series(x,v)=convert x (usually a vector) into a power series with variable v, starting with the constant coefficient."},
 {"set",1,(void*)gtoset,8,"G","set(x)=convert x into a set, i.e. a row vector with strictly increasing coefficients."},
 {"setintersect",2,(void*)setintersect,8,"GG","setintersect(x,y)=intersection of the sets x and y."},
 {"setminus",2,(void*)setminus,8,"GG","setminus(x,y)=set of elements of x not belonging to y."},
