@@ -1,5 +1,4 @@
-/*
-Copyright (C) 2000-2004  The PARI group.
+/* Copyright (C) 2000-2004  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -32,6 +31,7 @@ typedef enum {OCpushlong='A',OCpushgnil,OCpushgen,OCpushreal,OCpushstoi,OCpushva
               OCgetargs,OCdefaultarg,OClocalvar,OClocalvar0,
               OCcheckargs,OCcheckargs0,OCdefaultgen,OCdefaultlong,
               OCavma,OCgerepile,
-              OCcowvardyn,OCcowvarlex} op_code;
+              OCcowvardyn,OCcowvarlex,
+              OCdup,OCstoreptr,OCcheckuserargs} op_code;
 
 ENDEXTERN
