@@ -1,10 +1,8 @@
+/* A Bison parser, made by GNU Bison 2.5.  */
 
-/* A Bison parser, made by GNU Bison 2.4.1.  */
-
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,37 +39,37 @@
    enum yytokentype {
      KPARROW = 258,
      KARROW = 259,
-     KPE = 260,
-     KSE = 261,
-     KME = 262,
-     KDE = 263,
-     KDRE = 264,
-     KEUCE = 265,
-     KMODE = 266,
-     KAND = 267,
-     KOR = 268,
-     KID = 269,
-     KEQ = 270,
-     KNE = 271,
-     KGE = 272,
-     KLE = 273,
-     KSRE = 274,
-     KSLE = 275,
-     KSR = 276,
-     KSL = 277,
-     KDR = 278,
-     KPP = 279,
-     KSS = 280,
-     KINTEGER = 281,
-     KREAL = 282,
-     KENTRY = 283,
-     KSTRING = 284,
-     DEFFUNC = 285,
-     SEQ = 286,
-     LVAL = 287,
-     INT = 288,
-     SIGN = 289,
-     MAT = 290
+     KDOTDOT = 260,
+     KPE = 261,
+     KSE = 262,
+     KME = 263,
+     KDE = 264,
+     KDRE = 265,
+     KEUCE = 266,
+     KMODE = 267,
+     KAND = 268,
+     KOR = 269,
+     KID = 270,
+     KEQ = 271,
+     KNE = 272,
+     KGE = 273,
+     KLE = 274,
+     KSRE = 275,
+     KSLE = 276,
+     KSR = 277,
+     KSL = 278,
+     KDR = 279,
+     KPP = 280,
+     KSS = 281,
+     KINTEGER = 282,
+     KREAL = 283,
+     KENTRY = 284,
+     KSTRING = 285,
+     DEFFUNC = 286,
+     SEQ = 287,
+     LVAL = 288,
+     INT = 289,
+     SIGN = 290
    };
 #endif
 
