@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2006-2008  The PARI group.
+/* Copyright (C) 2006-2008  The PARI group.
 
 This file is part of the PARI package.
 
@@ -40,10 +38,10 @@ void
 pari_init_parser(void)
 {
   long i;
-  const char *opname[]={"_||_", "_&&_", "_===_", "_==_", "_!=_", "_>=_", "_>_", "_<=_", "_<_", "_-_","_+_","_<<_", "_>>_", "_%_", "_\\/_", "_\\_", "_/_", "_*_","_^_","__","_=_","_--","_++","_-=_", "_+=_", "_<<=_", "_>>=_", "_%=_", "_\\/=_", "_\\=_", "_/=_", "_*=_","+_","-_","!_","_!","_'","_~","%","#_","",""};
+  const char *opname[]={"_||_", "_&&_", "_===_", "_==_", "_!=_", "_>=_", "_>_", "_<=_", "_<_", "_-_","_+_","_<<_", "_>>_", "_%_", "_\\/_", "_\\_", "_/_", "_*_","_^_","__","_--","_++","_-=_", "_+=_", "_<<=_", "_>>=_", "_%=_", "_\\/=_", "_\\=_", "_/=_", "_*=_","+_","-_","!_","_!","_'","_~","[_.._]","[_|_<-_,_]","[_|_<-_,_;_]","%","%#","#_",""};
 
-  stack_init(&s_node,sizeof(*pari_tree),(void **)&pari_tree);
-  stack_alloc(&s_node,OPnboperator);
+  pari_stack_init(&s_node,sizeof(*pari_tree),(void **)&pari_tree);
+  pari_stack_alloc(&s_node,OPnboperator);
   parsestate_reset();
   for (i=0;i<OPnboperator;i++)
   {
@@ -56,7 +54,7 @@ pari_init_parser(void)
   }
 }
 void
-pari_close_parser(void) { stack_delete(&s_node); }
+pari_close_parser(void) { pari_stack_delete(&s_node); }
 
 static void
 unused_chars(const char *lex, int strict)
@@ -72,13 +70,13 @@ unused_chars(const char *lex, int strict)
 void
 compile_err(const char *msg, const char *str)
 {
-  pari_err(syntaxer, msg, str, pari_lex_start);
+  pari_err(e_SYNTAX, msg, str, pari_lex_start);
 }
 
 void
 compile_varerr(const char *str)
 {
-  pari_err(syntaxer, "variable name expected", str, pari_lex_start);
+  pari_err(e_SYNTAX, "variable name expected", str, pari_lex_start);
 }
 
 void
@@ -120,7 +118,10 @@ pari_compile_str(char *lex, int strict)
   struct pari_parsestate state;
   parsestate_save(&state);
   pari_lex_start = lex;
-  pari_discarded = 0;
+  pari_unused_chars=NULL;
+  pari_once=1;
+  pari_discarded=0;
+  pari_lasterror=NULL;
   if (pari_parse(&lex) || pari_discarded)
   {
     if (pari_unused_chars && !pari_discarded)
@@ -140,7 +141,7 @@ pari_compile_str(char *lex, int strict)
 static long
 newnode(Ffunc f, long x, long y, struct node_loc *loc)
 {
-  long n=stack_new(&s_node);
+  long n=pari_stack_new(&s_node);
   pari_tree[n].f=f;
   pari_tree[n].x=x;
   pari_tree[n].y=y;
@@ -165,6 +166,37 @@ newopcall(OPerator op, long x, long y, struct node_loc *loc)
     return newnode(Ffunction,op,newnode(Flistarg,x,y,loc),loc);
 }
 
+static long
+newopcall3(OPerator op, long x, long y, long z, struct node_loc *loc)
+{
+  return newopcall(op,newnode(Flistarg,x,y,loc),z,loc);
+}
+
+static long
+countarg(long n)
+{
+  long i;
+  for(i=1; pari_tree[n].f==Flistarg; i++)
+    n = pari_tree[n].x;
+  return i;
+}
+
+static long
+addcurrexpr(long n, long currexpr, struct node_loc *loc)
+{
+  long y, m = n;
+  while (pari_tree[m].x==OPcomprc)
+  {
+    y = pari_tree[m].y; if (countarg(y)==4) y = pari_tree[y].x;
+    m = pari_tree[y].y;
+  }
+  y = pari_tree[m].y; if (countarg(y)==4) y = pari_tree[y].x;
+  pari_tree[y].y = currexpr;
+  pari_tree[n].str=loc->start;
+  pari_tree[n].len=loc->end-loc->start;
+  return n;
+}
+
 static long
 newintnode(struct node_loc *loc)
 {
@@ -185,7 +217,7 @@ newfunc(CSTtype t, struct node_loc *func, long args, long code,
                    struct node_loc *loc)
 {
   long name=newnode(Fentry,newconst(t,func),-1,func);
-  return newnode(Faffect,name,newnode(Flambda,args,code,loc),loc);
+  return newnode(Fassign,name,newnode(Flambda,args,code,loc),loc);
 }
 
 
