@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000-2003  The PARI group.
+/* Copyright (C) 2000-2003  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -21,12 +19,12 @@ galoisnbpol(long a)
 {
   GEN n;
   pariFILE *F;
-  char *s = stackmalloc(strlen(pari_datadir) + 11 + 20 + 1);
+  char *s = stack_malloc(strlen(pari_datadir) + 11 + 20 + 1);
   sprintf(s,"%s/galpol/%ld/nb", pari_datadir, a);
   F = pari_fopengz(s);
-  if (!F) pari_err(talker,"Missing galpol file %s\n",s);
+  if (!F) pari_err_FILE("galpol file",s);
   n = gp_read_stream(F->file);
-  if (!n || typ(n)!=t_INT) pari_err(talker,"Incompatible galpol file %s\n",s);
+  if (!n || typ(n)!=t_INT) pari_err_FILE("galpol file [incompatible]",s);
   pari_fclose(F); return n;
 }
 
@@ -37,26 +35,29 @@ galoisgetpol(long a, long b, long sig)
   GEN V;
   const char *si;
   char *s;
-  if (a<=0 || b<0) pari_err(talker,"argument must be positive");
+  if (a<=0) pari_err_DOMAIN("galoisgetpol", "degree", "<=", gen_0, stoi(a));
+  if (b<0) pari_err_DOMAIN("galoisgetpol", "index", "<", gen_0, stoi(b));
   if (!b) return galoisnbpol(a);
   switch(sig)
   {
     case 1: si="real"; break;
     case 2: if (a%2==0) { si="complex"; break; }
-    default: /*FALL THROUGH*/
-      pari_err(talker,"invalid signature in galoisgetpol"); return NULL;
+      pari_err_DOMAIN("galoisgetpol", "s", ">", gen_1, stoi(sig));
+    default:
+      pari_err_FLAG("galoisgetpol");
+      return NULL;
   }
   s = pari_sprintf("%s/galpol/%ld/%ld/%s", pari_datadir, a,b,si);
-  F = pari_fopengz(s); free(s);
+  F = pari_fopengz(s); pari_free(s);
   if (!F)
   {
     long n = itos(galoisnbpol(a));
-    if (b>n)
-      pari_err(talker,"Only %ld group%s of order %ld",n,n>2?"s":"",a);
-    else pari_err(talker,"Missing galpol file");
+    if (b > n)
+      pari_err_DOMAIN("galoisgetpol", "group index", ">", stoi(n), stoi(b));
+    else pari_err_FILE("galpol file", F->name);
   }
   V = gp_read_stream(F->file);
-  if (!V || typ(V)!=t_VEC ) pari_err(talker,"Incompatible galpol file\n");
+  if (!V || typ(V)!=t_VEC) pari_err_FILE("galpol file", F->name);
   pari_fclose(F); return V;
 }
 
