@@ -279,7 +279,7 @@ factmz(GEN Q, GEN p, long *maxord)
   }
   if (m >= 3 && degpol(z) == 2)
   {
-    GEN F = factmod(z,p);
+    GEN F = FpX_factor(z,p);
     z = gcoeff(F,1,1);
   }
   *maxord = m; return z;
@@ -467,6 +467,16 @@ end:
   return polf;
 }
 
+/* a in Q[i], return a^3 mod 3 */
+static GEN
+zi_pow3mod(GEN a)
+{
+  GEN x, y;
+  if (typ(a) != t_COMPLEX) return gmodgs(a,3);
+  x = gmodgs(gel(a,1), 3);
+  y = gmodgs(gel(a,2), 3);
+  return mkcomplex(x, negi(y));
+}
 static GEN
 polymini_zi(GEN pol) /* polynome minimal dans Z[i] */
 {
@@ -494,7 +504,7 @@ polymini_zi(GEN pol) /* polynome minimal dans Z[i] */
     }
     RgX_to_6(polh, &a0,&a1,&a2,&a3,&a4,&a5,&a6);
     if (!gequal0(theta) || !myval_zi(a4) || !myval_zi(a5)) break;
-    rac = Fp_neg(Fp_powu(Fp_div(a6, a3, p), 3, p), p);
+    rac = zi_pow3mod(gdiv(a6, gneg(a3)));
   }
   if (alpha && myval_zi(a0) >= 3 && myval_zi(a1) >= 2 && myval_zi(a2) >= 1)
   {
@@ -505,23 +515,13 @@ polymini_zi(GEN pol) /* polynome minimal dans Z[i] */
   return mkvec3(theta, stoi(alpha), stoi(beta));
 }
 
-/* u in Z[i,Y]/(Y^2-3), compute u mod Y */
-static GEN
-zi2mod(GEN u)
-{
-  GEN a,b,unmodp = mkintmod(gen_1, stoi(3));
-  u = truecoeff(lift(u),0);
-  a = gmul(unmodp,real_i(u));
-  b = gmul(unmodp,imag_i(u));
-  return gadd(a,gmul(b,gen_I()));
-}
 /* pol is a ZX, minimal polynomial over Z_3[i,Y]/(Y^2-3) */
 static GEN
 polymini_zi2(GEN pol)
 {
   long alpha, beta, v = MAXVARN;
   GEN a0, a1, a2, a3, a4, a5, a6;
-  GEN p, polh, rac, theta, b3, b6, y = pol_x(v);
+  GEN p, polh, rac, theta, y = pol_x(v);
 
   p = stoi(3);
   if (polval(pol,p)) pari_err_BUG("polymini_zi2 [polynomial not minimal]");
@@ -552,9 +552,9 @@ polymini_zi2(GEN pol)
     }
     RgX_to_6(polh, &a0,&a1,&a2,&a3,&a4,&a5,&a6);
     if (!gequal0(theta) || !myval_zi2(a4) || !myval_zi2(a5)) break;
-    b3 = zi2mod(a3);
-    b6 = zi2mod(a6);
-    rac = lift(gpowgs(gdiv(b6,gneg(b3)), 3));
+    a3 = liftpol_shallow(a3); if (typ(a3)==t_POL) a3 = constant_term(a3);
+    a6 = liftpol_shallow(a6); if (typ(a6)==t_POL) a6 = constant_term(a6);
+    rac = zi_pow3mod(gdiv(a6,gneg(a3)));
   }
   if (alpha)
   {
@@ -2115,8 +2115,23 @@ genus2localred(struct igusa *I, struct igusa_p *Ip, GEN p, GEN polmini)
         GEN c = gel(list,i);
         if (!valp(gel(c,2))) prod = RgX_mul(prod,c);
       }
-      if (degpol(prod) > 2) pari_err_BUG("genus2localred [padicfactors 2]");
-      dism = valp(RgX_disc(prod));
+      switch(degpol(prod))
+      {
+        GEN e0, e1, e2;
+        case 0:
+          dism = 0; break;
+        case 1:
+          e1 = gel(prod,3);
+          dism = 2*valp(e1); break;
+        case 2:
+          e0 = gel(prod,2);
+          e1 = gel(prod,3);
+          e2 = gel(prod,4);
+          dism = valp(gsub(gsqr(e1),gmul2n(gmul(e0,e2),2))); break;
+        default:
+          pari_err_BUG("genus2localred [padicfactors 2]");
+          dism = 0;
+      }
       switch(itos(gmulgs(theta,12))+alpha-4)
       {
         case 0:
@@ -2157,6 +2172,17 @@ genus2localred(struct igusa *I, struct igusa_p *Ip, GEN p, GEN polmini)
   return condp;
 }
 
+static long
+chk_pol(GEN P) {
+  switch(typ(P))
+  {
+    case t_INT: break;
+    case t_POL: RgX_check_ZX(P,"genus2red"); return varn(P); break;
+    default: pari_err_TYPE("genus2red", P);
+  }
+  return -1;
+}
+
 /* P,Q are ZX, study Y^2 + Q(X) Y = P(X) */
 GEN
 genus2red(GEN Q, GEN P, GEN p)
@@ -2164,17 +2190,16 @@ genus2red(GEN Q, GEN P, GEN p)
   pari_sp av = avma;
   struct igusa I;
   GEN j22, j42, j2j6, a0,a1,a2,a3,a4,a5,a6, V,polr,facto,factp, vecmini, cond;
-  long i, l, dd;
+  long i, l, dd, vP,vQ;
 
-  if (typ(P) != t_POL) pari_err_TYPE("genus2red", P);
-  RgX_check_ZX(P, "genus2red");
-  switch (typ(Q))
+  vP = chk_pol(P);
+  vQ = chk_pol(Q);
+  if (vP < 0)
   {
-    case t_INT: Q = scalarpol(Q, varn(P)); break;
-    case t_POL:
-      RgX_check_ZX(Q, "genus2red");
-      if (varn(Q) != varn(P)) pari_err_VAR("genus2red",P,Q);
+    if (vQ < 0) pari_err_TYPE("genus2red",mkvec2(P,Q));
+    P = scalarpol(P,vQ);
   }
+  else if (vQ < 0) Q = scalarpol(Q,vP);
   if (p && typ(p) != t_INT) pari_err_TYPE("genus2red", p);
 
   polr = ZX_add(ZX_sqr(Q), gmul2n(P,2)); /* ZX */
