@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000-2004  The PARI group.
+/* Copyright (C) 2000-2004  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -87,9 +85,9 @@ calc_block(blockdata *B, GEN Z, GEN Y, GEN SB)
 
   Zp   = cgetg(lnon,t_VEC);
   Zpp  = cgetg(lnon,t_VEC); nn = 0;
-  for (i=1; i<r; i++) { n[i] = lg(Z[i])-1; nn += n[i]; }
+  for (i=1; i<r; i++) { n[i] = lg(gel(Z,i))-1; nn += n[i]; }
   lY = lg(Y); Yp = cgetg(lY+1,t_VEC);
-  for (j=1; j<lY; j++) Yp[j] = Y[j];
+  for (j=1; j<lY; j++) gel(Yp,j) = gel(Y,j);
 
   {
     pari_sp av = avma;
@@ -105,14 +103,14 @@ calc_block(blockdata *B, GEN Z, GEN Y, GEN SB)
   }
   gel(Yp,lY) = Zp;
 
-  K = divisors(utoipos(n[1])); lK = lg(K);
+  K = divisorsu(n[1]); lK = lg(K);
   for (i=1; i<lK; i++)
   {
-    long ngcd = n[1], k = itos(gel(K,i)), dk = B->size*k, lpn = 0;
+    long ngcd = n[1], k = K[i], dk = B->size*k, lpn = 0;
     for (j=2; j<r; j++)
       if (n[j]%k == 0)
       {
-        if (++lpn >= BIL) pari_err(talker,"overflow in calc_block");
+        if (++lpn >= BIL) pari_err_OVERFLOW("calc_block");
         pn[lpn] = n[j]; pnon[lpn] = j;
         ngcd = ugcd(ngcd, n[j]);
       }
@@ -204,7 +202,7 @@ append(GEN D, GEN a)
 {
   long i,l = lg(D), m = lg(a);
   GEN x = D + (l-1);
-  for (i=1; i<m; i++) x[i] = a[i];
+  for (i=1; i<m; i++) gel(x,i) = gel(a,i);
   setlg(D, l+m-1);
 }
 
@@ -226,7 +224,7 @@ print_block_system(blockdata *B, GEN Y, GEN SB)
     GEN Yi = gel(Y,i);
     long ki = 0, si = lg(Yi)-1;
 
-    for (j=1; j<=si; j++) { n[j] = lg(Yi[j])-1; ki += n[j]; }
+    for (j=1; j<=si; j++) { n[j] = lg(gel(Yi,j))-1; ki += n[j]; }
     ki /= B->size;
     De = cgetg(ki+1,t_VEC);
     for (j=1; j<=ki; j++) gel(De,j) = VOID;
@@ -310,7 +308,7 @@ polsimplify(GEN x)
 {
   long i,lx = lg(x);
   for (i=2; i<lx; i++)
-    if (typ(x[i]) == t_POL) gel(x,i) = constant_term(gel(x,i));
+    if (typ(gel(x,i)) == t_POL) gel(x,i) = constant_term(gel(x,i));
   return x;
 }
 
@@ -386,7 +384,7 @@ embedding(GEN g, GEN DATA, primedata *S, GEN den, GEN listdelta)
   long rt;
   pari_sp av;
 
-  T   = gel(DATA,1); rt = brent_kung_optpow(degpol(T), 2);
+  T   = gel(DATA,1); rt = brent_kung_optpow(degpol(T), 4, 3);
   maxp= gel(DATA,7);
   gp = RgX_deriv(g); av = avma;
   w0 = chinese_retrieve_pol(DATA, S, listdelta);
@@ -404,10 +402,10 @@ embedding(GEN g, GEN DATA, primedata *S, GEN den, GEN listdelta)
     if (wpow) a = FpX_FpXQV_eval(g,wpow, T,q);
     else      a = FpX_FpXQ_eval(g,w0, T,q); /* first time */
     /* now, a = 0 (p) */
-    a = gmul(gneg(h0), ZX_Z_divexact(a, p));
-    w1 = gadd(w0, gmul(p, FpXQ_red(a, T,p)));
+    a = FpXQ_mul(ZX_neg(h0), ZX_Z_divexact(a, p), T,p);
+    w1 = ZX_add(w0, ZX_Z_mul(a, p));
 
-    w1_Q = centermod(gmul(w1, remii(den,q)), q);
+    w1_Q = centermod(ZX_Z_mul(w1, remii(den,q)), q);
     if (ZX_equal(w1_Q, w0_Q))
     {
       GEN G = is_pm1(den)? g: RgX_rescale(g,den);
@@ -425,10 +423,10 @@ embedding(GEN g, GEN DATA, primedata *S, GEN den, GEN listdelta)
     wpow = FpXQ_powers(w1, rt, T, q2);
     /* h0 := h0 * (2 - h0 g'(w1)) mod (T,q)
      *     = h0 + h0 * (1 - h0 g'(w1)) */
-    a = gmul(gneg(h0), FpX_FpXQV_eval(gp, FpXV_red(wpow,q),T,q));
-    a = ZX_Z_add(FpX_rem(a, T,q), gen_1); /* 1 - h0 g'(w1) = 0 (p) */
-    a = gmul(h0, ZX_Z_divexact(a, p));
-    h0 = gadd(h0, gmul(p, FpXQ_red(a, T,p)));
+    a = FpXQ_mul(ZX_neg(h0), FpX_FpXQV_eval(gp, FpXV_red(wpow,q),T,q), T,q);
+    a = ZX_Z_add_shallow(a, gen_1); /* 1 - h0 g'(w1) = 0 (p) */
+    a = FpXQ_mul(h0, ZX_Z_divexact(a, p), T,p);
+    h0 = ZX_add(h0, ZX_Z_mul(a, p));
     w0 = w1; w0_Q = w1_Q; p = q; q = q2;
   }
   TR = gel(DATA,5);
@@ -448,7 +446,7 @@ get_bezout(GEN pol, GEN fk, GEN p)
     A = gel(fk,i);
     B = FpX_div(pol, A, p);
     d = FpX_extgcd(A,B,p, &u, &v);
-    if (degpol(d) > 0) pari_err(talker, "relatively prime polynomials expected");
+    if (degpol(d) > 0) pari_err_COPRIME("get_bezout",A,B);
     d = constant_term(d);
     if (!gequal1(d)) v = FpX_Fp_mul(v, Fp_inv(d, p), p);
     gel(U,i) = FpX_mul(B,v, p);
@@ -488,24 +486,6 @@ init_traces(GEN ff, GEN T, GEN p)
   return y;
 }
 
-/* return C in Z[X]/(p,T), C[ D[1] ] = 1, C[ D[i] ] = 0 otherwise. H is the
- * list of degree 1 polynomials X - D[i]  (come directly from factorization) */
-static GEN
-interpol(GEN H, GEN T, GEN p)
-{
-  long i, m = lg(H);
-  GEN X = pol_x(0),d,p1,p2,a;
-
-  p1=pol_1(0); p2=gen_1; a = gneg(constant_term(gel(H,1))); /* = D[1] */
-  for (i=2; i<m; i++)
-  {
-    d = constant_term(gel(H,i)); /* -D[i] */
-    p1 = FpXQX_mul(p1,gadd(X,d), T,p);
-    p2 = Fq_mul(p2, gadd(a,d), T,p);
-  }
-  return FqX_Fq_mul(p1,Fq_inv(p2, T,p), T,p);
-}
-
 static void
 init_primedata(primedata *S)
 {
@@ -526,10 +506,13 @@ init_primedata(primedata *S)
   S->fk = cgetg(N+1, t_VEC);
   for (l=1,j=1; j<lff; j++)
   { /* compute roots and fix ordering (Frobenius cycles) */
-    GEN F = FpX_factorff_irred(gel(S->ff,j), T, p);
-    gel(S->interp,j) = interpol(F,T,p);
+    GEN F = gel(S->ff, j), deg1 = FpX_factorff_irred(F, T,p);
+    GEN H = gel(deg1,1), a = Fq_neg(constant_term(H), T,p);
+    GEN Q = FqX_div(F, H, T,p);
+    GEN q = Fq_inv(FqX_eval(Q, a, T,p), T,p);
+    gel(S->interp,j) = FqX_Fq_mul(Q, q, T,p); /* = 1 at a, 0 at other roots */
     S->firstroot[j] = l;
-    for (i=1; i<lg(F); i++,l++) S->fk[l] = F[i];
+    for (i=1; i<lg(deg1); i++,l++) gel(S->fk, l) = gel(deg1, i);
   }
   S->Trk     = init_traces(S->ff, T,p);
   S->bezoutC = get_bezout(S->pol, S->ff, p);
@@ -538,21 +521,21 @@ init_primedata(primedata *S)
 static void
 choose_prime(primedata *S, GEN pol, GEN dpol)
 {
-  byteptr di = diffptr + 1;
-  long i, j, k, r, lcm, oldlcm, pp, N = degpol(pol), minp = N*N / 4;
-  GEN Z, p, ff, oldff, n, oldn;
+  long i, j, k, r, lcm, oldlcm, N = degpol(pol);
+  ulong p, pp;
+  GEN Z, ff, oldff, n, oldn;
   pari_sp av;
+  forprime_t T;
 
-  p = utoipos(2);
-  while (p[2] <= minp) NEXT_PRIME_VIADIFF(p[2], di);
+  u_forprime_init(&T, (N*N) >> 2, ULONG_MAX);
   oldlcm = 0;
   oldff = oldn = NULL; pp = 0; /* gcc -Wall */
   av = avma;
   for(k = 1; k < 11 || !oldlcm; k++,avma = av)
   {
-    do NEXT_PRIME_VIADIFF(p[2], di); while (!smodis(dpol, p[2]));
-    if (k > 5 * N) pari_err(talker,"sorry, too many block systems in nfsubfields");
-    ff = gel(FpX_factor(pol, p), 1);
+    if (k > 5 * N) pari_err_OVERFLOW("choose_prime [too many block systems]");
+    do p = u_forprime_next(&T); while (!umodiu(dpol, p));
+    ff = gel(Flx_factor(ZX_to_Flx(pol,p), p), 1);
     r = lg(ff)-1;
     if (r == N || r >= BIL) continue;
 
@@ -560,14 +543,15 @@ choose_prime(primedata *S, GEN pol, GEN dpol)
     for (j=2; j<=r; j++) { n[j] = degpol(gel(ff,j)); lcm = clcm(lcm, n[j]); }
     if (lcm <= oldlcm) continue; /* false when oldlcm = 0 */
 
-    if (DEBUGLEVEL) err_printf("p = %ld,\tlcm = %ld,\torbits: %Ps\n",p[2],lcm,n);
-    pp = p[2];
+    if (DEBUGLEVEL) err_printf("p = %ld,\tlcm = %ld,\torbits: %Ps\n",p,lcm,n);
+    pp = p;
     oldn = n;
     oldff = ff;
     oldlcm = lcm; if (r == 1) break;
     av = avma;
   }
   if (DEBUGLEVEL) err_printf("Chosen prime: p = %ld\n", pp);
+  FlxV_to_ZXV_inplace(oldff);
   S->ff = oldff;
   S->lcm= oldlcm;
   S->p  = utoipos(pp);
@@ -590,7 +574,7 @@ bound_for_coeff(long m, GEN rr, GEN *maxroot)
   GEN p1,b1,b2,B,M, C = matpascal(m-1);
 
   for (r1=1; r1 < lrr; r1++)
-    if (typ(rr[r1]) != t_REAL) break;
+    if (typ(gel(rr,r1)) != t_REAL) break;
   r1--;
 
   rr = gabs(rr,0); *maxroot = vecmax(rr);
@@ -833,7 +817,7 @@ subfields_poldata(GEN T, poldata *PD)
   }
   else
   {
-    PD->den = initgaloisborne(T,NULL,ZX_max_lg(T), &L,NULL,&dis);
+    PD->den = initgaloisborne(T,NULL,nbits2prec(bit_accuracy(ZX_max_lg(T))), &L,NULL,&dis);
     PD->roo = L;
     PD->dis = absi(dis);
   }
@@ -869,10 +853,9 @@ subfieldsall(GEN nf)
   pol = PD.pol;
 
   v0 = varn(pol); N = degpol(pol);
-  dg = divisors(utoipos(N)); ld = lg(dg)-1;
+  dg = divisorsu(N); ld = lg(dg)-1;
   if (DEBUGLEVEL) err_printf("\n***** Entering subfields\n\npol = %Ps\n",pol);
 
-
   LSB = _subfield(pol_x(0), gen_0);
   if (ld > 2)
   {
@@ -882,7 +865,7 @@ subfieldsall(GEN nf)
     choose_prime(&S, PD.pol, PD.dis);
     for (i=ld-1; i>1; i--)
     {
-      B.size  = itos(gel(dg,i));
+      B.size  = dg[i];
       B.d = N / B.size;
       NLSB = subfields_of_given_degree(&B);
       if (NLSB) { LSB = concat(LSB, NLSB); gunclone(NLSB); }
