@@ -368,8 +368,10 @@ LLL_1stPass(GEN *pB0, GEN kappa, baker_s *BS, GEN *pBx)
   else
     triv = addir(sqri(BS->Ind), sqrr(B0));
 
-  gcoeff(lllmat, 3, 1) = roundr(negr(mulir(C, BS->lambda)));
-  gcoeff(lllmat, 3, 2) = roundr(negr(mulir(C, BS->delta)));
+  gcoeff(lllmat, 3, 1) = grndtoi(negr(mulir(C, BS->lambda)), &e);
+  if (e >= 0) return -1;
+  gcoeff(lllmat, 3, 2) = grndtoi(negr(mulir(C, BS->delta)), &e);
+  if (e >= 0) return -1;
   gcoeff(lllmat, 3, 3) = C;
   lllmat = ZM_lll(lllmat, 0.99, LLL_IM|LLL_INPLACE);
 
@@ -513,7 +515,17 @@ MiddleSols(GEN *pS, GEN bound, GEN roo, GEN poly, GEN rhs, long s, GEN c1)
         add_pm(pS, p, q, z, d, rhs);
       }
     }
-    if (j == lg(t)) pari_err_BUG("Short continued fraction in thue");
+    if (j == lg(t))
+    {
+      long prec;
+      if (j > nmax) pari_err_BUG("thue [short continued fraction]");
+      /* the theoretical value is bit_prec = gexpo(ro)+1+log2(bound) */
+      prec = precdbl(precision(real_i(gel(roo,k))));
+      if (DEBUGLEVEL>1) pari_warn(warnprec,"thue",prec);
+      roo = vec_shorten(cleanroots(poly, prec), s);
+      k--;
+    }
+
   }
   return bndcf;
 }
@@ -800,6 +812,7 @@ get_Bx_LLL(long i1, GEN Delta, GEN Lambda, GEN eps5, long prec, baker_s *BS)
       for (cf = 0; cf < cfMAX; cf++, kappa = muliu(kappa,10))
       {
         int res = LLL_1stPass(&B0, kappa, BS, &Bx);
+        if (res < 0) return NULL;
         if (res) break;
         if (DEBUGLEVEL>1) err_printf("LLL failed. Increasing kappa\n");
       }
@@ -1001,9 +1014,10 @@ LargeSols(GEN P, GEN tnf, GEN rhs, GEN ne, GEN *pS)
 
 PRECPB:
   ne = gerepilecopy(av, ne);
-  prec += 5 * (DEFAULTPREC-2);
+  prec += nbits2extraprec(5 * prec2nbits(DEFAULTPREC));
   if (DEBUGLEVEL>1) pari_warn(warnprec,"thue",prec);
   tnf = inithue(P, bnf, 0, prec);
+  *pS = cgetg(1, t_VEC); /* reset */
   return LargeSols(P, tnf, rhs, ne, pS);
 }
 
@@ -1097,12 +1111,10 @@ thue(GEN tnf, GEN rhs, GEN ne)
       ry = nfrootsQ(Rab);
       for (k = 1; k < lg(ry); k++)
         if (typ(gel(ry,k)) == t_INT) check_y(&S, P, POL, gel(ry,k), rhs);
-      if (odd(e)) {
-        Rab = gsubst(gsubst(R, va, negi(df)), vb, negi(dg));
-        ry = nfrootsQ(Rab);
-        for (k = 1; k < lg(ry); k++)
-          if (typ(gel(ry,k)) == t_INT) check_y(&S, P, POL, gel(ry,k), rhs);
-      }
+      Rab = gsubst(gsubst(R, va, negi(df)), vb, odd(e)? negi(dg): dg);
+      ry = nfrootsQ(Rab);
+      for (k = 1; k < lg(ry); k++)
+        if (typ(gel(ry,k)) == t_INT) check_y(&S, P, POL, gel(ry,k), rhs);
     }
   }
   return gerepilecopy(av, filter_sol_x(S, L));
