@@ -1,3 +1,5 @@
+void nchar2nlong(){}
+void newblock(){}
 void remsBIL(){}
 void bit_accuracy(){}
 void is_bigint(){}
@@ -12,5 +14,10 @@ void gerepilemanysp(){}
 void ceilr(){}
 void roundr(){}
 void pari_err(){}
+void pari_err_BUG(){}
+void pari_err_INV(){}
+void pari_err_OVERFLOW(){}
+void pari_err_PREC(){}
 void pari_warn(){}
 void quadratic_prec_mask(){}
+void prec2nbits(){}
