@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2001  The PARI group.
+/* Copyright (C) 2001  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -86,7 +84,7 @@ rand_INT(long n)
   pari_sp av = avma;
   GEN x, N = int2n(n*BITS_IN_LONG);
   do x = randomi(N); while (lgefint(x) != n+2);
-  if (!mpodd(x)) x = addis(x,1); /*For Montgomery REDC */
+  if (!mpodd(x)) x = addis(x,1); /*For Barrett REDC */
   return gerepileuptoint(av, x);
 }
 /* real, n words */
@@ -234,19 +232,26 @@ static double speed_Flx_sqr(speed_param *s)
 { TIME_FUN(Flx_sqr(s->x, s->l)); }
 
 static double speed_Flx_inv(speed_param *s)
-{ TIME_FUN(Flx_invMontgomery(s->x, s->l)); }
+{ TIME_FUN(Flx_invBarrett(s->x, s->l)); }
 
 static double speed_Flx_mul(speed_param *s)
 { TIME_FUN(Flx_mul(s->x, s->y, s->l)); }
 
+static double speed_Flx_divrem(speed_param *s)
+{
+  GEN r, x = rand_NFlx((degpol(s->x)-1)*2, s->l);
+  TIME_FUN(Flx_divrem(x, s->x, s->l, &r));
+}
+
 static double speed_Flx_rem(speed_param *s) {
   GEN x = rand_NFlx((degpol(s->x)-1)*2, s->l);
   TIME_FUN(Flx_rem(x, s->x, s->l));
 }
 
-static double speed_Flxq_pow(speed_param *s) {
-  GEN x = rand_Flx(degpol(s->x)-1, s->l);
-  TIME_FUN( Flxq_pow(x, utoipos(s->l), s->x, s->l) );
+static double speed_Flxq_red(speed_param *s) {
+  GEN x = rand_NFlx((degpol(s->x)-1)*2, s->l);
+  GEN q = Flx_get_red(s->x, s->l);
+  TIME_FUN(Flx_rem(x, q, s->l));
 }
 
 static double speed_Flx_halfgcd(speed_param *s)
@@ -259,7 +264,13 @@ static double speed_Flx_extgcd(speed_param *s)
 { GEN u,v; TIME_FUN(Flx_extgcd(s->x, s->y, s->l, &u, &v)); }
 
 static double speed_FpX_inv(speed_param *s)
-{ TIME_FUN(FpX_invMontgomery(s->x, s->p)); }
+{ TIME_FUN(FpX_invBarrett(s->x, s->p)); }
+
+static double speed_FpX_divrem(speed_param *s)
+{
+  GEN r, x = rand_NFpX((degpol(s->x)-1)*2);
+  TIME_FUN(FpX_divrem(x, s->x, s->p, &r));
+}
 
 static double speed_FpX_rem(speed_param *s)
 {
@@ -267,9 +278,10 @@ static double speed_FpX_rem(speed_param *s)
   TIME_FUN(FpX_rem(x, s->x, s->p));
 }
 
-static double speed_FpXQ_pow(speed_param *s) {
-  GEN x = rand_NFpX(degpol(s->x)-1);
-  TIME_FUN( FpXQ_pow(x, s->p, s->x, s->p) );
+static double speed_FpXQ_red(speed_param *s) {
+  GEN x = rand_NFpX((degpol(s->x)-1)*2);
+  GEN q = FpX_get_red(s->x, s->p);
+  TIME_FUN(FpX_rem(x, q, s->p));
 }
 
 static double speed_FpX_halfgcd(speed_param *s)
@@ -318,15 +330,17 @@ static tune_param param[] = {
 {0,   var(Flx_SQR_SQRI_LIMIT),     t_Fl1x,5,0, speed_Flx_sqr},
 {0,   var(Flx_MUL_MULII2_LIMIT),   t_Fl2x,5,20000, speed_Flx_mul,0.05},
 {0,   var(Flx_SQR_SQRI2_LIMIT),    t_Fl2x,5,20000, speed_Flx_sqr,0.05},
-{0,   var(Flx_INVMONTGOMERY_LIMIT),t_NFlx,10,0, speed_Flx_inv,0.05},
-{0,  var(Flx_REM_MONTGOMERY_LIMIT),t_NFlx,10,0, speed_Flx_rem,0.05},
-{0,  var(Flx_POW_MONTGOMERY_LIMIT),t_NFlx,10,0, speed_Flxq_pow},
+{0,   var(Flx_INVBARRETT_LIMIT),  t_NFlx,10,0, speed_Flx_inv,0.05},
+{0,  var(Flx_DIVREM_BARRETT_LIMIT),t_NFlx,10,0, speed_Flx_divrem,0.05},
+{0,  var(Flx_REM_BARRETT_LIMIT),  t_NFlx,10,0, speed_Flx_rem,0.05},
+{0,  var(Flx_BARRETT_LIMIT),      t_NFlx,10,0, speed_Flxq_red},
 {0,  var(Flx_HALFGCD_LIMIT),       t_Flx,10,0, speed_Flx_halfgcd},
 {0,  var(Flx_GCD_LIMIT),           t_Flx,10,0, speed_Flx_gcd,0.1},
 {0,  var(Flx_EXTGCD_LIMIT),        t_Flx,10,0, speed_Flx_extgcd},
-{0,  var(FpX_INVMONTGOMERY_LIMIT), t_NFpX,10,0, speed_FpX_inv,0.05},
-{0,  var(FpX_REM_MONTGOMERY_LIMIT),t_NFpX,10,0, speed_FpX_rem,0.05},
-{0,  var(FpX_POW_MONTGOMERY_LIMIT),t_NFpX,10,0, speed_FpXQ_pow},
+{0,  var(FpX_INVBARRETT_LIMIT),   t_NFpX,10,0, speed_FpX_inv,0.05},
+{0,  var(FpX_DIVREM_BARRETT_LIMIT),t_NFpX,10,0, speed_FpX_divrem,0.05},
+{0,  var(FpX_REM_BARRETT_LIMIT),  t_NFpX,10,0, speed_FpX_rem,0.05},
+{0,  var(FpX_BARRETT_LIMIT),      t_NFpX,10,0, speed_FpXQ_red},
 {0,  var(FpX_HALFGCD_LIMIT),       t_FpX,10,0, speed_FpX_halfgcd},
 {0,  var(FpX_GCD_LIMIT),           t_FpX,10,0, speed_FpX_gcd,0.1},
 {0,  var(FpX_EXTGCD_LIMIT),        t_FpX,10,0, speed_FpX_extgcd},
@@ -368,7 +382,7 @@ time_fun(speed_function_t fun, speed_param *s, long enabled)
                      * speed_unittime * speed_precision
                      / maxdd(t[i], speed_unittime));
       if (reps_d > 2e9 || reps_d < 1.0)
-        pari_err(talker, "Fatal error: new reps bad: %.2f\n", reps_d);
+        pari_err(e_MISC, "Fatal error: new reps bad: %.2f", reps_d);
 
       s->reps = (ulong)reps_d;
     }
@@ -386,7 +400,7 @@ time_fun(speed_function_t fun, speed_param *s, long enabled)
         if (t[j] <= t[j-e+1] * TOLERANCE) { avma = av; return t[j-e+1]; }
     }
   }
-  pari_err(talker,"couldn't measure time");
+  pari_err(e_MISC,"couldn't measure time");
   return -1.0; /* not reached */
 }
 
@@ -411,7 +425,7 @@ diag(const char *format, ...)
 }
 void
 print_define(const char *name, long value)
-{ printf("#define __%-25s  %5ld\n\n", name, value); }
+{ printf("#define __%-25s  %5ld\n", name, value); }
 
 long
 analyze_dat(int final)
@@ -455,6 +469,7 @@ Test(tune_param *param)
   int since_positive, since_change, thresh, new_thresh;
   speed_param s;
   long save_var_disable = -1;
+  pari_timer T;
 
   if (param->kernel == AVOID) { print_define(param->name, -1); return; }
 
@@ -471,7 +486,10 @@ Test(tune_param *param)
   dftmod(&s);
   ndat = since_positive = since_change = thresh = 0;
   if (option_trace >= 1)
-    diag("Setting %s... (default %ld)\n", param->name, *(param->var));
+  {
+    timer_start(&T);
+    diag("\nSetting %s... (default %ld)\n", param->name, *(param->var));
+  }
   if (option_trace >= 2)
   {
     diag("              algorithm-A  algorithm-B   ratio  possible\n");
@@ -535,6 +553,8 @@ Test(tune_param *param)
     }
   }
   thresh = dat[analyze_dat(1)].size;
+  if (option_trace >= 1)
+    diag("Total time: %gs\n", (double)timer_delay(&T)/1000.);
   print_define(param->name, thresh);
   *(param->var) = thresh; /* set to optimal value for next tests */
   if (param->var_disable) *(param->var_disable) = save_var_disable;
@@ -542,8 +562,15 @@ Test(tune_param *param)
 
 void error(char **argv) {
   long i;
-  diag("usage: tune [-t] [-s step_factor] [-p mod] [-u unittime] var1 var2 ...\n");
-  diag("Tunable variables: (omitting variable indices tunes everybody)\n");
+  diag("This is the PARI/GP tuning utility. Usage: tune [OPTION] var1 var2...\n");
+  diag("Options:\n");
+  diag("  -t:     verbose output\n");
+  diag("  -tt:    very verbose output\n");
+  diag("  -ttt:   output everything\n");
+  diag("  -s xxx: set step factor between successive sizes to xxx (default 0.01)\n");
+  diag("  -p xxx: set Flx modulus to xxx (default 27449)\n");
+  diag("  -u xxx: set speed_unittime to xxx (default 1e-4s)\n");
+  diag("Tunable variables (omitting variable indices tunes everybody):\n");
   for (i = 0; i < (long)numberof(param); i++)
     diag("  %2ld: %-25s (default %4ld)\n", i, param[i].name, *(param[i].var));
   exit(1);
@@ -555,7 +582,7 @@ main(int argc, char **argv)
   int i, r, n = 0;
   GEN v;
   pari_init(8000000, 2);
-  (void) init_modular(&DFLT_mod);
+  DFLT_mod = 27449;
   LARGE_mod=subis(powuu(3,128),62);
 #ifdef LONG_IS_64BIT
   DFLT_hmod = 257;
@@ -570,7 +597,10 @@ main(int argc, char **argv)
     char *s = argv[i];
     if (*s == '-') {
       switch(*++s) {
-        case 't': option_trace += 2; break;
+        case 't': option_trace++;
+          while (*++s == 't') option_trace++;
+          break;
+
         case 'p':
           if (!*++s)
           {
